function alphashape(src,ev)
%Alpha Shape
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');


radiustext=text(.275-.10317,.870*1.0317-.0317,'Enter Normalized Radius (0-1):','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
    'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

radiusedit=uicontrol('position',[origin(1)+70 origin(2)+510 60 22],...
    'style','edit','string','0.5','callback',[],...
    'backgroundcolor',[1 1 1],'tag','radiusedit');

gobutton=uicontrol('position',[origin(1)+55 origin(2)+450 165 30],...
    'style','pushbutton','string','Find Alpha Shapes','backgroundcolor',handles.colors.buttonbg,...
    'callback',@make_ashapes,'enable','on','tag','startbutton');

%update tabobjects:
handles.tabobjects(end+1:end+3)=[radiustext radiusedit gobutton];

guidata(src,handles);


function make_ashapes(src,ev)
handles=guidata(src);

main=findobj(handles.activegraph,'tag','main');

xdata=get(main,'xdata');
ydata=get(main,'ydata');

xmax=max(xdata);
xmin=min(xdata);
ymax=max(ydata);
ymin=min(ydata);

xdata=(xdata-xmin)./(xmax-xmin);
ydata=(ydata-ymin)./(ymax-ymin);


radiusedit=findobj(handles.tabobjects,'tag','radiusedit');
radius=str2double(get(radiusedit,'string'));

P = ashape(xdata,ydata,radius,'-nc');

%restore to original scale;

xdata=P.x*(xmax-xmin)+xmin;
ydata=P.y*(ymax-ymin)+ymin;

shapez=[];
for i=1:numel(P.apatch)
    %     shapez(i)=patch('xdata',xdata(P.apatch{i}),'ydata',ydata(P.apatch{i}),'cdata',ydata(P.apatch{i}),...
    %         'edgecolor',[0 0 1],'facecolor','none','parent',handles.activegraph,'hittest','on','buttondownfcn',@lineobject_callback);
    try %#ok<TRYNC>
        shapez(end+1)=line(xdata(P.apatch{i}),ydata(P.apatch{i}),'color',[0 0 1],'parent',handles.activegraph,...
            'hittest','on','buttondownfcn',@lineobject_callback); %#ok<AGROW>
    end
end

userdata.handles=shapez;
userdata.delete=1;
set(shapez,'userdata',userdata); %needed for lineobject_callback



