function possiblepaths(src,ev)
%Possible Paths
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%TO DO: update this to use paretofront instead of pareto_filter

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

if ~ismember(handles.activetype,{'scatter';'scatter3d';'scattermatrix'})
    return
end


%explanatory text and a box asking how many paths to generate:
%check that both variables are functions:
if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %text headers:
    numpathstext=text(.25-.10317,.850*1.0317-.0317,'Number of paths to generate:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    %make an edit box for how many paths to generate:
    npathsedit=uicontrol('position',[origin(1)+55 origin(2)+500 165 22],...
        'style','edit','string','500','callback',[],...
        'backgroundcolor',[1 1 1],'tag','npathsedit');
    
    % go button
    gobutton=uicontrol('position',[origin(1)+55 origin(2)+460 165 30],...
        'style','pushbutton','string','Generate Paths','backgroundcolor',handles.colors.buttonbg,...
        'callback',@generatepaths_Callback,'enable','off','tag','gobutton');
    
    %get variables for these two functions:
    funvars=union(handles.funvars{udata.view.dataset}{udata.view.x},handles.funvars{udata.view.dataset}{udata.view.y});
    varnames=handles.allnames{udata.view.dataset}(funvars);
    
    %make an invisible listbox of the variables, for use in the
    %generatepaths_Callback function
    varordermenu=uicontrol('position',[-1030 origin(2)+300 180 210],...
        'style','listbox','string',varnames,'callback',[],...
        'backgroundcolor',[1 1 1],'tag','varordermenu','max',1000);
    
    
    %filtering uicontrols:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    filterheader=text(.15-.10317,.70*1.0317-.0317,'Filtering','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    targetstext=text(.25-.10317,.670*1.0317-.0317,'Enter target values:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    targ1text=text(.30-.10317,.635*1.0317-.0317,handles.allnames{udata.view.dataset}{udata.view.x},'horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    targ2text=text(.30-.10317,.590*1.0317-.0317,handles.allnames{udata.view.dataset}{udata.view.y},'horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    targ1edit=uicontrol('position',[origin(1)+190 origin(2)+380 50 22],...
        'style','edit','string','-inf','callback',[],...
        'backgroundcolor',[1 1 1],'tag','targ1edit');
    
    targ2edit=uicontrol('position',[origin(1)+190 origin(2)+350 50 22],...
        'style','edit','string','-inf','callback',[],...
        'backgroundcolor',[1 1 1],'tag','targ2edit');
    
    methodtext=text(.25-.10317,.540*1.0317-.0317,'Choose Filtering Method:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    meth1radio=uicontrol('position',[origin(1)+55 origin(2)+300 150 22],...
        'style','radiobutton','string','Paths not on Pareto Front','callback',@meth1radio_Callback,...
        'backgroundcolor',[1 1 1],'tag','meth1radio','value',1);
    meth2radio=uicontrol('position',[origin(1)+55 origin(2)+275 150 22],...
        'style','radiobutton','string','Dominated Paths','callback',@meth2radio_Callback,...
        'backgroundcolor',[1 1 1],'tag','meth2radio');
    
    filterbutton=uicontrol('position',[origin(1)+55 origin(2)+230 165 30],...
        'style','pushbutton','string','Filter Paths','backgroundcolor',handles.colors.buttonbg,...
        'callback',@filterpaths_Callback,'enable','off','tag','filterbutton');
    
    %update tabobjects:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    handles.tabobjects(end+1:end+14)=[numpathstext npathsedit gobutton varordermenu filterheader...
        targetstext targ1edit targ2edit targ1text targ2text filterbutton methodtext meth1radio meth2radio];
    
    %set graph callback to allow point selection
    set(handles.activegraph,'hittest','on','buttondownfcn',@point2pointselect)
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    
    handles.tabobjects(end+1)=nonotext;
end


guidata(src,handles);


function generatepaths_Callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
% x=udata.view.x;
% y=udata.view.y;

start=udata.explore.start;
finish=udata.explore.end;

%delete existing lines: (lineseries without a pentagram marker)
linez=findobj(handles.activegraph,'tag','possiblepaths');
if ~isempty(linez)
    delete(linez)
end

%determine which inputs variables are different between these two rows:
isdifferent=and(~handles.isresponse{d},handles.alldata{d}(start,:)~=handles.alldata{d}(finish,:));
allnames=handles.allnames{d};

%remove spaces from these to match varnames format (ravejmp2m removes
%spaces in variable names)
for j=1:length(allnames)
    allnames{j}=allnames{j}(allnames{j}~=char(32));
end

%get names of different variables:
diffnames=allnames(isdifferent);

%put names in order in which to make changes:
varordermenu=findobj(handles.tabobjects,'tag','varordermenu');
npathsedit=findobj(handles.tabobjects,'tag','npathsedit');
sortnames=get(varordermenu,'string');
includesorted=ismember(sortnames,diffnames);
sortnames=sortnames(includesorted);

%includethisvar=zeros(1,numel(sortnames));
realindex=zeros(1,numel(sortnames));

%find the index into allnames of sortnames.
for jj=1:numel(sortnames)
    realindex(jj)=find(strcmp(allnames,sortnames{jj}));
end

%LOOP FOR GENERATING MULTIPLE PATHS:
npaths=str2double(get(npathsedit,'string'));

%if npaths>full factorial, do full factorial.
npoints=7;
nvars=length(realindex);

xstore=zeros(npaths,nvars*npoints);
ystore=zeros(npaths,nvars*npoints);

for iii=1:npaths
    %reorder index randomly:
    realindex=realindex(randperm(nvars));
    
    %generate "DOE" table of inputs to evaluate:
    %use 7 runs for each variable: two near each end point, and three in the
    %middle, to calculate value and 1st derivatives at each end and in the
    %middle. note, rundata will contain incorrect values for response
    %variables, but these are never used anywhere.
    
    rundata=repmat(handles.alldata{d}(start,:),[npoints*nvars 1]); %start with matrix of starting point
    
    for v=1:nvars
        thisvar=realindex(v);
        startvalue=handles.alldata{d}(start,thisvar);
        endvalue=handles.alldata{d}(finish,thisvar);
        
        if startvalue~=endvalue
            values=(startvalue:(endvalue-startvalue)/(npoints-1):endvalue)';
        else
            values=startvalue*ones(npoints,1);
        end
        
        newcolumn=[startvalue*ones((v-1)*npoints,1);values;endvalue*ones((nvars-v)*npoints,1)];
        rundata(:,realindex(v))=newcolumn;
    end
    
    outs=raveevaluateregression(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],rundata);
    
    xvalues=outs(:,1);
    yvalues=outs(:,2);
    
    
    xstore(iii,:)=xvalues;
    ystore(iii,:)=yvalues;
    
end

%remove duplicate rows:

data=[xstore ystore];
uniquedata=unique(data,'rows');
xunique=uniquedata(:,1:npoints*nvars);
yunique=uniquedata(:,npoints*nvars+1:end);

npaths=size(yunique,1);

%make plots:

colors=hsv(npaths);

for jjj=1:npaths;
    line(xunique(jjj,:),yunique(jjj,:),'color',colors(jjj,:),'linewidth',2,...
        'parent',handles.activegraph,'hittest','on','buttondownfcn','uistack(gco,''top'')','tag','possiblepaths');
end


%bring stars to top:
stars=findobj(handles.activegraph,'type','line','marker','p');
uistack(stars,'top');

%enable filterbutton
filterbutton=findobj(handles.tabobjects,'tag','filterbutton');
set(filterbutton,'enable','on')

%store data
handles.temp.pathsdata=cat(3,xunique,yunique);
handles.temp.npaths=npaths; %number of paths.
handles.temp.nvars=nvars; %number of segments per path
handles.temp.npoints=npoints; %number of points per segment

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

guidata(src,handles);

function filterpaths_Callback(src,eventdata)

handles=guidata(src);

paretodata=handles.temp.pathsdata;

targ1edit=findobj(handles.tabobjects,'tag','targ1edit');
targ2edit=findobj('tag','targ2edit');
paretotarget=[str2double(get(targ1edit,'string')) str2double(get(targ2edit,'string'))];


meth1=findobj(handles.tabobjects,'tag','meth1radio');
method=get(meth1,'value');


%Reformat data such that a larger number is always best.

for j=1:2
    if paretotarget(j)==inf
        paretodata(:,:,j)=paretodata(:,:,j); %max is best
    elseif paretotarget(j)==-inf
        paretodata(:,:,j)=-paretodata(:,:,j); %min is best
    else
        paretodata(:,:,j)=-abs(paretodata(:,:,j)-paretotarget(j)); %nominal is best
    end
end


%% DO FILTERING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if method %METHOD 1: FILTER PATHS WITH NO POINTS ON PARETO FRONT
    %NOTE: only considers end points of paths, not intermediates. this is to avoid problems where
    %intermediate points of a suboptimal path are actually pareto optimal because they slip through
    %a gap between points on the true optimal paths.
    
    %remove intermediate points from paretodata array:
    paretodata=paretodata(:,[1 7:7:end],:);
    
    %reformat paretodata into 2D array, grouping all paths together:
    paretodata=reshape(paretodata,size(paretodata,1)*size(paretodata,2),2);
    
    isnondominated=pareto_filter(paretodata);
    
    %restore original data (replace max-is-best data):
    paretodata=handles.temp.pathsdata;
    paretodata=paretodata(:,[1 7:7:end],:);
    paretodata=reshape(paretodata,size(paretodata,1)*size(paretodata,2),2);
    
    %remove duplicate nondominated points (ie, identical points that lie on different paths):
    nondompoints=paretodata(isnondominated,:);
    nondompoints=unique(nondompoints,'rows');
    
    %if start or end point are nondominated, exclude them, since they are in every path:
    startpoint=paretodata(1,:);
    endpoint=paretodata(end,:);
    nondompoints=nondompoints(all(~ismember(nondompoints,startpoint)&~ismember(nondompoints,endpoint),2),:);
    
    %plot these points:
    %crap=line(nondompoints(:,1),nondompoints(:,2),'marker','+','linestyle','none','tag','crap');
    
    %find paths that contain these points:
    xdata=handles.temp.pathsdata(:,:,1);
    ydata=handles.temp.pathsdata(:,:,2);
    
    xvec=reshape(xdata,[size(xdata,1)*size(xdata,2),1]);
    yvec=reshape(ydata,[size(ydata,1)*size(ydata,2),1]);
    
    keepers=ismember(xvec,nondompoints(:,1)) & ismember(yvec,nondompoints(:,2));
    
    keepers=reshape(keepers,[size(xdata,1),size(xdata,2)]);
    
    keeppath=find(any(keepers,2));
    
    %delete existing paths
    lines=findobj(handles.activegraph,'type','line','-not','marker','p');
    npaths=length(lines);
    if ~isempty(lines)
        delete(lines)
    end
    
    
    colors=hsv(npaths);
    
    %plot keepers:
    for kk=1:length(keeppath)
        line(xdata(keeppath(kk),:),ydata(keeppath(kk),:),'marker','none','linewidth',2,...
            'color',colors(keeppath(kk),:),'hittest','on','buttondownfcn','delete(gco)')
    end
    
else %METHOD 2: FILTER DOMINATED PATHS
    
    %under construction.
    
end


%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

function meth1radio_Callback(hObject,eventdata)
handles=guidata(hObject);
set(hObject,'value',1)
meth2=findobj(handles.tabobjects,'tag','meth2radio');
set(meth2,'value',0)

function meth2radio_Callback(hObject,eventdata)
handles=guidata(hObject);
set(hObject,'value',1)
meth1=findobj(handles.tabobjects,'tag','meth1radio');
set(meth1,'value',0)


