function formattabderivativeprofiler(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%rave format tab for a scatterplot matrix graph.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %i think the (1) is for if multiple graphs are selected.
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);       
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%find this subax in the matrix:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);


%% text headers 
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblobsubax_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblobsubax_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblobsubax_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblobsubax_callback,'userdata',4,'tag','bgblob');

gridindex=eye(3);
gridin=format.grid;

%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activesubgraph,'xlim'); %for scatterplot matrix, can select individual subplots and change their properities from here.
ylim=get(handles.activesubgraph,'ylim');

if ~udata.view.singleyaxis
    xlab=get(get(handles.subax{handles.activeindex}(end,c),'xlabel'),'string');
    xlab=xlab{4};
    ylab=get(get(handles.subax{handles.activeindex}(r,1),'ylabel'),'string');
    ylab=ylab{1};
else
    xlab='';
    ylab='';

end

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',xlab,'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',ylab,'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);

titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

titel=findobj(handles.activegraph,'tag','title');

titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(titel,'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%% DATA SERIES UICONTROLS

smoothtext=uicontrol('position',[origin(1)+25 origin(2)+274 180 20],...
    'style','text','string','Resolution:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%smoothness slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+110 origin(2)+262 140 40]);
jslider.setPaintTicks(1)

jslider.setSnapToTicks(1)
jslider.setMinimum(4)
jslider.setMaximum(16)
jslider.setMajorTickSpacing(2);
jslider.setValue(udata.view.numpts);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@jslider_Callback,handles.fig});

smoothtext2=uicontrol('position',[origin(1)+107 origin(2)+248 150 20],...
    'style','text','string','Low                                  High','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','left','fontweight','normal');

linecolortext=uicontrol('position',[origin(1)+25 origin(2)+220 180 20],...
    'style','text','string','Profiler Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

haircolortext=uicontrol('position',[origin(1)+25 origin(2)+190 180 20],...
    'style','text','string','Tracer Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

linecolorbutton=uicontrol('position',[origin(1)+125 origin(2)+222 22 22],...
    'style','pushbutton','string','','backgroundcolor',udata.format.linecolor,'callback',@linecolor_Callback);

haircolorbutton=uicontrol('position',[origin(1)+125 origin(2)+192 22 22],...
    'style','pushbutton','string','','backgroundcolor',udata.format.haircolor,'callback',@haircolor_Callback);

smartcolorcheck=uicontrol('position',[origin(1)+55 origin(2)+160 200 26],...
    'style','checkbox','string','Draw invalid tracers in gray',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.smarthaircolors,'callback',@smartcolor_Callback);

linesizetext=uicontrol('position',[origin(1)+58 origin(2)+163-30 44 20],...
    'style','text','string','Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%width slider:
[jslider2 hslider2]=javacomponent('javax.swing.JSlider',[origin(1)+130 origin(2)+167-30 100 27]);
jslider2.setPaintTicks(1)
jslider2.setSnapToTicks(1)
jslider2.setMinimum(1)
jslider2.setMaximum(5)
jslider2.setMajorTickSpacing(1);
jslider2.setValue(format.linesize);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@linewidth_callback,handles.fig});

handles.javacomps=[jslider jslider2];

widthaxes=axes('units','pixels','position',[origin(1)+125 origin(2)+167-35 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');


showpointscheck=uicontrol('position',[origin(1)+25 origin(2)+89 200 26],...
    'style','checkbox','string','Show Data Set',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@showpoints_Callback);

flatcolorcheck=uicontrol('position',[origin(1)+25 origin(2)+69 200 26],...
    'style','radiobutton','string','Fixed Color:',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@flatcolor_Callback,'tag','flatcolorcheck');

ccolorcheck=uicontrol('position',[origin(1)+25 origin(2)+49 200 26],...
    'style','radiobutton','string','Color by Variable:',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@ccolor_Callback,'tag','ccolorcheck');

pointcolorbutton=uicontrol('position',[origin(1)+125 origin(2)+90 22 22],...
    'style','pushbutton','string','','backgroundcolor',udata.format.pointcolor,'callback',@pointcolor_Callback);




%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton ...
    titletext titlebox smoothtext smoothtext2 linecolortext haircolortext...
    linecolorbutton haircolorbutton smartcolorcheck hslider linesizetext...
    widthaxes hslider2 showpointscheck pointcolorbutton flatcolorcheck ccolorcheck];

handles.javacomps=[jslider jslider2];
   
handles.activetab='format';

set(handles.infobar,'string','Note: You cannot change profiler variable values while on Format Tab')

guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(1)=newval;

%get coords of this subgraph:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function xmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function ymin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(1)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function ymax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function xresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activesubgraph,'xlimmode','auto')
xlim=get(handles.activesubgraph,'xlim');

[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);
set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)


xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activesubgraph,'ylimmode','auto')
ylim=get(handles.activesubgraph,'ylim');


[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if udata.view.singleyaxis
    set(get(handles.subax{handles.activeindex}(1),'ylabel'),'string',{get(src,'string');'';'';''})   

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function jslider_Callback(src,ev,fighandle) %#ok<INUSL>

moving = get(src,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end

handles=guidata(fighandle);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.numpts=src.getValue;

set(handles.fakeax(handles.activeindex),'userdata',udata)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function linecolor_Callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%udata.format.linecolor=uisetcolor(udata.format.linecolor);

udata.format.linecolor=ravecolorchooser(handles.fig,udata.format.linecolor,'Pick a Color');



udata.format.linecolorString=num2str(udata.format.linecolor);
set(src,'backgroundcolor',udata.format.linecolor)

targets=findobj(cell2mat(get(handles.subax{handles.activeindex},'children')),'tag','keeper');
set(targets,'color',udata.format.linecolor)

set(handles.fakeax(handles.activeindex),'userdata',udata);

for r=1:size(handles.subax{handles.activeindex},1)
    oldstring=get(get(handles.subfakeax{handles.activeindex}(r,1),'ylabel'),'string');
    start=find(oldstring{1}=='{');
    finish=find(oldstring{1}=='}');  
    newstring=[oldstring{1}(1:start),udata.format.linecolorString,oldstring{1}(finish:end)]; 
    set(get(handles.subfakeax{handles.activeindex}(r,1),'ylabel'),'string',{newstring;'';''});
end

for c=1:size(handles.subax{handles.activeindex},2)
    oldstring=get(get(handles.subfakeax{handles.activeindex}(end,c),'xlabel'),'string');
    start=find(oldstring{3}=='{');
    finish=find(oldstring{3}=='}');  
    newstring=[oldstring{3}(1:start),udata.format.linecolorString,oldstring{3}(finish:end)]; 
    set(get(handles.subfakeax{handles.activeindex}(end,c),'xlabel'),'string',{' ';' ';newstring});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function haircolor_Callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%udata.format.haircolor=uisetcolor(udata.format.haircolor);

oldcolor=udata.format.haircolor;
udata.format.haircolor=ravecolorchooser(handles.fig,udata.format.haircolor,'Pick a Color');


set(src,'backgroundcolor',udata.format.haircolor)

targets=findobj(cell2mat(get(handles.subax{handles.activeindex},'children')),'tag','hair','color',oldcolor);
set(targets,'color',udata.format.haircolor)

set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function smartcolor_Callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.smarthaircolors=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatederivativeprofiler(src,[]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

titel=findobj(handles.activegraph,'tag','title');
titlestring=get(src,'string');
set(titel,'string',titlestring)

udata.format.title=titlestring;
set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);

if isempty(titlestring)
    titoffset=0;
else
    titoffset=25;
end

handles=guidata(src);

%copied from ravemovegraph:
numy=size(handles.subax{handles.activeindex},1);
numx=size(handles.subax{handles.activeindex},2);
pos = get(handles.activegraph,'Position');

vspace=0.05;
hspace=0.00;

width = pos(3)/numx;
height = (pos(4)-titoffset)/numy;

pos(1:2) = pos(1:2) + [hspace*width vspace*height];

ax=handles.subax{handles.activeindex};
fakeax=handles.subfakeax{handles.activeindex};
fakerax=handles.subfakerax{handles.activeindex};

%set up grid of axes.
for r=numy:-1:1,
    for c=numx:-1:1,
        axPos = [pos(1)+(c-1)*width pos(2)+(numy-r)*height ...
            width*(1-hspace) height*(1-vspace)];
        set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',axPos); 
        
    end
end


function linewidth_callback(src,eventdata,fighandle) %#ok<INUSL>

handles=guidata(fighandle);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=src.getValue;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=findobj(handles.subax{handles.activeindex},'tag','keeper');

%filter out line series with no marker
% nottargets=findobj(targets,'linestyle','none');
% 
% targets=setdiff(targets,nottargets);

%update graph
set(targets,'linewidth',udata.format.linesize);


function showpoints_Callback(src,ev)
handles=guidata(src);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.showpoints=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatederivativeprofiler(src,[]);



function pointcolor_Callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.pointcolor=ravecolorchooser(handles.fig,udata.format.haircolor,'Pick a Color');
set(src,'backgroundcolor',udata.format.pointcolor)

targets=findobj(cell2mat(get(handles.subax{handles.activeindex},'children')),'tag','points');
set(targets,'markerfacecolor',udata.format.pointcolor)

set(handles.fakeax(handles.activeindex),'userdata',udata);


function flatcolor_Callback(src,ev)

function ccolor_Callback(src,ev)

