function ravecreatederivativeprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


targetaxes=handles.activegraph; %=handles.activegraph
ycol=udata.view.y;
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

xdials=handles.xdials{d}{a};

numy=length(ycol);
numx=length(xcol);
numpts=udata.view.numpts;

if numx==0 %this can happen when switching to a new yvar that shares no vars with previous xvar selection.
    numx=1;
    xcol=1;
end

if numy==0
    set(handles.infobar,'string','Error: No variables are modeled by Equations.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})


%get data
xnames=handles.allnames{d}(xcol);
ynames=handles.allnames{d}(ycol);

set(targetaxes,'Visible','off')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graphtitle,'fontsize',handles.prefs.fonts.graphtitlesize,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=25;
end

if ~udata.view.singleyaxis
    rows=numy;
else
    rows=1;
end
cols=numx;

% Create and plot into small axes
ax = zeros(rows,cols);
fakeax = zeros(rows,cols);
fakerax = zeros(rows,cols);

pos = get(targetaxes,'Position');


hspace = handles.prefs.profilerhspacing;
vspace = handles.prefs.profilervspacing; % 5 percent space between subaxes

width = floor((pos(3)-hspace*numx)/numx);
height = floor((pos(4)-titoffset-vspace*numy)/numy);

xlim = zeros([numy numx 2]);
ylim = zeros([numy numx 2]);

BigAxParent = get(targetaxes,'Parent');

%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);
ydatatypes=handles.datatype{d}(ycol);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));

for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end

rowz=sum(totalrows);
xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:numx
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
    
else
    for k=1:numx
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:

yvalues=raveevaluateregression(src,d,udata.view.y,xfixed);

%update axis limits only if needed:
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end


ycurrent=raveevaluateregression(src,d,udata.view.y,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));
ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end


%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

%% set up grid of axes.
for r=rows:-1:1,
    for c=cols:-1:1,
        axPos = [round(pos(1))+(c-1)*(width+hspace) pos(2)+(rows-r)*(height+vspace) ...
            width height];
        
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,'color',bgcolor,...
            'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@raveupdateprofiler,...
            'userdata',handles.activeindex,'color','none',...
            'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %userdata contains the index of the bigax for this subgraph.
        set(ax(r,c),'visible','on');
        set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on');
        
        set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ticklength',[.02 .02])
        if strcmp(ydatatypes{r},'logical')
            set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ytick',[0 1],'yticklabel',{handles.prefs.logicalfalse handles.prefs.logicaltrue})
        end
        
        thisxcolor=udata.format.haircolor;
        thisycolor=udata.format.haircolor;
        if udata.format.smarthaircolors
            if handles.isconstant{udata.view.dataset}(xcol(c))
                thisxcolor=get(handles.fakeax(handles.activeindex),'xcolor');
                set(ax(r,c),'buttondownfcn','') %this var is constant so shouldnt update profiler when clicking it.
            end
            
            if ~ismember(xcol(c),handles.funvars{udata.view.dataset}{ycol(r)})
                thisycolor=get(handles.fakeax(handles.activeindex),'xcolor');
            end
        end
        %hairline data series:
        h.xhair(r,c)=line([xdials(xcol(c)) xdials(xcol(c))],[0 1],'visible','on','linestyle',':','color',thisxcolor,...
            'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1);
        %response hairline series:
        if ~udata.view.singleyaxis
            
            h.yhair(r,c)=line([-10000000 1000000000],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',':','color',thisycolor,...
                'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1);
        else
            for R=1:numy
                thisycolor=handles.variablecolors{d}(ycol(R),:);
                h.yhair(R,c)=line([-10000000 1000000000],[ycurrent{R} ycurrent{R}],'visible','on','linestyle',':','color',thisycolor,...
                    'parent',ax(r,c),'hittest','off','tag','hair');
            end
        end
        %main data series:
        
        if  ~any(strcmp(datatypes{c},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(c)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{c}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            if ~udata.view.singleyaxis
                h.main(r,c)=line(xvals(xmap),yvalues{r}{c}(ymap),'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                    'color',udata.format.linecolor,'hittest','off','tag','keeper','parent',ax(r,c));
            else
                for R=1:numy
                    h.main(R,c)=line(xvals(xmap),yvalues{R}{c}(ymap),'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                        'color',handles.variablecolors{d}(ycol(R),:),'hittest','off','tag','keeper','parent',ax(r,c));
                end
            end
            
            
            
        else
            if ~udata.view.singleyaxis
                h.main(r,c)=line(xsteps{c},yvalues{r}{c},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                    'color',udata.format.linecolor,'hittest','off','tag','keeper','parent',ax(r,c));
                %                h.label(r,c)=[];
                
            else
                for R=1:numy
                    h.main(R,c)=line(xsteps{c},yvalues{R}{c},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                        'color',handles.variablecolors{d}(ycol(R),:),'hittest','off','tag','keeper','parent',ax(r,c));
                    %                     h.label(r,c)=text(min(get(ax(r,c),'xlabel')),ycurrent{R},'blah  ','parent',ax(r,c),'color',handles.variablecolors{d}(ycol(R),:),...
                    %                         'tag','mainlabel','userdata',R,'hor','right');
                end
            end
            
            
            
        end
        
        if udata.format.showpoints && ~udata.view.singleyaxis
            nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
            xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.x(c));
            ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(r));
            
            h.points(r,c)=line(xdata,ydata,'parent',ax(r,c),'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
                'marker','o','markersize',6,'hittest','on','buttondownfcn',@raveupdateprofiler);
            uistack(h.points(r,c),'bottom');
        end
        
        set(ax(r,c),'xlimmode','auto','ylim',[handles.minlimit{d}{a}(ycol(r)) handles.maxlimit{d}{a}(ycol(r))],'ylimmode','manual','xgrid','off','ygrid','off')
        xlim(r,c,:) = get(ax(r,c),'xlim');
        ylim(r,c,:) = get(ax(r,c),'ylim');
    end
end

if numel(ycol)>1 && udata.view.singleyaxis
    leg1=legend(handles.activegraph,h.main(:,1),handles.allnames{d}(ycol),'interpreter',handles.prefs.interpreter);
    set(handles.fakerax(handles.activeindex),'userdata',leg1)
else
    leg1=[];
end



xlimmin = xmin(xcol);
xlimmax = xmax(xcol);
ylimmin=handles.minlimit{d}{a}(ycol);
ylimmax=handles.maxlimit{d}{a}(ycol);

for u=1:cols
    if xlimmin(u)==xlimmax(u)
        xlimmin(u)=.9999*xlimmin(u);
        xlimmax(u)=1.0001*xlimmax(u);
    end
end

% Try to be smart about axes limits and labels.  Set all the limits of a
% row or column to be the same and inset the tick marks by 10 percent.
inset = .10;

%fix y limits
for r=1:rows,
    if ~udata.view.singleyaxis
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
        set(h.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
    else
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
        set(h.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
    end
    
    
    %force non exponential labels:
    if ~strcmp(ydatatypes{r},'logical')
        set(ax(r,1),'YTickLabel',get(ax(r,1),'YTick'))
    end
    set(fakeax(r,1),'YTickLabel','')
    set(fakerax(r,1),'YTickLabel','')
    
    %make axes labels:
    if ~udata.view.singleyaxis
        set(get(ax(r,1),'ylabel'),'string',{ynames{r};'';'';''},'hittest','off','interpreter','none');
        set(get(fakeax(r,1),'ylabel'),'string',{['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(ycurrent{r},5)];'';''},'interpreter','tex','hittest','off');
    else
        set(get(ax(r,1),'ylabel'),'string','','hittest','off','interpreter','none');
        set(get(fakeax(r,1),'ylabel'),'string','','interpreter','tex','hittest','off');
        
    end
    
    
end
dx = zeros(1,cols);

%fix x limits
for c=1:cols,
    
    set([ax(1,c) fakeax(1,c) fakerax(1,c)],'xlim',[xlimmin(c) xlimmax(c)])
    dx(c) = diff(get(ax(1,c),'xlim'))*inset;
    set([ax(:,c) fakeax(:,c) fakerax(:,c)],'xlim',[xlimmin(c)-dx(c) xlimmax(c)+dx(c)])
    
    %force non exponential labels:
    set(ax(end,c),'XTickLabel',get(ax(end,c),'XTick'))
    set(fakeax(end,c),'XTickLabel','')
    set(fakerax(end,c),'XTickLabel','')
    
    %make axes labels:
    set(get(ax(1,c),'title'),'string',xnames{c},'interpreter','none');
    set(get(ax(end,c),'xlabel'),'string',{'';'';'';xnames{c}},'verticalalignment','top','hittest','off','interpreter','none');
    set(get(fakeax(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(xdials(xcol(c)),5)]},'verticalalignment','top','interpreter','tex','hittest','off');
    
end

set(ax(1:rows-1,:),'xticklabel','')
set(ax(:,2:cols),'yticklabel','')

set(fakeax(1:rows-1,:),'xticklabel','')
set(fakeax(:,2:cols),'yticklabel','')

set(fakerax(1:rows-1,:),'xticklabel','')
set(fakerax(:,2:cols),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','derivativeprofiler','tag','DerivativeProfilerBigAx');

set(ax,'xlimmode','manual','ylimmode','manual')




%if more than 4 variables, rotate x axis labels 90 degrees:

%if less than 250 pixels per variable, rotate x axis labels 90 degrees: (use this for all labels so hittest can
%bet set to off.


if pos(3)/cols<250
    rotationz=90;
    halign='right';
    valign='default';
else
    rotationz=0;
    halign='center';
    valign='top';
end

for y=1:numx
    
    thistype=datatypes{y};
    switch thistype
        case {'continuous','constant','integer'}
            putthemhere=get(ax(end,y),'XTick');
            usethesevalues=get(ax(end,y),'XTickLabel');
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{y};
            if numel(discretez)<=12;
                putthemhere=discretez;
                usethesevalues=discretez;
            else %too many, just treat as continuous
                putthemhere=get(ax(end,y),'XTick');
                usethesevalues=get(ax(end,y),'XTickLabel');
            end
        case 'string'
            usethesevalues=handles.discretevalues{d}{a}{y};
            putthemhere=unique(handles.alldata{d}(:,y));
            
        case 'logical'
            usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            putthemhere=[0 1];
    end
    
    set(ax(end,y),'XTickLabel',[]);
    set(fakeax(end,y),'XTickLabel',[]);
    set(fakerax(end,y),'XTickLabel',[]);
    
    
    ylimits=get(ax(end,1),'Ylim');
    ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
    
    text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
        'HorizontalAlignment',halign,'rotation',rotationz,'parent',ax(end,y),'hittest','off',...
        'verticalalignment',valign,'handlevisibility','off','tag','xticklabel');
end


%also replace yticklabels with strings, this allows y (axis) labels to be aligned with each other.
for y=1:rows
    
    usethesevalues=get(ax(y,1),'YTickLabel');
    set(ax(y,1),'YTickLabel',[]);
    set(fakeax(y,1),'YTickLabel',[]);
    set(fakerax(y,1),'YTickLabel',[]);
    
    putthemhere=get(ax(y,1),'YTick');
    
    xlimits=get(ax(end,1),'Xlim');
    xpos=xlimits(1)-.02*(xlimits(2)-xlimits(1));
    
    text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
        'HorizontalAlignment','right','rotation',0,'parent',ax(y,1),'hittest','off',...
        'verticalalignment','middle','handlevisibility','off','tag','yticklabel');
end

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')

%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
handles.statsax{handles.activeindex}=[];

%select the subaxes in position (1,2) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%store handles to the data sets:
handles.profilers.main{handles.activeindex}=h.main;
handles.profilers.xhair{handles.activeindex}=h.xhair;
handles.profilers.yhair{handles.activeindex}=h.yhair;

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

%update da handles
guidata(src,handles)

%shrink axes to encompass labels
raveresizeaxes(handles.fig,[],handles.activeindex)

handles=guidata(src);

%copied from ravemovegraph:
numy=size(handles.subax{handles.activeindex},1);
numx=size(handles.subax{handles.activeindex},2);
pos = get(handles.activegraph,'Position');

vspace=handles.prefs.matrixspacing;
hspace=handles.prefs.matrixspacing;

if strcmp(handles.activetype,'derivativeprofiler')
    %pos=pos;
    vspace=handles.prefs.profilervspacing;
    hspace=handles.prefs.profilerhspacing;
end

titel=findobj(handles.activegraph,'tag','title');
titlestring=get(titel,'string');

if isempty(titlestring)
    titoffset=0;
else
    titoffset=10;
    if strcmp(handles.activetype,'derivativeprofiler')
        titoffset=25;
    end
end


width = floor((pos(3)-numx*hspace)/numx);
height = floor((pos(4)-titoffset-numy*vspace)/numy);



ax=handles.subax{handles.activeindex};
fakeax=handles.subfakeax{handles.activeindex};
fakerax=handles.subfakerax{handles.activeindex};

%set up grid of axes.
for r=numy:-1:1,
    for c=numx:-1:1,
        axPos = [pos(1)+(c-1)*(width+hspace) pos(2)+(numy-r)*(height+vspace) ...
            width height];
        set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',axPos);
        
    end
end

%store axes labels so don't need to waste time "get"ing them later:


ylabels=get(handles.subfakeax{handles.activeindex}(:,1),'ylabel');
if iscell(ylabels)
    ylabels=cell2mat(ylabels);
end
udata.view.ylabels=ylabels;
set(handles.fakeax(handles.activeindex),'userdata',udata);







