function ravecreatecarpet2d(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z; %"sweep through values of"
ccol=udata.view.c; %"draw a line for each value of"

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontsize',14,'tag','title')

sweepvar1=zcol;
[sweepvalues1,totalrows]=ravecalculatexsteps(handles.fig,d,a,sweepvar1,udata.view.numpts1,0);
sweepvalues1=sweepvalues1{1};

sweepvar2=ccol;
[sweepvalues2,totalrows]=ravecalculatexsteps(handles.fig,d,a,sweepvar2,udata.view.numpts2,0);
sweepvalues2=sweepvalues2{1};


if numel(sweepvalues1)==1;
    sweepvalues1=[sweepvalues1 sweepvalues1];
end

if numel(sweepvalues2)==1;
    sweepvalues2=[sweepvalues2 sweepvalues2];
end


if ccol~=zcol && ccol~=0 && ~isempty(udata.view.steps)   
    zcalldata=repmat(sweepvalues1(:),numel(udata.view.steps),1);
    stepdata=repmat(udata.view.steps(:)',numel(sweepvalues1),1);
    stepdata=stepdata(:);
    
else
    zcalldata=sweepvalues1(:);
    stepdata=[];
end

[S1,S2]=meshgrid(sweepvalues1,sweepvalues2);

S1=S1(:);
S2=S2(:);

constants=repmat(handles.xdials{d}{a},[length(S1),1]);
constants(:,zcol)=S1;
constants(:,ccol)=S2;

%evaluate the functions:

outvalues=raveevaluateregression(src,d,[udata.view.x udata.view.y],constants);
xvalues=outvalues(:,1);
yvalues=outvalues(:,2);

[S1,S2]=meshgrid(sweepvalues1,sweepvalues2);

Xvalues=reshape(xvalues,size(S1));
Yvalues=reshape(yvalues,size(S2));

Zvalues=zeros(size(S1));

% constants(:,xcol)=xvalues;
% constants(:,ycol)=yvalues;
% 
% ncurves=numel(zcalldata)./numel(sweepvalues1);
% npts=numel(sweepvalues1);




%if ccol~=0

%normalize cdata to desired colormap:
if ~udata.format.reversecolors1
    C1=(S1-min(S1(:)))/(max(S1(:))-min(S1(:))).*127+1+128.*(udata.format.mcmap1-1);
else
    C1=-(S1-min(S1(:)))/(max(S1(:))-min(S1(:)))*127+128*(udata.format.mcmap1);
end
    
    
%normalize cdata to desired colormap:
if ~udata.format.reversecolors1
    C2=(S2-min(S2(:)))/(max(S2(:))-min(S2(:))).*127+1+128.*(udata.format.mcmap2-1);
else
    C2=-(S2-min(S2(:)))/(max(S2(:))-min(S2(:)))*127+128*(udata.format.mcmap2);
end
      
    
% else    
%     %c=zeros(length(x),1);
%     if format.ncolor
%         c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'+10*rand(length(x),1)-.5; %noisy colors
%     else
%         c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'; %non-noisy colors cdata=0.5      
%     end
% end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);


surf1=surface(Xvalues,Yvalues,Zvalues,'parent',targetaxes,'facecolor','none','edgecolor','flat','tag','main1','meshstyle','column','cdata',C1);
surf2=surface(Xvalues,Yvalues,Zvalues,'parent',targetaxes,'facecolor','none','edgecolor','flat','tag','main2','meshstyle','row','cdata',C2);


if ~udata.view.isvisible1;set(surf2,'visible','off');end
if ~udata.view.isvisible2;set(surf1,'visible','off');end



% 
% 
% for n=1:ncurves %#ok<BDSCI>
%    theserows=constants((n-1)*npts+1:n*npts,:);
%    line(theserows(:,xcol),theserows(:,ycol),'parent',targetaxes)
% end


xlim = get(targetaxes,'xlim');
ylim = get(targetaxes,'ylim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',ylim);
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',ylim);





%% make tick marks:

labelsize=12;
fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter','none','fontname','arial')
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)

set(targetaxes,'hittest','on','buttondownfcn',@raveupdatecontour);

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar
ravemakecolorbar(handles.fig,[]);
%end

ravemaketextticks(targetaxes,[],xcol(1),ycol,[],[])

raveresizeaxes(src,[],targetindex)


%draw constraints:

constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end







