function ravecreatecontour(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
% 
% xnames=handles.allnames{d}(xcol);
% ynames=handles.allnames{d}(ycol);

%titlestring=udata.format.title;
%make overall title
% text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
%     'interpreter',handles.prefs.interpreter,'fontsize',handles.prefs.fonts.graphsize+2,'fontname',handles.prefs.fonts.graph,'tag','title')


%prepare data for gridding:
if isempty(xcol);
    return %no functions loaded.
end


xcol=[xcol ycol];

datatypes=handles.datatype{d}(xcol);
totalrows=zeros(size(datatypes));
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end

%precompute data so contour levels can be shared among all plots:
allthedata=cell(3);

thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,xcol(2))=ycalldata;

%evaluate the functions:

zvalues=raveevaluateregression(src,d,udata.view.z,constants);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata(3);
zdataz=zdataz(:);
zdataz=cell2mat(zdataz);
zdataz=zdataz(:);

doupdate=0;
if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end
if doupdate;guidata(handles.fig,handles);end;


lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);

udata.view.clim=[lowestvalue highestvalue];
set(handles.fakeax(handles.activeindex),'userdata',udata);


zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

% contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
mainhandles=contourplot(C,'style','line','parent',targetaxes,'linewidth',udata.format.linesize,'hittest','off');


%use single precision axes limits for correct positioning on screen:
xlimz=single([allthedata{1}(1,1) allthedata{1}(1,end)]);
ylimz=single([allthedata{2}(1,1) allthedata{2}(end,1)]);

set(targetaxes,'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual','xgrid','off','ygrid','off','layer','top')

%make tracers
if udata.format.usevariablecolors
    L1=line([xdials(xcol(1)) xdials(xcol(1))],[xmin(xcol(2)) xmax(xcol(2))],'parent',targetaxes,'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(c),:));
    L2=line([xmin(xcol(1)) xmax(xcol(1))],[xdials(xcol(2)) xdials(xcol(2))],'parent',targetaxes,'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(r),:));
else
    L1=line([xdials(xcol(1)) xdials(xcol(1))],[xmin(xcol(2)) xmax(xcol(2))],'parent',targetaxes,'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',udata.format.tracercolor);
    L2=line([xmin(xcol(1)) xmax(xcol(1))],[xdials(xcol(2)) xdials(xcol(2))],'parent',targetaxes,'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',udata.format.tracercolor);
    
end

xtracerhandles=L1;
ytracerhandles=L2;



xlim = get(targetaxes,'xlim');
ylim = get(targetaxes,'ylim');


clear contourplot %clear persistent vars;

handles.profilers.main{handles.activeindex}=mainhandles;
handles.profilers.xhair{handles.activeindex}=xtracerhandles;
handles.profilers.yhair{handles.activeindex}=ytracerhandles;



set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',ylim);

set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',ylim);





%% make tick marks:

labelsize=12;
fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph)

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname','arial')
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)

set(targetaxes,'hittest','on','buttondownfcn',@raveupdatecontour);

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar
ravemakecolorbar(handles.fig,[]);
%end

ravemaketextticks(targetaxes,[],xcol(1),ycol,[],[])

raveresizeaxes(src,[],targetindex)


%draw constraints:

constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end







