function formattabhyperslice(src,eventdata)
%rave format tab for a scatterplot matrix graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.16*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblobsubax_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblobsubax_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblobsubax_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblobsubax_callback,'userdata',4,'tag','bgblob');

gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activesubgraph,'xlim'); %for scatterplot matrix, can select individual subplots and change their properities from here.
ylim=get(handles.activesubgraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activesubgraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback,'tag','xlabelbox');
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activesubgraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback,'tag','ylabelbox');
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


titel=findobj(handles.activegraph,'tag','title');

titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(titel,'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%% DATA SERIES UICONTROLS

markercolortext=uicontrol('position',[origin(1)+25 origin(2)+280 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});


markercolor=uicontrol('position',[origin(1)+75 origin(2)+282 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);

linestyletext=uicontrol('position',[origin(1)+25 origin(2)+195 55 20],...
    'style','text','string','Tracers:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
linesizetext=uicontrol('position',[origin(1)+25 origin(2)+240 44 20],...
    'style','text','string','Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

linestyleaxes=axes('units','pixels','position',[origin(1)+95 origin(2)+195 100 22],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);

offcolor=[.5 .5 .5];
oncolor=handles.colors.text;
line([0 1],[.9 .9],'linestyle','-','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.62 .62],'linestyle','--','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.38 .38],'linestyle',':','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.1 .1],'linestyle','-.','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')

hhh=findobj(linestyleaxes,'tag','lineblob','linestyle',udata.format.linestyle);
set(hhh,'color',oncolor)

%width slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+80 origin(2)+247 100 27]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(5)
jslider.setMajorTickSpacing(1);
jslider.setValue(udata.format.linesize);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@linewidth_callback,handles.fig});

handles.javacomps=jslider;

widthaxes=axes('units','pixels','position',[origin(1)+75 origin(2)+237 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');

boldcheck=uicontrol('position',[origin(1)+94 origin(2)+170 100 26],...
    'style','checkbox','string','Thick Lines',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.bold,'callback',@boldcheck_callback);

facecheck=uicontrol('position',[origin(1)+94 origin(2)+105 150 20],'style','radiobutton',...
    'string','','value',~udata.format.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@facecheck_callback,'tag','facecheck');
facetext=uicontrol('position',[origin(1)+111 origin(2)+102 150 20],...
    'style','text','string','Face Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

facecolor=uicontrol('position',[origin(1)+195 origin(2)+102 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.tracercolor,'value',0,...
    'callback',@faceblob_callback);

ortext=uicontrol('position',[origin(1)+141 origin(2)+125 50 20],...
    'style','text','string','or','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','normal');

varcolorscheck=uicontrol('position',[origin(1)+94 origin(2)+147 150 20],'style','radiobutton',...
    'string','','value',udata.format.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@varcheck_callback,'tag','varcolors');
varcolorstext=uicontrol('position',[origin(1)+111 origin(2)+144 150 20],...
    'style','text','string','Use Variable Colors','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+48 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold');

colorbarcheck=uicontrol('position',[origin(1)+85 origin(2)+26 160 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);


%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton markercolortext...
   markercolor titletext titlebox colorbarcheck boldcheck...
    legendbutton linestyletext linesizetext hslider linestyleaxes widthaxes...
    legendtext facecheck facetext facecolor ortext varcolorscheck varcolorstext];

handles.activetab='format';

set(handles.infobar,'string','Note, you cannot interact with graphs while on Format tab');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(1)=newval;

%get coords of this subgraph:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function xmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function ymin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(1)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function ymax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function xresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activesubgraph,'xlimmode','auto')
xlim=get(handles.activesubgraph,'xlim');

[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);
set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activesubgraph,'ylimmode','auto')
ylim=get(handles.activesubgraph,'ylim');


[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,ev)

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

newcmap=ravechoosecolormap(src,[],udata.format.mcmap);
drawnow;
if newcmap==udata.format.mcmap;return;end
udata.format.mcmap=newcmap;

colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehyperslice(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

titel=findobj(handles.activegraph,'tag','title');

titlestring=get(src,'string');
set(titel,'string',titlestring)

udata.format.title=titlestring;
set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);

if isempty(titlestring)
    titoffset=0;
else
    titoffset=10;
end

handles=guidata(src);

%copied from ravemovegraph:
numy=size(handles.subax{handles.activeindex},1);
numx=size(handles.subax{handles.activeindex},2);
pos = get(handles.activegraph,'Position');

vspace=0.03;
hspace=0.03;

width = pos(3)/numx;
height = (pos(4)-titoffset)/numy;

pos(1:2) = pos(1:2) + [hspace*width vspace*height];

ax=handles.subax{handles.activeindex};
fakeax=handles.subfakeax{handles.activeindex};
fakerax=handles.subfakerax{handles.activeindex};

%set up grid of axes.
for r=numy:-1:1,
    for c=numx:-1:1,
        axPos = [pos(1)+(c-1)*width pos(2)+(numy-r)*height ...
            width*(1-hspace) height*(1-vspace)];
        set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',axPos); 
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lineblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

lineblobs=findobj(handles.tabobjects,'tag','lineblob');

offcolor=[.5 .5 .5];
set(lineblobs,'color',offcolor)
set(src,'color',handles.colors.text)

udata.format.linestyle=get(src,'linestyle');
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(cell2mat(handles.profilers.xhair{handles.activeindex}(:)),'linestyle',get(src,'linestyle'));
set(cell2mat(handles.profilers.yhair{handles.activeindex}(:)),'linestyle',get(src,'linestyle'));

function linewidth_callback(jsrc,ev,src)

handles=guidata(src);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=jsrc.getValue;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=cell2mat(handles.profilers.main{handles.activeindex}(:)');

%update graph
set(targets,'linewidth',udata.format.linesize);

function boldcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.bold=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(cell2mat(handles.profilers.xhair{handles.activeindex}(:)),'linewidth',1+get(src,'value'));
set(cell2mat(handles.profilers.yhair{handles.activeindex}(:)),'linewidth',1+get(src,'value'));

function facecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
varcolors=findobj(handles.tabobjects,'tag','varcolors');
set(varcolors,'value',0);
udata.format.usevariablecolors=0;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:


function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.tracercolor,'Choose a marker fill color');
set(src,'backgroundcolor',newcolor);
udata.format.tracercolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
if ~udata.format.usevariablecolors
    set(cell2mat(handles.profilers.xhair{handles.activeindex}(:)),'color',newcolor); 
    set(cell2mat(handles.profilers.yhair{handles.activeindex}(:)),'color',newcolor); 
end



function varcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
facecheck=findobj(handles.tabobjects,'tag','facecheck');
set(facecheck,'value',0);
udata.format.usevariablecolors=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:




function colorbarcheck_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
legdata=get(handles.fakerax(handles.activeindex),'userdata');

if get(src,'value')
        udata.format.colorbar=1;
        set(handles.fakeax(handles.activeindex),'userdata',udata);
        ravemakecolorbar(handles.fig,[]);
else
    %check for existing colorbar:
    cbar=findobj('tag','Colorbar','userdata',handles.activeindex);
    if ~isempty(cbar)
        legdata=setdiff(legdata,cbar);
    end
    %turn if off:
    colorbar('off','peer',handles.activegraph);
    udata.format.colorbar=0;
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
set(handles.fakerax(handles.activeindex),'userdata',legdata);

ravecreatehyperslice(src,[]);







