function raveupdatehyperslicexdials(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
getudata;
ai=handles.activeindex;
xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;


%prepare data for gridding:
datatypes=handles.datatype{d}(xcol) ;
totalrows=zeros(size(datatypes));
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

%
% set(handles.profilers.xhair{ai},{'xdata'},[xdials(xcol) xdials(xcol)])
% set(handles.profilers.yhair{ai},{'ydata'},[xdials(ycol) xdials(ycol)])

for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end



%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);

constraints=udata.view.constraints(udata.view.constraints~=0);
if ~isempty(constraints)
    alltheconstraints=cell(numel(xcol),numel(xcol),numel(constraints));
end

for r=numel(ycol):-1:1,
    for c=numel(ycol):-1:1,
        if r==c;continue;end
        
        
        thisxdata=xsteps{c};
        thisydata=xsteps{r};
        
        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);
        
        xcalldata=xgrid(:);
        ycalldata=ygrid(:);
        
        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;
        
        %evaluate the functions:
        
        zvalues=raveevaluateregression(src,d,udata.view.z,constants);
        
        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
        
        for q=1:numel(constraints)
            constraintvalues=raveevaluateconstraint(src,constraints(q),constants,false(1,size(constants,2)));
            alltheconstraints{r,c,q}=reshape(constraintvalues,numel(thisydata),numel(thisxdata));
        end
    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=zdataz(~cellfun(@isempty,zdataz));


lowestvalue=min(zdataz{1}(:));
highestvalue=max(zdataz{1}(:));
for b=2:numel(zdataz)
    lowestvalue=min(lowestvalue,min(zdataz{b}(:)));
    highestvalue=max(highestvalue,max(zdataz{b}(:)));
end

%check against existing limits:
if lowestvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=lowestvalue;end
if highestvalue<handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=highestvalue;end
guidata(src,handles);

udata.view.clim=[lowestvalue highestvalue];
set(handles.fakeax(handles.activeindex),'userdata',udata);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;
% 
% zplotdata=allthedata{3};
% if ~udata.format.reversecolors
%     zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
% else
%     zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
% end

ax=handles.subax{handles.activeindex};
fakeax=handles.subfakeax{ai};

for r=numel(xcol):-1:1
    for c=numel(xcol):-1:1
         if r==c %make prediction profiler plot:
            thisxdata=xsteps{c};
            constants=repmat(xdials,[length(thisxdata),1]);
            constants(:,xcol(c))=thisxdata;
            vvalues=raveevaluateregression(src,d,udata.view.z,constants);
            %normalize vvalues to fit scale of box:
            vvalues=((vvalues-lowestvalue)./(highestvalue-lowestvalue))*(max(xsteps{r})-min(xsteps{r}))+min(xsteps{r});

          %  cla(ax(r,c));
          %  set(ax(r,c),'xlim',single([xmin(xcol(c)) xmax(xcol(c))]),'ylim',single([xmin(ycol(r)) xmax(ycol(r))]))
            delete(findobj(fakeax(r,c),'tag','junkk'));
            line(thisxdata,vvalues,'parent',fakeax(r,c),'tag','junkk')
         else
%             cla(ax(r,c));
%             set(ax(r,c),'xlim',single([xmin(xcol(c)) xmax(xcol(c))]),'ylim',single([xmin(ycol(r)) xmax(ycol(r))]))

            zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            % contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            delete(handles.profilers.main{ai}{r,c});
            mainhandles{r,c}=contourplot(C,'style','line','parent',ax(r,c),'linewidth',udata.format.linesize,'hittest','off');


            %draw constraints:
            for q=1:numel(constraints)
                constraintdata=alltheconstraints{r,c,q};
                if ~udata.format.reversecolors
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                else
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=128-(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                end

                C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),constraintdata,[clevelz clevelz]);
                ravedrawconstraintcore(ax(r,c),handles.activeindex,C,constraints(q),xcol(c),xcol(r),1);
            end


            %use single precision axes limits for correct positioning on screen:
            xlimz=single([allthedata{r,c,1}(1,1) allthedata{r,c,1}(1,end)]);
            ylimz=single([allthedata{r,c,2}(1,1) allthedata{r,c,2}(end,1)]);

            set(ax(r,c),'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual','xgrid','off','ygrid','off','layer','top')

            %update tracers
            
            set(handles.profilers.xhair{ai}{r,c},'xdata',[xdials(xcol(c)) xdials(xcol(c))],'ydata',[xmin(xcol(r)) xmax(xcol(r))])%,'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(c),:));
            set(handles.profilers.yhair{ai}{r,c},'xdata',[xmin(xcol(c)) xmax(xcol(c))],'ydata',[xdials(xcol(r)) xdials(xcol(r))])%,'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(r),:));
            

          
        end
        
    end
end



clear contourplot %clear persistent vars;

handles.profilers.main{ai}=mainhandles;
guidata(src,handles); %save now in case an error happens later.


%udpate colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    
    cbar=findobj(legdata,'tag','Colorbar');
    
    ylim=get(cbar,'ylim');
    clim=udata.view.clim;
    
    %determine current c value:
    thisvalue=raveevaluateregression(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});
    
    thiscoord=interp1(clim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
end

