function raveupdatesurfaceprofiler(src,ev)
%update the current surface profiler because the xdials have changed.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

prefs=handles.preferences{d};
targets=handles.targetvalues{d};


xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


xdials=handles.xdials{d}{a};


%get type of each variable:
datatypes=handles.datatype{d}([xcol ycol]);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

numpts=udata.view.numpts;

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));

colz=[xcol ycol];



for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(colz(j));
            totalrows(j)=1;
        case 'continuous'
            %totalrows(j)=numpts+1;
            xsteps{j}=sort((xmin(colz(j)):step(colz(j)):xmax(colz(j))))'; %do not include xdials for surface profiler, resolution isnt good enough to notice.
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{colz(j)})<20
                %totalrows(j)=numel(handles.discretevalues{d}{a}{xcol(j)});
                xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
                totalrows(j)=numel(xsteps{j});
            else
                %totalrows(j)=numpts+1;
                xsteps{j}=sort([xmin(colz(j)):step(colz(j)):xmax(colz(j)) xdials(colz(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            %if numel(handles.discretevalues{d}{a}{xcol(j)})<12
            % totalrows(j)=numel(handles.discretevalues{d}{a}{xcol(j)});
            xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
            totalrows(j)=numel(xsteps{j});
            %else
            %    totalrows(j)=12;
            %end
        case 'string'
            %totalrows(j)=numel(handles.discretevalues{d}{a}{xcol(j)});
            xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            %totalrows(j)=2;
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end


rowz=sum(totalrows);





%evaluation grid will be numel(xsteps{2}) by numel(xsteps{1}) elements.

yevals=repmat(xsteps{2}(:),[numel(xsteps{1}) 1]);
index=repmat(1:numel(xsteps{1}),[numel(xsteps{2}) 1]);
xevals=xsteps{1}(index(:));


%make whole matrix of poitns to eval. this is fixed array of xdials, with xeval/yeval subbed in:

indata=repmat(xdials,[numel(xevals) 1]);
indata(:,xcol)=xevals;
indata(:,ycol)=yevals;


%eval z at indata:
zvalues=raveevaluateregression(src,d,zcol,indata);


%reshape to square matrices:
xdata=reshape(xevals,[numel(xsteps{2}) numel(xsteps{1})]);
ydata=reshape(yevals,[numel(xsteps{2}) numel(xsteps{1})]);
zdata=reshape(zvalues,[numel(xsteps{2}) numel(xsteps{1})]);


if ccol~=0
    c=raveevaluateregression(src,d,ccol,indata);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap-1)+1;
    cdata=reshape(c,[numel(xsteps{2}) numel(xsteps{1})]);

else
    cdata=[0 .5 1]; %non-noisy colors cdata=0.5
end

%if either xcol or ycol is discrete, need to fold surface:
main=findobj(handles.activegraph,'tag','main');



%fold if xvar is discrete:
if  ~any(strcmp(datatypes{1},{'continuous';'constant'})) %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{xcol};
    endind=numel(allowed);

    xvals=xsteps{1}(:); %force to be column.
    xintermediates=xvals+[diff(xvals)./2;0];
    xvals=sort([xvals;xintermediates(1:end-1)]);
    
    xmap=repmat(2:2:2*(endind-1),2,1);
    xmap=[1 xmap(:)' xmap(end)+1];
    
    zmap=repmat(1:numel(xmap)/2,2,1);
    zmap=zmap(:)';
    
    xdata=repmat(xvals(xmap)',size(xdata,1),1);
    ydata=repmat(ydata(:,1),1,numel(xmap));
    zdata=zdata(:,zmap);
    cdata=cdata(:,zmap);   
end

%fold if yvar is discrete:
if  ~any(strcmp(datatypes{2},{'continuous';'constant'})) %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{ycol};
    endind=numel(allowed);

    yvals=xsteps{2}(:); %force to be column.
    yintermediates=yvals+[diff(yvals)./2;0];
    yvals=sort([yvals;yintermediates(1:end-1)]);

    ymap=repmat(2:2:2*(endind-1),2,1);
    ymap=[1 ymap(:)' ymap(end)+1];
    
    zmap=repmat(1:numel(ymap)/2,2,1);
    zmap=zmap(:)';
    
    ydata=repmat(yvals(ymap),1,size(ydata,2));
    xdata=repmat(xdata(1,:),numel(ymap),1);
    zdata=zdata(zmap,:);
    cdata=cdata(zmap,:);   
end






 set(main,'xdata',xdata,'ydata',ydata,'zdata',zdata,'cdata',cdata);

