function raveupdategreatness(src,eventdata)
%this is the buttondownfcn for derivative profiler axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%need to check if this graph is active. if not , return and do nothing.
thisindex=get(src,'userdata');
if thisindex~=handles.activeindex
    return
end

%if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
%changes and such:

if strcmp(handles.activetab,'format')
    raveselectsubgraph(src,eventdata)
    return
end

numy=numel(udata.view.y);
numx=numel(udata.view.x);


%determine which callback to use based on which type of graph was clicked:
deezhandles=[handles.subax{handles.activeindex} handles.subfakeax{handles.activeindex} handles.subfakerax{handles.activeindex}] ;

xyhandles=deezhandles(1:numy*numx,:);
xxhandles=deezhandles(numy*numx+1:numy*numx+((numx)^2-numx)/2,:);
yyhandles=deezhandles(numy*numx+1+(numx^2-numx)/2:end,:);

if ismember(src,xyhandles)
    raveslidexy(handles.fig,[]);
    set(handles.fig,'windowbuttonmotionfcn',@raveslidexy)
    set(handles.fig,'windowbuttonupfcn',@ravegreatnessdone)
    
    
elseif ismember(src,xxhandles)
    raveslidexx(handles.fig,[]);
    set(handles.fig,'windowbuttonmotionfcn',@raveslidexx)
    set(handles.fig,'windowbuttonupfcn',@ravegreatnessdone)
    
    
elseif ismember(src,yyhandles)
    raveslideyy(handles.fig,[]);
    set(handles.fig,'windowbuttonmotionfcn',@raveslideyy)
    set(handles.fig,'windowbuttonupfcn',@ravegreatnessdone)
    
end



return






function raveslidexy(src,eventdata)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

numx=numel(udata.view.x);
numy=numel(udata.view.y);



xmin=min(handles.alldata{udata.view.dataset}(:,:));
xmax=max(handles.alldata{udata.view.dataset}(:,:));

%turn these subax back into a matrix:
subax=handles.subax{handles.activeindex}(1:numx*numy);
subax=reshape(subax,numx,numy);
subax=subax';

%find this plot's location in the matrix:



[crap,index]=find(subax==gca); %index is column that was clicked.

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
val=clicked(1,1);



%move marker within bounds
if val>xmax(udata.view.x(index))
    val=xmax(udata.view.x(index));
elseif val<xmin(udata.view.x(index))
    val=xmin(udata.view.x(index));
end
set(handles.profilers.xhair{handles.activeindex}{1}(:,index),'xdata',[val val])

xcol=udata.view.x;
ycol=udata.view.y;
d=udata.view.dataset;
a=udata.view.analysis;

%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(index)};


if  strcmp(thistype,'constant')
    val=handles.minlimit{d}{a}{xcol(index)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(index)};
    val=allowed(abs(allowed-val)==min(abs(allowed-val)));
end

%make text label:
if ~any(strcmp(thistype,{'string','logical'}))
    valtext=strtrim(sprintf('%5.5g',val));
elseif strcmp(thistype,'string')
    valtext=handles.discretevalues{d}{a}{xcol(index)}{val};
else %logical
    if val
        valtext=handles.prefs.logicaltrue;
    else
        valtext=handles.prefs.logicalfalse;
    end
end




%set xlabel:
subfakeax=handles.subfakeax{handles.activeindex}(1:numx*numy);
subfakeax=reshape(subfakeax,numx,numy);
subfakeax=subfakeax';
set(get(subfakeax(end,index),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');


%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:
xdials=handles.xdials{d}{a};

if xdials(xcol(index))==val;
    return
end

xdials(xcol(index))=val;

handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);


%calculate profiler data for each graph:
cols=length(udata.view.x);
rows=length(udata.view.y);
numpts=udata.view.numpts;



%get type of each variable:
datatypes=handles.datatype{d}(xcol);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));


for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end


rowz=sum(totalrows);

xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:cols
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
    
else
    for k=1:cols
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:
yvalues=raveevaluateregression(src,d,ycol,xfixed);
ycurrent=raveevaluateregression(src,d,ycol,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));
ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%update current-value texts for responses:
for tt=1:rows
    %set y label:
    g=udata.view.ylabels(tt);
    
    set(g,'string',{['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{tt}))];'';''});
    
    
    
    for cc=1:cols
        if  ~any(strcmp(datatypes{cc},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(cc)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{cc}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            
            set(handles.profilers.yhair{handles.activeindex}{1}(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
            set(handles.profilers.main{handles.activeindex}{1}(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap))
            
        else
            
            set(handles.profilers.yhair{handles.activeindex}{1}(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
            set(handles.profilers.main{handles.activeindex}{1}(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc})
            
        end
    end
end

%UPDATE response profilers:
ylimmin=min(handles.alldata{d}(:,ycol));
ylimmax=max(handles.alldata{d}(:,ycol));

for r=1:1:(numy-1),
    %for c=(numx-1):-1:r,
    for c=r+1:1:(numy)
%         h.yyxhair(r,c-1)=line([ycurrent{c} ycurrent{c}],[ylimmin(r) ylimmax(r)],'visible','on','linestyle',':','color',thisycolor,...
%             'parent',yyax(r,c-1),'hittest','off','tag','hair');
        
        set(handles.profilers.xhair{handles.activeindex}{3}(r,c-1),'xdata',[ycurrent{c} ycurrent{c}],'ydata',[ylimmin(r) ylimmax(r)]);
        
        %response hairline series:
%         h.yyyhair(r,c)=line([ylimmin(c) ylimmax(c)],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',':','color',thisycolor,...
%             'parent',yyax(r,c),'hittest','off','tag','hair');
%         
        set(handles.profilers.yhair{handles.activeindex}{3}(r,c-1),'xdata',[ylimmin(c) ylimmax(c)],'ydata',[ycurrent{r} ycurrent{r}]);
        
        
        for x=1:numx
            
            set(handles.profilers.main{handles.activeindex}{3}{r,c-1}(x),'xdata',yvalues{c}{x},'ydata',yvalues{r}{x})

        end
        
        
    end
end


%UPDATE CONTOUR PLOTS:
try %avoid erors when clicking first or last column:
    set(handles.profilers.xhair{handles.activeindex}{2}(1:index-1,index-1),'xdata',[val val])
end

try
    set(handles.profilers.yhair{handles.activeindex}{2}(index,index:end),'ydata',[val val])
end


%prepare data for gridding:
datatypes=handles.datatype{d}(xcol);
totalrows=zeros(size(datatypes));
numpts=20;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};


for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end



%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);
for r=1:1:(numx-1)
    for c=(r+1):1:(numx)
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluateregression(src,d,udata.view.z,constants);

        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=cell2mat(zdataz);
zdataz=zdataz(:);

lowestvalue=min(zdataz);
highestvalue=max(zdataz);

levelz=linspace(1,128,20)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;


idx=tril(true(numx-1));
subax=zeros(numx-1);
subax(idx)=handles.subax{handles.activeindex}(numy*numx+1:numy*numx+((numx)^2-numx)/2,:);
subax=subax';
for r=1:1:(numx-1)
    for c=(r+1):1:(numx)
        %if r==c;continue;end
        zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            
            delete(cell2mat(handles.profilers.main{handles.activeindex}{2}(r,c)))
            contourplot(C,handles.profilers.main{handles.activeindex}{2}{r,c},'update');
            handles.profilers.main{handles.activeindex}{2}{r,c}=contourplot(C,'style','line','parent',subax(r,c-1),'linewidth',udata.format.linesize,'hittest','off');
            uistack(handles.profilers.main{handles.activeindex}{2}{r,c},'bottom');

 
    end
end


guidata(src,handles);

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials')
end








function raveslidexx(src,ev)

function raveslideyy(src,ev)


function ravegreatnessdone(src,ev) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','')
set(handles.fig,'windowbuttonupfcn','')

%update other analyses to reflect new values:

raveupdateanalysis(src,ev,'xdials')



