function ravecreateonevarsweep(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z; %"sweep through values of"
ccol=udata.view.c; %"draw a line for each value of"

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontsize',14,'tag','title')

sweepcol=[zcol];


[sweepvalues,totalrows]=ravecalculatexsteps(handles.fig,d,a,sweepcol,udata.view.numpts);

sweepvalues=sweepvalues{1};


if ccol~=zcol && ccol~=0 && ~isempty(udata.view.steps)   
    zcalldata=repmat(sweepvalues(:),numel(udata.view.steps),1);
    stepdata=repmat(udata.view.steps(:)',numel(sweepvalues),1);
    stepdata=stepdata(:);
    
else
    zcalldata=sweepvalues(:);
    stepdata=[];
end


constants=repmat(handles.xdials{d}{a},[length(zcalldata),1]);
constants(:,zcol)=zcalldata;
if ~isempty(stepdata)
constants(:,ccol)=stepdata;
end

%evaluate the functions:

outvalues=raveevaluateregression(src,d,[udata.view.x udata.view.y],constants);
xvalues=outvalues(:,1);
yvalues=outvalues(:,2);

constants(:,xcol)=xvalues;
constants(:,ycol)=yvalues;

ncurves=numel(zcalldata)./numel(sweepvalues);
npts=numel(sweepvalues);






for n=1:ncurves %#ok<BDSCI>
   theserows=constants((n-1)*npts+1:n*npts,:);
   line(theserows(:,xcol),theserows(:,ycol),'parent',targetaxes)
end





xlim = get(targetaxes,'xlim');
ylim = get(targetaxes,'ylim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',ylim);
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',ylim);





%% make tick marks:

labelsize=12;
fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter','none','fontname','arial')
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)

set(targetaxes,'hittest','on','buttondownfcn',@raveupdatecontour);

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar
ravemakecolorbar(handles.fig,[]);
%end

ravemaketextticks(targetaxes,[],xcol(1),ycol,[],[])

raveresizeaxes(src,[],targetindex)


%draw constraints:

constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end







