function viewtabline(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:

%inputs=get(handles.fakeax(handles.activeindex),'userdata');
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.77*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.29*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
xaxistext=text(.25-.10317,.74*1.0317-.0317,'X-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
yaxistext=text(.25-.10317,.67*1.0317-.0317,'Y-Axis Variable(s):','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
consttext=text(.25-.10317,.36*1.0317-.0317,'Draw Lines of Constant:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',view.dataset);
xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+420 180 30],...
    'style','popupmenu','string',handles.allnames{view.dataset},'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu');
yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+309 180 96],...
    'style','listbox','string',handles.allnames{view.dataset},'callback',@yaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',1000);
zaxischeck=uicontrol('position',[origin(1)+33 origin(2)+283 200 22],...
    'style','checkbox','string',' ',...
    'backgroundcolor',handles.colors.tabbg,'value',0,'callback',@zaxischeck_callback,'tag','zaxischeck','enable','off');

zaxismenu=uicontrol('position',[origin(1)+55 origin(2)+230 180 70],...
    'style','listbox','string',handles.allnames{view.dataset},'callback',@zaxismenu_callback,'tag','zaxismenu','max',1000,'enable','off');

constmenu=uicontrol('position',[origin(1)+55 origin(2)+92 180 110],...
    'style','popupmenu','string',[handles.allnames{view.dataset}(1:end),cellstr('')],'callback',@constmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','constmenu','value',length(handles.allnames{view.dataset}));

lefttext=text(.26-.10317,.57*1.0317-.0317,'Left Axis','horizontalalignment','left',...
    'backgroundcolor','none',...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','normal','rotation',90);
righttext=text(.26-.10317,.42*1.0317-.0317,'Right','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','normal','rotation',90);

%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+30 180 24],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);
changeunitsbutton=uicontrol('position',[origin(1)+55 origin(2)+2 180 24],...
    'style','pushbutton','string','Change Units','backgroundcolor',handles.colors.buttonbg);



usevariablecolorscheck=uicontrol('position',[origin(1)+55 origin(2)+135 200 26],...
    'style','checkbox','string','Use fixed variable colors','value',view.usevariablecolors,...
    'backgroundcolor',handles.colors.tabbg,'tag','usevariablecolorscheck','callback',@usevariablecolorscheck_callback);
autoscalecheck=uicontrol('position',[origin(1)+55 origin(2)+115 200 26],...
    'style','checkbox','string','Autoscale Y-axis data',...
    'backgroundcolor',handles.colors.tabbg,'value',view.autoscale,'callback',@autoscalecheck_callback,'tag','autoscalecheck');
useresponsedata=uicontrol('position',[origin(1)+55 origin(2)+95 200 26],...
    'style','checkbox','string','Limit Y-axis to Response Variables',...
    'backgroundcolor',handles.colors.tabbg,'value',view.response,'callback',@useresponsedata_callback,'tag','useresponsedata');
sortdatacheck=uicontrol('position',[origin(1)+55 origin(2)+75 200 26],...
    'style','checkbox','string','Sort Data by X-Axis Variable',...
    'backgroundcolor',handles.colors.tabbg,'value',0,'tag','sortdatacheck');
excludecheck=uicontrol('position',[origin(1)+55 origin(2)+55 200 26],...
    'style','checkbox','string','Hide constant columns in menus',...
    'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');




%% group items and finish up
handles.tabobjects=[datasettext xaxistext yaxistext consttext datasetmenu xaxismenu yaxismenu constmenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext useresponsedata,...
    changeunitsbutton sortdatacheck excludecheck zaxismenu zaxischeck lefttext righttext autoscalecheck usevariablecolorscheck];

handles.activetab='view';

guidata(src,handles);
% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:

function viewtabmenufilter(src,eventdata)

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%get old strings/values
oldxvalue=view.x;
oldyvalue=view.y;
oldcvalue=view.c;


d=view.dataset;

%if get(useresponsedata,'value') && get(excludecheck,'value') && ~all(handles.isresponse{d}==0)
if view.response && view.exclude && ~all(handles.isresponse{d}==0)
    %filter menus by responses and hide constant cols

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d} & ~handles.isresponse{d});
    newxvalue=find(strcmp(handles.allnames{d}(oldxvalue),newxstring));
    if isempty(newxvalue)
        newxvalue=1;
    end

    newystring=handles.allnames{d}(~handles.isconstant{d} & handles.isresponse{d});
    newyvalue=find(ismember(newystring,handles.allnames{d}(oldyvalue)));
    if isempty(newyvalue)
        newyvalue=1;
    end

    constcols=setdiff(1:length(newxstring),newxvalue); %unselected x columns. 
    newcstring=[newxstring(constcols),cellstr('')]; 
    %newcvalue=find(strcmp(oldcstring(oldcvalue),newcstring));
    if oldcvalue~=0 %something selected
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring); %last entry in c string (='')
    end
    
    if isempty(newcvalue)
        newcvalue=1;
    end
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d} & ~handles.isresponse{d}));
    handles.visibleycols=find((~handles.isconstant{d} & handles.isresponse{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];

    %reset menu strings/values to match displayed data.
    set(yaxismenu,'string',newystring,'value',newyvalue);
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

%elseif get(useresponsedata,'value') && ~all(handles.isresponse{d}==0)
elseif view.response && ~all(handles.isresponse{d}==0)    
    %filter responses only

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isresponse{d});
    newxvalue=find(strcmp(handles.allnames{d}(oldxvalue),newxstring));
    if isempty(newxvalue)
        newxvalue=1;
    end

    newystring=handles.allnames{d}(handles.isresponse{d});
    newyvalue=find(ismember(newystring,handles.allnames{d}(oldyvalue)));

    constcols=setdiff(1:length(newxstring),newxvalue);
    newcstring=[newxstring(constcols),cellstr('')];
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isresponse{d}));
    handles.visibleycols=find((handles.isresponse{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];
    
    %reset menu strings/values to match displayed data.
    set(yaxismenu,'string',newystring,'value',newyvalue);
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

%elseif get(excludecheck,'value') %hide constant cols only
elseif view.exclude %hide constant cols only

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d});
    newxvalue=find(strcmp(handles.allnames{d}(oldxvalue),newxstring));
    if isempty(newxvalue)
        newxvalue=1;
    end

    newystring=handles.allnames{d}(~handles.isconstant{d});
    newyvalue=find(ismember(newystring,handles.allnames{d}(oldyvalue)));
    constcols=setdiff(1:length(newxstring),newxvalue);
    newcstring=[newxstring(constcols),cellstr('')];
    %newcvalue=find(strcmp(oldcstring(oldcvalue),newcstring));
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d}));
    handles.visibleycols=find((~handles.isconstant{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];
    
    %reset menu strings/values to match displayed data.
    set(yaxismenu,'string',newystring,'value',newyvalue);
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

else %show all variables

    %get new strings/values
    newxstring=handles.allnames{d};
    newxvalue=find(strcmp(handles.allnames{d}(oldxvalue),newxstring));
    if isempty(newxvalue)
        newxvalue=1;
    end

    newystring=handles.allnames{d};
    newyvalue=find(ismember(newystring,handles.allnames{d}(oldyvalue)));
    constcols=setdiff(1:length(newxstring),newxvalue);
    newcstring=[newxstring(constcols),cellstr('')];
    %newcvalue=find(strcmp(oldcstring(oldcvalue),newcstring));
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end   
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=1:1:length(handles.allnames{d});
    handles.visibleycols=1:1:length(handles.allnames{d});
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];

    %reset menu strings/values to match displayed data.
    set(yaxismenu,'string',newystring,'value',newyvalue);
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

end

guidata(src,handles)


%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%update other menus to show correct variables
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

d=get(datasetmenu,'value');

if d==udata.view.dataset;
    return
end


%get initial columns for this data set:
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),1);
    y=find((~handles.isconstant{d} & handles.isresponse{d}),1);
    c=0;
else
    x=find((~handles.isconstant{d}),1);
    y=find((~handles.isconstant{d}),1);
    c=0;

end



udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=x;
udata.view.y=y;
udata.view.c=c;
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);


handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

xaxismenustring=get(xaxismenu,'string');

constcols=setdiff(1:length(xaxismenustring),get(src,'value'));
newcstring=[xaxismenustring(constcols);cellstr('')];

set(constmenu,'string',newcstring);

handles.visibleccols=[find(ismember(handles.allnames{get(datasetmenu,'value')},newcstring)) 0];

%if selected isconstant, turn off filter constant button
if any(handles.isconstant{get(datasetmenu,'value')}(handles.visiblexcols(get(src,'value'))))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.c=handles.visibleccols(get(constmenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value


set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yaxismenu_callback(src,eventdata)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected ~responses, turn off filter response button
if ~all(handles.isresponse{get(datasetmenu,'value')}(handles.visibleycols(get(src,'value'))))
    set(useresponsedata,'enable','off')  
else
    set(useresponsedata,'enable','on')    
end

%if selected isconstant, turn off filter constant button
if any(handles.isconstant{get(datasetmenu,'value')}(handles.visibleycols(get(src,'value'))))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visibleycols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(constmenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zaxismenu_callback(src,eventdata)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function constmenu_callback(src,eventdata)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected isconstant, turn off filter constant button
if get(src,'value')~=numel(get(src,'string'))
    if any(handles.isconstant{get(datasetmenu,'value')}(handles.visibleccols(get(src,'value'))))
        set(excludecheck,'enable','off')
    else
        set(excludecheck,'enable','on')
    end
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visibleycols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(constmenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function useresponsedata_callback(src,eventdata)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visibleycols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(constmenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function excludecheck_callback(src,eventdata)

handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
constmenu=findobj(handles.tabobjects,'tag','constmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visibleycols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(constmenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

function zaxischeck_callback(src,ev)
handles=guidata(src);
zaxismenu=findobj(handles.tabobjects,'tag','zaxismenu');

if get(src,'value')
    set(zaxismenu,'enable','on','backgroundcolor',[1 1 1])
else
    set(zaxismenu,'enable','off','backgroundcolor',[.75 .75 .75])
end

function usevariablecolorscheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.usevariablecolors=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);



function autoscalecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.autoscale=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);




