function viewtabscatter(src,eventdata) %#ok<INUSD>
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.
d=udata.view.dataset;

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.35*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
xaxistext=text(.25-.10317,.72*1.0317-.0317,'X-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
yaxistext=text(.25-.10317,.65*1.0317-.0317,'Y-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
colortext=text(.25-.10317,.58*1.0317-.0317,'Color Points by:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
sizetext=text(.25-.10317,.51*1.0317-.0317,'Size Points by:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');


%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',view.dataset);
xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+405 180 30],...
    'style','popupmenu','string',handles.allnames{d},'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu');
yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+360 180 30],...
    'style','popupmenu','string',handles.allnames{d},'callback',@yaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','yaxismenu');
colormenu=uicontrol('position',[origin(1)+55 origin(2)+315 180 30],...
    'style','popupmenu','string',[handles.allnames{d}(1:end),cellstr('<Use Row Colors>')],'callback',@colormenu_callback,...
    'backgroundcolor',[1 1 1],'tag','colormenu','value',length(handles.allnames{1}));
sizemenu=uicontrol('position',[origin(1)+55 origin(2)+270 180 30],...
    'style','popupmenu','string',[handles.allnames{d}(1:end),cellstr('<Uniform Size>')],'callback',@sizemenu_callback,...
    'backgroundcolor',[1 1 1],'tag','sizemenu','value',length(handles.allnames{1}));


%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+165 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);
changeunitsbutton=uicontrol('position',[origin(1)+55 origin(2)+130 180 30],...
    'style','pushbutton','string','Change Units','backgroundcolor',handles.colors.buttonbg);
useresponsedata=uicontrol('position',[origin(1)+55 origin(2)+100 200 30],...
    'style','checkbox','string','Limit Y-axis to Response Variables',...
    'backgroundcolor',handles.colors.tabbg,'value',view.response,'callback',@useresponsedata_callback,'tag','useresponsedata');
sortdatacheck=uicontrol('position',[origin(1)+55 origin(2)+78 200 30],...
    'style','checkbox','string','Sort Data by X-Axis Variable',...
    'backgroundcolor',handles.colors.tabbg,'value',0,'tag','sortdatacheck','visible','off');
excludecheck=uicontrol('position',[origin(1)+55 origin(2)+56 200 30],...
    'style','checkbox','string','Hide constant columns in menus',...
    'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');


%% group items and finish up
handles.tabobjects=[datasettext xaxistext yaxistext colortext datasetmenu xaxismenu yaxismenu colormenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext useresponsedata,...
    changeunitsbutton sortdatacheck excludecheck sizetext sizemenu];

handles.activetab='view';

guidata(src,handles);
% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:

function viewtabmenufilter(src,eventdata) %#ok<INUSD>

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

% datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
colormenu=findobj(handles.tabobjects,'tag','colormenu');
sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
% useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
% excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%get old strings/values
oldxvalue=view.x;
oldyvalue=view.y;
oldcvalue=view.c;
oldsvalue=view.s;


%d=get(datasetmenu,'value');
d=view.dataset;

%if get(useresponsedata,'value') && get(excludecheck,'value') && ~all(handles.isresponse{d}==0)
if view.response && view.exclude && ~all(handles.isresponse{d}==0)
    %filter menus by responses and hide constant cols

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d} & ~handles.isresponse{d});

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d} & ~handles.isresponse{d}));
    %handles.visibleycols=find((~handles.isconstant{d} & handles.isresponse{d}));
    handles.visibleccols=[find((~handles.isconstant{d} & handles.isresponse{d})) 0];


%elseif get(useresponsedata,'value') && ~all(handles.isresponse{d}==0)
elseif view.response && ~all(handles.isresponse{d}==0)    
    %filter responses only

    %get new strings/values
    newxstring=handles.allnames{d}(handles.isresponse{d});

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isresponse{d}));
    %handles.visibleycols=find((handles.isresponse{d}));
    handles.visibleccols=[find((handles.isresponse{d})) 0];
    


%elseif get(excludecheck,'value') %hide constant cols only
elseif view.exclude %hide constant cols only

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d});

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d}));
    handles.visibleycols=find((~handles.isconstant{d}));
    handles.visibleccols=[find((~handles.isconstant{d})) 0];
    


else %show all variables

    %get new strings/values
    newxstring=handles.allnames{d};
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=1:1:length(handles.allnames{d});
    handles.visibleycols=1:1:length(handles.allnames{d});
    handles.visibleccols=[1:1:length(handles.allnames{d}) 0];


end

    newxvalue=find(strcmp(handles.allnames{d}(oldxvalue),newxstring));
    if isempty(newxvalue);newxvalue=1;end
    
    newyvalue=find(ismember(newxstring,handles.allnames{d}(oldyvalue)));
    if isempty(newyvalue);newyvalue=1;end
   
    newcstring=[newxstring,cellstr('<Use Row Colors>')];
    newsstring=[newxstring,cellstr('')];
    
    if oldcvalue~=0 %something selected
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring); %last entry in c string (='')
    end
    
    if isempty(newcvalue);newcvalue=1;end
    
    if oldsvalue~=0 %something selected
        newsvalue=find(strcmp(handles.allnames{d}(oldsvalue),newcstring));
    else %'none selected'
        newsvalue=numel(newcstring); %last entry in c string (='')
    end
    
    if isempty(newsvalue);newsvalue=1;end   

    %reset menu strings/values to match displayed data.
    set(yaxismenu,'string',newxstring,'value',newyvalue);
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(colormenu,'string',newcstring,'value',newcvalue);
    set(sizemenu,'string',newsstring,'value',newsvalue);



guidata(src,handles)


%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=get(src,'value');

%get initial columns for this data set:
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),1);
    y=find((~handles.isconstant{d} & handles.isresponse{d}),1);
    c=0;
    s=0;
else
    x=find((~handles.isconstant{d}),1);
    y=find((~handles.isconstant{d}),1);
    c=0;
    s=0;
end

udata.view.dataset=d;
udata.view.x=x;
udata.view.y=y;
udata.view.c=c;
udata.view.s=s;

set(handles.fakeax(handles.activeindex),'userdata',udata);


handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);
viewtabmenufilter(src,[])
ravecreatescatter(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected ~response, turn off filter response button
if ~handles.isresponse{udata.view.dataset}(handles.visiblexcols(get(src,'value')))
    set(useresponsedata,'enable','off')  
else
    set(useresponsedata,'enable','on')    
end

%if selected isconstant, turn off filter constant button
if handles.isconstant{udata.view.dataset}(handles.visiblexcols(get(src,'value')))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata

udata.view.x=handles.visiblexcols(get(src,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yaxismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected ~response, turn off filter response button
if ~handles.isresponse{udata.view.dataset}(handles.visibleycols(get(src,'value')))
    set(useresponsedata,'enable','off')  
else
    set(useresponsedata,'enable','on')    
end

%if selected isconstant, turn off filter constant button
if handles.isconstant{udata.view.dataset}(handles.visibleycols(get(src,'value')))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata

udata.view.y=handles.visiblexcols(get(src,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);
    
ravecreatescatter(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colormenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

colormenu=findobj(handles.tabobjects,'tag','colormenu');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected isconstant, turn off filter constant button
if get(src,'value')~=numel(get(src,'string'))
    if any(handles.isconstant{udata.view.dataset}(handles.visibleccols(get(src,'value'))))
        set(excludecheck,'enable','off')
    else
        set(excludecheck,'enable','on')
    end
end

guidata(src,handles);

udata.view.c=handles.visibleccols(get(colormenu,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizemenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected isconstant, turn off filter constant button
if get(src,'value')~=numel(get(src,'string'))
    if any(handles.isconstant{udata.view.dataset}(handles.visibleccols(get(src,'value'))))
        set(excludecheck,'enable','off')
    else
        set(excludecheck,'enable','on')
    end
end

guidata(src,handles);

%set fakeax userdata

udata.view.s=handles.visibleccols(get(src,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function useresponsedata_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%set fakeax userdata
udata.view.response=get(src,'value'); %response check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);
viewtabmenufilter(src,[])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function excludecheck_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.exclude=get(src,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])




