function viewtabscattermatrix(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);   
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.


%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.19*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels

analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

layouttext=text(.25-.10317,.72*1.0317-.0317,'Layout:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

switch udata.view.layout

    case 'rectangle'
        xaxistext=text(.25-.10317,.64*1.0317-.0317,'Choose X Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
            'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

        xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+295 180 90],...
            'style','listbox','string',handles.allnames{udata.view.dataset},'value',[1 2],...
            'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback);
        
        yaxistext=text(.25-.10317,.45*1.0317-.0317,'Choose Y Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
            'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

        yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+172 180 90],...
            'style','listbox','string',handles.allnames{udata.view.dataset},'value',[1 2],...
            'backgroundcolor',[1 1 1],'tag','yaxismenu','max',50,'callback',@yaxismenu_callback); 
        
        caxistext=text(.25-.10317,.27*1.0317-.0317,'Color Points by:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
            'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
            
        caxismenu=uicontrol('position',[origin(1)+55 origin(2)+115 180 30],...
            'style','popupmenu','string',[cellstr(''),handles.allnames{udata.view.dataset}(1:end)],...
            'backgroundcolor',[1 1 1],'tag','caxismenu','callback',@caxismenu_callback,'value',udata.view.c+1);

        layouthandles=[xaxistext xaxismenu yaxistext yaxismenu caxistext caxismenu];
    
    
    
    otherwise
        xaxistext=text(.25-.10317,.64*1.0317-.0317,'Choose Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
            'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

        xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+185 180 200],...
            'style','listbox','string',handles.allnames{udata.view.dataset},'value',[1 2],...
            'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback);
        
        caxistext=text(.25-.10317,.29*1.0317-.0317,'Color Points by:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
            'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
            
        caxismenu=uicontrol('position',[origin(1)+55 origin(2)+125 180 30],...
            'style','popupmenu','string',[cellstr(''),handles.allnames{udata.view.dataset}(1:end)],...
            'backgroundcolor',[1 1 1],'tag','caxismenu','callback',@caxismenu_callback,'value',udata.view.c+1);

        layouthandles=[xaxistext xaxismenu caxistext caxismenu];
end


%% variable lists

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',udata.view.analysis);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',udata.view.dataset);

layoutvalue=find(ismember({'square';'upper';'lower';'rectangle'},udata.view.layout));


layoutmenu=uicontrol('position',[origin(1)+55 origin(2)+405 180 30],...
    'style','popupmenu','string',{'Square';'Upper Triangle';'Lower Triangle';'Rectangle'},...
    'backgroundcolor',[1 1 1],'tag','layoutmenu','callback',@layoutmenu_callback,'value',layoutvalue);



%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+55 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);

useresponsedata=uicontrol('position',[origin(1)+55 origin(2)+25 200 30],...
    'style','checkbox','string','Limit Menu to Response Variables','backgroundcolor',handles.colors.tabbg,...
    'tag','useresponsedata','callback',@useresponsedata_callback);
excludecheck=uicontrol('position',[origin(1)+55 origin(2)+1 200 30],...
    'style','checkbox','string','Hide constant columns in menus',...
    'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');

%% group items and finish up
handles.tabobjects=[datasettext datasetmenu layouttext layoutmenu layouthandles,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext excludecheck useresponsedata];

handles.activetab='view';

guidata(src,handles);

% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:

function viewtabmenufilter(src,eventdata)

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');

%get old strings/values
oldxvalue=view.x;

d=udata.view.dataset;

%if get(useresponsedata,'value') && get(excludecheck,'value') && ~all(handles.isresponse{d}==0)
if view.response && view.exclude && ~all(handles.isresponse{d}==0)
    %filter menus by responses and hide constant cols

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d} & ~handles.isresponse{d}); %list of variable names with the constant ones removed.
    newxvalue=zeros(1,length(oldxvalue));
    for jj=1:length(oldxvalue)
        newxvalue(jj)=find(strcmp(handles.allnames{d}(oldxvalue(jj)),newxstring));
    end
    
    if isempty(newxvalue)
        newxvalue=1;
    end

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d} & ~handles.isresponse{d}));

    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);

%elseif get(useresponsedata,'value') && ~all(handles.isresponse{d}==0)
elseif view.response && ~all(handles.isresponse{d}==0)    
    %filter responses only

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isresponse{d}); %list of variable names with the constant ones removed.
    newxvalue=zeros(1,length(oldxvalue));
    for jj=1:length(oldxvalue)
        newxvalue(jj)=find(strcmp(handles.allnames{d}(oldxvalue(jj)),newxstring));
    end
    
    if isempty(newxvalue)
        newxvalue=1;
    end
  
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isresponse{d}));
   
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);

%elseif get(excludecheck,'value') %hide constant cols only
elseif view.exclude %hide constant cols only

    %get new strings/values
    newxstring=handles.allnames{d}(~handles.isconstant{d}); %list of variable names with the constant ones removed.
    newxvalue=zeros(1,length(oldxvalue));
    for jj=1:length(oldxvalue)
        newxvalue(jj)=find(strcmp(handles.allnames{d}(oldxvalue(jj)),newxstring));
    end
    
    if isempty(newxvalue)
        newxvalue=1;
    end
      
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d}));
    
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);


else %show all variables

    %get new strings/values
    newxstring=handles.allnames{d}; %list of variable names with the constant ones removed.
    newxvalue=zeros(1,length(oldxvalue));
    for jj=1:length(oldxvalue)
        newxvalue(jj)=find(strcmp(handles.allnames{d}(oldxvalue(jj)),newxstring));
    end
    
    if isempty(newxvalue)
        newxvalue=1;
    end

      
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=1:1:length(handles.allnames{d});

    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);


end

guidata(src,handles)

%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
%update other menus to show correct variables

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

d=get(datasetmenu,'value');

%get initial columns for this data set: (need at least 2 x's for a
%scatterplot matrix)
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),2);
else
    x=find((~handles.isconstant{d}),2);
end

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.dataset=get(datasetmenu,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

%update plots
% 
% inputs.targetaxes=handles.activegraph;
% inputs.x=x;
% inputs.pscale=[]; %project scale factors
% inputs.gscale=[]; %graph scale factors
% inputs.analysis=get(analysismenu,'value');
% inputs.dataset=get(datasetmenu,'value');

ravecreatescattermatrix(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

ravecreatescattermatrix(src,[]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

xaxismenustring=get(xaxismenu,'string');

%constcols=setdiff(1:length(xaxismenustring),get(src,'value'));
constcols=setdiff(1:length(xaxismenustring),union(get(yaxismenu,'value'),get(xaxismenu,'value')));
newcstring=[xaxismenustring(constcols);cellstr('')];

handles.visibleccols=[find(ismember(handles.allnames{get(datasetmenu,'value')},newcstring)) length(handles.allnames{get(datasetmenu,'value')})+1];


%if selected isconstant, turn off filter constant button
if any(handles.isconstant{get(datasetmenu,'value')}(handles.visiblexcols(get(src,'value'))))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

%ensure at least 2 vars are selected:
% if length(udata.view.x)==1
%     udata.view.x=[1 2]; %select first 2 vars
%     set(handles.infobar,'string','You need to select at least two variables for this graph.')
% else
%     set(handles.infobar,'string','');
% end


set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescattermatrix(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function useresponsedata_callback(src,eventdata)
handles=guidata(src);

useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.response=get(useresponsedata,'value'); %response check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

function excludecheck_callback(src,eventdata)

handles=guidata(src);

excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

function layoutmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

optionz={'square';'upper';'lower';'rectangle'};

udata.view.layout=optionz{get(src,'value')};

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescattermatrix(src,[]);

viewtab(src,[]);

function yaxismenu_callback(src,ev)
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

yaxismenustring=get(yaxismenu,'string');

constcols=setdiff(1:length(yaxismenustring),union(get(yaxismenu,'value'),get(xaxismenu,'value')));
newcstring=[yaxismenustring(constcols);cellstr('')];

handles.visibleccols=[find(ismember(handles.allnames{get(datasetmenu,'value')},newcstring)) length(handles.allnames{get(datasetmenu,'value')})+1];


%if selected isconstant, turn off filter constant button
if any(handles.isconstant{get(datasetmenu,'value')}(handles.visiblexcols(get(src,'value'))))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.y=handles.visiblexcols(get(yaxismenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

%ensure at least 2 vars are selected:
% if length(udata.view.x)==1
%     udata.view.x=[1 2]; %select first 2 vars
%     set(handles.infobar,'string','You need to select at least two variables for this graph.')
% else
%     set(handles.infobar,'string','');
% end


set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescattermatrix(src,[]);

function caxismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.c=get(src,'value')-1;

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreatescattermatrix(src,[]);

