function ravecreatehistogram(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex)

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
xcol=udata.view.x;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
a=udata.view.analysis; 
d=udata.view.dataset; %data set index



%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');
delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})
delete(handles.statsax{handles.activeindex})


%% make the graph.

%how many graphs to draw?
numvars=length(xcol);
set(targetaxes,'visible','off','xticklabel','','yticklabel','');
% set(targetaxes,'Visible','off','color','none','xcolor',handles.colors.wsbg,'ycolor',handles.colors.wsbg,'zcolor',handles.colors.wsbg,'box','off',...
%     'xticklabel','','yticklabel','')


%% %% %%

% doshowstatistics=0;

if udata.view.showstats
    margin=150;
else
    margin=20;
end

pos = get(targetaxes,'Position');%+[30 30 -30 -30]; %shrink a little to fit tick labels.

xlabelspace=20;% pixels to move up axes to make room for labels at the bottom.
ylabelspace=32;% pixels to allow between graphs to make room for axes labels. (note scattermatrix doesnt have this problem.)
%note y labelspace used for horizontal stack, xlabelspace used for vertical stack.

if  pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
    
    width = (pos(3)-ylabelspace*(numvars-1)-10)/numvars;
    height = pos(4)-xlabelspace-margin;
    space = .00; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];

else %vertical stacking
    width=pos(3)-ylabelspace-margin;
    height=pos(4)/numvars-xlabelspace;
    space = .03; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
end


if width<3;width=3;end
if height<3;height=3;end


BigAxParent = get(targetaxes,'Parent');

nothidden=handles.isvisible{d}{a};
isselected=handles.isselected{d}{a};


%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

for c=1:numvars
    x=handles.alldata{d}(:,xcol(c));
    %nbars=min(udata.view.nbars-1,numel(unique(x))); %dont make more bars than there are unique values.
    nbars=udata.view.nbars-1;

    if min(x)==max(x) %x is constant
        edges=[x(1)-1 x(1)+1];
    elseif ~isempty(x)
        edges=linspace(min(x),max(x),nbars+1);
    else
        edges=[0 1];
    end
    
    %check is for all practical purposes, variable is a constant.
    if numel(unique(edges))<numel(edges)
       edges=[x(1)-1 x(1)+1];        
    end


    count=histc(x(nothidden),edges);
    if ~isempty(x);        
        selectedcount=histc(x(isselected & nothidden),edges);
    else
        selectedcount=[0 0];
    end

    if udata.view.normalize %normalize counts (make pdfs)
        count=count./length(x);
        selectedcount=selectedcount./length(x);
    end

    
    if  pos(3) > pos(4) % &&  ~udata.view.stack  %horizontal stacking

        axPos = [pos(1)+(c-1)*(width+ylabelspace)+ylabelspace pos(2)+margin ...
            width-ylabelspace height*(1-space)]; %subtract margin to make space for statistics
        statsPos=[axPos(1) axPos(2)-150 axPos(3) 100];

    else %vertical stacking
        axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
            width-ylabelspace height*(1-space)-20]; %subtract margin to make space for statistics
        statsPos=[axPos(1)+width-ylabelspace+10 axPos(2)-15 150 100];

    end
    
    
    fakerax(c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,'color',bgcolor,...
        'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,...
        'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize); %#ok<AGROW>
    fakeax(c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
        'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize); %#ok<AGROW>
    ax(c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
        'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnhistogram,...
        'userdata',handles.activeindex,'color','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
        'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %#ok<AGROW> %userdata contains the index of the bigax for this subgraph.
    set(ax(c),'visible','on');
    set([fakerax(c) fakeax(c)],'units','pixels','visible','on');

    if strcmp(udata.format.orient,'horizontal') 
        view(ax(c),90,90);
        view(fakeax(c),90,90);
        view(fakerax(c),90,90);
    end


    %main data series:
    mainhandle=bar(ax(c),edges,count,udata.view.barwidth);
    if udata.format.drawedge
        edgecolor=udata.format.edgecolor;
    else
        edgecolor=udata.format.facecolor;
    end

    set(mainhandle,'facecolor',udata.format.facecolor,'hittest','on','tag','main','edgecolor',edgecolor,'buttondownfcn',@pointselecthistogram);

    %selected data series
    selectedhandle=bar(ax(c),edges,selectedcount,udata.view.barwidth);
    set(selectedhandle,'facecolor',udata.format.facecolor*.6,'hittest','off','tag','selected','edgecolor',edgecolor*.6);


    axis(ax(c),'tight');
    set(ax(c),'ylimmode','auto','tickdir','in','layer','top');
    xlim{c} = get(ax(c),'xlim');  %#ok<AGROW>
    ylim{c} = get(ax(c),'ylim'); %#ok<AGROW>
    
    ravemaketextticks(ax(c),[],xcol(c),[],[],[]) 
    set([fakeax(c) fakerax(c)],'xtick',get(ax(c),'xtick'),'xticklabel','','ytick',get(ax(c),'ytick'),'yticklabel','','ztick',get(ax(c),'ztick'),'zticklabel','')
    xlabel(ax(c),handles.allnames{d}{xcol(c)},'interpreter',handles.prefs.interpreter,'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph)


    if udata.view.showstats
        statsax(c)=axes('units','pixels','Position',statsPos,'parent',BigAxParent,...
            'visible','off','nextplot','add','box','on','hittest','off','buttondownfcn','',...
            'userdata',handles.activeindex,'color','none'); %#ok<AGROW>
        
        if height<120
            statsize=handles.prefs.fonts.graphsize-2;
            set(statsax(c),'pos',statsPos.*[1 1 1 .8]);
        else
            statsize=handles.prefs.fonts.graphsize;
        end
        
        text(0,.95,'Mean:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.80,'Median:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.65,'Variance:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.50,'Std Dev:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.35,'Skewness:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.20,'Kurtosis:','parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.95,['                   ',num2str(mean(x))],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.80,['                   ',num2str(median(x))],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.65,['                   ',num2str(var(x))],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.50,['                   ',num2str(std(x))],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.35,['                   ',num2str(skewness(x))],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
        text(0,.20,['                   ',num2str(kurtosis(x)-3)],'parent',statsax(c),'tag','histogramstat','fontsize',statsize,'fontname',handles.prefs.fonts.graph);
    else
        statsax=[];
    end
end

%fix axes limts:
for r=1:numvars
    set(fakerax(r),'xlim',xlim{r},'ylim',ylim{r})
    set(fakeax(r),'xlim',xlim{r},'ylim',ylim{r})
end

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
handles.statsax{handles.activeindex}=statsax;


%% %% %%

set(handles.activegraph,'userdata','histogram');


%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%select the subaxes in position (1) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%update da handles
guidata(src,handles)



