function ravecreateparallelcoords(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
xcol=udata.view.x;
%ycols=udata.view.y;
ccol=udata.view.c;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset;

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');

%font formatting
fweight='normal';
labelsize=12;


udata.view.varmins=handles.minlimit{d}{a};
udata.view.varranges=handles.maxlimit{d}{a}-handles.minlimit{d}{a};
set(handles.fakeax(handles.activeindex),'userdata',udata)

%get data
x=handles.alldata{d}(:,xcol);
%normalize xdata:
data1=bsxfun(@minus,x,handles.minlimit{d}{a}(xcol));
data2=bsxfun(@rdivide,data1,(handles.maxlimit{d}{a}(xcol)-handles.minlimit{d}{a}(xcol)));
data2(isnan(data2))=.5;
x=data2;


%maintain selection:
isselected=handles.isselected{udata.view.dataset}{udata.view.analysis}; %in case of non-logical indexing. 

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};



%filter based on max rows
if udata.view.usemaxlines
    switch udata.view.maxlinesmode
        case 'top'
            if udata.view.maxlines>=size(x,1)
                index=1:size(x,1);
            else

                index=1:udata.view.maxlines;
            end

        case 'random'
            if udata.view.maxlines>=size(x,1)
                index=1:size(x,1);
            else
                rand1=randperm(size(x,1));
                index=rand1(1:udata.view.maxlines);
            end
    end
    
else    
    index=1:size(x,1);    
    
end

index=sort(index,'ascend');
udata.view.maxlineindex=index;
setudata;

nothidden=nothidden(index);
isselected=isselected(index);
x=x(index,:);

%% get constantvalues:
% if constcol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the constcol column.


%append a column of ones on the end for when constcol="none selected"...
%might need to move this line somewhere else in code.


allcolorz=get(handles.fig,'colormap');

if ccol~=0
    c=handles.alldata{d}(index,ccol);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.lcmap-1)+1; 
    if udata.format.reversecolors
        c=(1-(c-min(c))/(max(c)-min(c)))*127+128*(udata.format.lcmap-1)+1; 
    end
    
    %cdata=[linterp(1:size(allcolorz,1),allcolorz(:,1),c)' linterp(1:size(allcolorz,1),allcolorz(:,2),c)' linterp(1:size(allcolorz,1),allcolorz(:,3),c)'];   
    c(isnan(c))=128*(udata.format.lcmap-1)+1;

    cdata=allcolorz(floor(c),:);
    
    
else %USE ROW COLORS

    c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis}(index))'; %non-noisy colors cdata=0.5     
    cdata=allcolorz(c,:);
    
    if udata.format.ncolor %do noisy colors
        noise=0.3; %noise multiplier
        cdata=cdata+(rand(size(cdata))-.5)*noise; 
        cdata(cdata<0)=0;
        cdata(cdata>1)=1;
           
    end 
end


set(handles.activegraph,'drawmode','normal','ylim',[0 1],'xlim',[0 1],'ylimmode','manual','xlimmode','manual','visible','off');
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'visible','off');
nvars=numel(xcol);
spacing=1/(nvars-1);

%remove hidden rows:
%x=x(nothidden,:);
%cdata=cdata(nothidden,:);


%% make background:

background=rectangle('position',[0 0 1 1],'parent',handles.fakerax(handles.activeindex),'linestyle','none',...
    'facecolor',handles.colors.axes{udata.format.bg}(2,:),'tag','background','hittest','off','buttondownfcn',@parallelcoordsdeselect);

%% make data:

udata.view.mainhandles=zeros(size(handles.alldata{d},1),1);
udata.view.selectedhandles=zeros(size(handles.alldata{d},1),1);
udata.view.hiddenhandles=zeros(size(handles.alldata{d},1),1);

for r=1:size(handles.alldata{d},1);
     udata.view.hiddenhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','hidden',...
        'marker','none','color',udata.view.offcolor,'userdata',cdata(r,:),'linewidth',udata.format.linesize/2,'visible','off');
    udata.view.mainhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','main',...
        'marker','none','color',cdata(r,:),'userdata',cdata(r,:),'linewidth',udata.format.linesize/2);
    udata.view.selectedhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','selected',...
        'marker','none','color',cdata(r,:),'linewidth',udata.format.linesize/2+1.5,'visible','off','userdata',cdata(r,:));
end

%filter data:


%one last filter....
if ~isempty(x)
    
    %filter only on visible vars:    
%      allkidz=sort(findobj(handles.activegraph,'tag','main'));
%      selected=sort(findobj(handles.activegraph,'tag','selected'));
%      allydata=cell2mat(get(allkidz,'ydata'));
%      maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
%      minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));
%      outliers=minoutlier | maxoutlier;
%      outliers=any(outliers,2);
%     
    %filter on all vars:
    allkidz=udata.view.mainhandles;
    selected=udata.view.selectedhandles;
    allydata=handles.alldata{d}(index,:);
    allydata=bsxfun(@minus,allydata,udata.view.varmins);
    allydata=bsxfun(@rdivide,allydata,udata.view.varranges);
    
    maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter);
    minoutlier=bsxfun(@lt,allydata,udata.view.minfilter);
    outliers=minoutlier | maxoutlier;
    outliers=any(outliers,2);
    
    
    
    
    %need to repeat filter cause uistack screws up the handle orders.
    
    %set(allkidz(outliers),'color',udata.view.offcolor)
    %set(selected(outliers),'color',udata.view.offcolor)
    
    set(udata.view.hiddenhandles(outliers),'visible','on');
    set(allkidz(outliers),'visible','off');
    set(selected(outliers),'visible','off');
    
    
    
    uistack(selected(~outliers),'bottom');
    uistack(allkidz(~outliers),'bottom');
    uistack(selected(outliers),'bottom');
    uistack(allkidz(outliers),'bottom');
    uistack(udata.view.hiddenhandles,'bottom');
    
    udata.view.filtered=outliers;
end



set(handles.fakeax(handles.activeindex),'userdata',udata);

%turn on visiblity of selected lines:
%set(udata.view.mainhandles(~nothidden),'visible','off');
%set(udata.view.selectedhandles(isselected & nothidden),'visible','on');

  
%% make axis labels:
axpos=get(handles.activegraph,'position');
width=axpos(3)/(nvars-1);

maxchars=round(width/8);

T=ones(nvars,1);
for g=nvars:-1:1 %note: MUST draw these in reverse order.
T(g)=text(spacing*(g-1),-0.02,linewrap(handles.allnames{udata.view.dataset}{udata.view.x(g)},maxchars),'parent',handles.activegraph,...
    'tag','axeslabel','verticalalignment','top','horizontalalignment','center','hittest','off','interpreter',handles.prefs.interpreter,...
    'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph);
end

set(T(end),'horizontalalignment','right','pos',[1.02 -0.02]);
set(T(1),'horizontalalignment','left','pos',[-0.02 -0.02],'clipping','off');


%make filter value labels:

if udata.format.bg==4
    axcolor=[.8 .8 .8];
else
    axcolor=[0 0 0];
end

if udata.format.showlimits
    viz='on';
else
    viz='off';
end

minz=handles.minlimit{d}{a}(udata.view.x);
maxz=handles.maxlimit{d}{a}(udata.view.x);
rangez=maxz-minz;

maxfilterz=udata.view.maxfilter(udata.view.x).*rangez+minz;
minfilterz=udata.view.minfilter(udata.view.x).*rangez+minz;


for g=nvars:-1:1 %note, MUST draw these in reverse order.
text(spacing*(g-1),udata.view.maxfilter(udata.view.x(g)),['   ',strtrim(sprintf('%-8.3g',maxfilterz(g)))],...
    'parent',handles.activegraph,'tag','maxtexter',...
    'backgroundcolor',get(background,'facecolor'),'color',axcolor,'fontsize',handles.prefs.fonts.graphsize,'fontweight','normal','horizontalalignment','left',...
    'margin',1,'visible',viz,'hittest','on','buttondownfcn',@parallelcoordssetfilter,'fontname',handles.prefs.fonts.graph);
text(spacing*(g-1),udata.view.minfilter(udata.view.x(g)),['   ',strtrim(sprintf('%-8.3g',minfilterz(g)))],'parent',handles.activegraph,'tag','mintexter',...
    'backgroundcolor',get(background,'facecolor'),'color',axcolor,'fontsize',handles.prefs.fonts.graphsize,'fontweight','normal','horizontalalignment','left',...
    'margin',1,'visible',viz,'hittest','on','buttondownfcn',@parallelcoordssetfilter,'fontname',handles.prefs.fonts.graph);
end


%% set up axes
axcolors=[0 0 0;0 0 0;0 0 0;.8 .8 .8];

L=ones(nvars,1);
for q=1:nvars
   L(q)=line(spacing*(q-1)*ones(1,11),0:.1:1,'parent',handles.activegraph,'tag','parallelaxes','marker','+','color',axcolors(udata.format.bg,:));   
end
set(L(1),'xdata',.001*ones(1,11)) %prevents first line getting cut off when you export/rasterize. 


%% make top and bottom draggers:

xdat=spacing*((1:nvars)-1);
% 
% line(spacing*((1:nvars)-1),udata.view.maxfilter(xcol),'parent',handles.activegraph,'tag','maxdragger','marker','v','linestyle','none','markeredgecolor',[0 0 0],...
%     'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize)    
% line(spacing*((1:nvars)-1),udata.view.minfilter(xcol),'parent',handles.activegraph,'tag','mindragger','marker','^','linestyle','none','markeredgecolor',[0 0 0],...
%     'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize)

for j=1:nvars
line(xdat(j),udata.view.maxfilter(xcol(j)),'parent',handles.activegraph,'tag','maxdragger','marker','v','linestyle','none','markeredgecolor',[0 0 0],...
    'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize,'userdata',j)    
line(xdat(j),udata.view.minfilter(xcol(j)),'parent',handles.activegraph,'tag','mindragger','marker','^','linestyle','none','markeredgecolor',[0 0 0],...
    'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize,'userdata',j)
end


%update da handles
guidata(src,handles)

raveresizeaxes(src,[],handles.activeindex)

ravemakecolorbar(handles.fig,[]);




function parallelcoordsdeselect(src,ev)
%deselect all rows.


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


handles.isselected{udata.view.dataset}{udata.view.analysis}=false(size(handles.alldata{udata.view.dataset},1),1);

selected=findobj(handles.activegraph,'tag','selected');
set(selected,'visible','off');


guidata(handles.fig,handles);





