function ravecreatedensity(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

cla(targetaxes);
legend(targetaxes,'off');


set(targetaxes,'xtickmode','auto','ytickmode','auto')


%font formatting
fweight='normal';
labelsize=handles.prefs.fonts.graphsize;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

isselected=handles.isselected{udata.view.dataset}{udata.view.analysis};
%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};



%Normalize Data:

minx=min(x);
miny=min(y);

rangex=max(x)-min(x);
rangey=max(y)-min(y);

x=x-minx;
x=x./rangex;

y=y-miny;
y=y./rangey;





%% Determine inpolygon counts:



% FOR hexagons....
% Generate hexagonal grid
nx=udata.view.gridsize; %n must be even
ny=udata.view.gridsize;
[X Y] = meshgrid(linspace(0,2/sqrt(3),nx),linspace(0,1,ny));

X = sqrt(3)/2*X;
Y = Y + repmat([0 0.5*1/(ny-1)],[nx,ny/2]);

Y=Y*2/sqrt(3);

X=X(:);
Y=Y(:);

%hexagon geometry:
dx=2/sqrt(3)/(nx+2);
r=dx/1.5;
thetas=[0 60 120 180 240 300]*pi/180;


%calculate in-ness:
thismany=zeros(numel(X),1);
inhere=false(numel(X),numel(x));
for c=1:numel(X)
    xhex=r*cos(thetas)+X(c);
    yhex=r*sin(thetas)+Y(c);
    %patch(xhex+X(c),yhex+Y(c),rand(1,3),'facealpha',0.5);
    inhere(c,:)=inpolygon(x,y,xhex,yhex)';
    thismany(c)=sum(inhere(c,:));
end

tilescale=sqrt(thismany)./max((sqrt(thismany)));

tilescale(tilescale<0.3)=0.3;


%% get constantvalues:
% if color/sizecol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the column.


%ccol = 0 means constant color.
%ccol = -1 means use row colors.

if ccol==-1 %use most populous row color
    
    
    facecolor=repmat([0 0 0],numel(X),1);
    
    
    
    
elseif ccol==0 %use constant color
    facecolor=repmat(udata.format.facecolor,numel(X),1);
    
    
    
else %color according to avg of selected variable.
    c=zeros(numel(X),1);
    for h=1:numel(X)
        c(h)=mean(handles.alldata{d}(inhere(h,:)',ccol));
    end
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        facecolor=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        facecolor=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
    end
    
end



set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);


%% SIZE TILES
if scol==-1 %Do density sizing
    
    sizescale=tilescale;
    
elseif scol==0 %Do color sizing
    sizescale = ones(numel(X),1);
else %size by avg of selected variable.
    
    s=zeros(numel(X),1);
    for h=1:numel(X)
        s(h)=mean(handles.alldata{d}(inhere(h,:)',scol));
    end
    
    sizescale=s-min(s);
    sizescale=sizescale./max(sizescale);
    
    
    
    
end


%% Draw the patches:

%denormalize data:
if ~isempty(x)
    X=(X*rangex)+minx;
    Y=(Y*rangey)+miny;
    
    x=(x*rangex)+minx;
    y=(y*rangey)+miny;
    
    
    %draw hexagons:
    for h=1:numel(X)
        if thismany(h)>1
            xhex=sizescale(h)*r*cos(thetas)*rangex;
            yhex=sizescale(h)*r*sin(thetas)*rangey;
            patch(xhex+X(h),yhex+Y(h),facecolor(h,:),'parent',targetaxes,'edgecolor','none');
        elseif thismany(h)==1
            thisone=inpolygon(x,y,r*cos(thetas)*rangex+X(h),r*sin(thetas)*rangey+Y(h));
            xhex=sizescale(h)*r*cos(thetas)*rangex;
            yhex=sizescale(h)*r*sin(thetas)*rangey;
            patch(xhex+x(thisone),yhex+y(thisone),facecolor(h,:),'parent',targetaxes,'edgecolor','none');
        end
    end
end





%%

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)


%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar && udata.view.c;
ravemakecolorbar(handles.fig,[]);
%end

ravemaketextticks(targetaxes,[],xcol,ycol,[],[])


raveresizeaxes(src,[],targetindex)


%set(targetaxes,'xlimmode','manual','ylimmode','manual')








