function ravecreatebarcategorical(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

cla(targetaxes);
legend(targetaxes,'off');

set(targetaxes,'xtickmode','auto','ytickmode','auto')

if xcol==0
    return
end
    



%get data

if strcmp(handles.datatype{d}{xcol},'string')
    xvalues=1:numel(handles.discretevalues{d}{a}{xcol});
else
    xvalues=handles.discretevalues{d}{a}{xcol}; %discrete
end

if zcol>0
    zvalues=1:numel(handles.discretevalues{d}{a}{zcol});
elseif zcol==0 %row color.
    zvalues=1:10;
else
    zvalues=1;
end

if ycol>0
    y=handles.alldata{d}(:,ycol);
else
    y=ones(size(handles.alldata{d},1),1);
end

x=handles.alldata{d}(:,xcol);

if zcol>0
    z=handles.alldata{d}(:,zcol);
end

%make bar data:
bardata=zeros(numel(xvalues),numel(zvalues));

if zcol>0 %stack by variable
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            bardata(r,c)=aggregate(y(z==zvalues(c) & x==xvalues(r)),udata.view.aggregateby);
        end
    end
elseif zcol==0 %stack by row color
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            bardata(r,c)=aggregate(y(handles.rowcolors{d}{a}==c & x==xvalues(r)),udata.view.aggregateby);
        end
    end
    
elseif zcol==-1 %no stacking
    
    for r=1:numel(xvalues)
        bardata(r)=aggregate(y(x==xvalues(r)),udata.view.aggregateby);
    end
    
    
end

if udata.view.stack
    layoutstring='stacked';
else
    layoutstring='grouped';
end


hh=bar(targetaxes,xvalues,bardata,'barlayout',layoutstring,'barwidth',1);

nstacks=numel(hh);

if zcol~=0
allcolors=handles.prefs.barcolors;
else
    cmap=get(handles.fig,'colormap');
    allcolors=cmap(handles.prefs.rowcolors,:);
end


ncolors=size(allcolors,1);

reps=ceil(nstacks/ncolors);
allcolors=repmat(allcolors,reps,1);

for n=1:nstacks
   set(get(hh(n),'children'),'facecolor',allcolors(n,:),'edgecolor',[0 0 0]); 
    
end


if strcmp(udata.format.orient,'horizontal')
    view(targetaxes,90,90);
    view(handles.fakeax(handles.activeindex),90,90);
    view(handles.fakerax(handles.activeindex),90,90);
end


set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))


 if strcmp(udata.format.orient,'horizontal') 
        view(handles.activegraph,90,90);
        view(handles.fakeax(handles.activeindex),90,90);
        view(handles.fakerax(handles.activeindex),90,90);
 end



%% make tick marks:

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize+2,...
    'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)

if ycol>0 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.prefs.fonts.graphsize+2,'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','Count',...
        'fontsize',handles.prefs.fonts.graphsize+2,'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)    
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

set(targetaxes,'userdata','barcategorical');


%update da handles
guidata(src,handles)

ravemaketextticks(targetaxes,[],xcol,[],[],[])
raveresizeaxes(src,[],targetindex)


function y=aggregate(yvec,mode)

if mode==1
    y=sum(yvec);
elseif mode==2
    y=mean(yvec);
else
    y=numel(yvec);
end

