function viewtabimage(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
delete(handles.tabobjects)

%% get input info from graph:

%inputs=get(handles.fakeax(handles.activeindex),'userdata');
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

path=udata.view.path;

%get name of current directory:
lastslash=find(path=='\',2,'last');
if max(lastslash)==length(path) %path ends in a slash
    dirname=path(lastslash(1)+1:end-1);
else
    dirname=path(lastslash(2)+1:end);
end

%get possible flowpaths from this directory:
filez=dir(path);
filenamez=cell(0);
for g=1:numel(filez)
    thisname=filez(g).name;
    if length(thisname)>5 && ismember(lower(thisname(end-2:end)),{'bmp';'jpg';'tif';'peg';'iff';'gif';'df4';'pcx';'pgm';'png';'ras'})
        filenamez{end+1}=thisname; %#ok<AGROW> cant determine final size ahead of time.
    end
    
end

if isempty(filenamez)
    filenamez='No valid image files found';
    udata.view.invalid=1;
end

t=1;

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'File Info','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.35*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
displaymode=text(.25-.10317,.72*1.0317-.0317,'Display Mode:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
dirtext=text(.25-.10317,.65*1.0317-.0317,'Current Directory:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
filestext=text(.25-.10317,.58*1.0317-.0317,'Choose File:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

% notetext=text(.25,.38,'Note: Max simultaneous plots is 4.','horizontalalignment','left',...
%     'backgroundcolor',handles.colors.tabbg,...
%     'parent',handles.tabaxes,'fontsize',8,'fontname','arial','color',[0 0 0],'fontweight','normal');

%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',view.dataset);
modemenu=uicontrol('position',[origin(1)+55 origin(2)+405 180 30],...
    'style','popupmenu','string',{'Standalone';'Linked'},'callback',@modemenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu');
diredit=uicontrol('position',[origin(1)+55 origin(2)+368 150 22],...
    'style','edit','string',dirname,'callback','',...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',1,'horizontalalignment','left');
browsebutton=uicontrol('position',[origin(1)+210 origin(2)+368 24 24],...
    'style','pushbutton','cdata',imread('browse.bmp'),'callback',@browsebutton_callback); 

filemenu=uicontrol('position',[origin(1)+55 origin(2)+235 180 110],...
    'style','listbox','string',filenamez,'callback',@filemenu_callback,...
    'backgroundcolor',[1 1 1],'tag','constmenu','value',1,'max',4);


%% buttons



%% group items and finish up
handles.tabobjects=[datasettext displaymode dirtext filestext datasetmenu modemenu diredit filemenu,...
    analysistext projecttext analysismenu dataseriestext optionstext,...
    browsebutton];

handles.activetab='view';

guidata(src,handles);


%% callbacks:

function modemenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if get(src,'value')==1 %standalone mode
    set(handles.infobar,'string','')
    udata.view.linked=0;
    
else %linked mode
   set(handles.infobar,'string','Image will update each time a single point is selected.')
   udata.view.linked=1;
    
end

set(handles.fakeax(handles.activeindex),'userdata',udata);


function browsebutton_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

path=uigetdir(handles.defaultdir,'Choose a directory');
if path==0;return;end

udata.view.path=path;
set(handles.fakeax(handles.activeindex),'userdata',udata);

guidata(src,handles)

%refresh this tab
viewtabimage(handles.fig,[]);




function filemenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

filenamez=get(src,'string');

if any(strcmp(filenamez,'No valid image files found'))
    udata.view.invalid=1;
else
    udata.view.invalid=0;
end

values=get(src,'value');

if numel(values)>4
    values=values(1:4);
end


udata.view.filename=filenamez(values);
set(handles.fakeax(handles.activeindex),'userdata',udata);

guidata(src,handles)

ravecreateimage(handles.fig,[]);



