function viewtabtableallpoints(src,eventdata) %#ok<INUSD>
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;
a=udata.view.analysis;

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',a);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',d);

%% options:

coltext=text(.25-.10317,.71,'Variables to Display:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');


xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+230 180 210],...
    'style','listbox','string',handles.allnames{d},'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','value',udata.view.x,'max',100);


rowtext=text(.25-.10317,.35,'Rows to Display:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

unselectedcheck=uicontrol('position',[origin(1)+55 origin(2)+190 180 22],...
    'style','checkbox','string','Unselected Rows','callback',@rowcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','unselectedcheck','value',udata.view.showrows(1));
selectedcheck=uicontrol('position',[origin(1)+55 origin(2)+166+2 180 22],...
    'style','checkbox','string','Selected Rows','callback',@rowcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','selectedcheck','value',udata.view.showrows(2));
hiddencheck=uicontrol('position',[origin(1)+55 origin(2)+142+4 180 22],...
    'style','checkbox','string','Hidden Rows','callback',@rowcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','hiddencheck','value',udata.view.showrows(3));
currentcheck=uicontrol('position',[origin(1)+55 origin(2)+118+6 180 22],...
    'style','checkbox','string','Current Point','callback',@rowcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','currentcheck','value',udata.view.showrows(4),'visible','off');

sorttext=text(.25-.10317,.17,'Sort Rows By:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

sortmenu=uicontrol('position',[origin(1)+55 origin(2)+65 180 30],...
    'style','popupmenu','string',[cellstr('<No Sorting>') handles.allnames{d}],'callback',@sortmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','sortmenu','value',udata.view.s+1);

ascendcheck=uicontrol('position',[origin(1)+55 origin(2)+45 180 22],...
    'style','radio','string','Ascending','callback',@sortcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','ascendcheck','value',udata.view.sortascending);
descendcheck=uicontrol('position',[origin(1)+155 origin(2)+45 100 22],...
    'style','radio','string','Descending','callback',@sortcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','descendcheck','value',~udata.view.sortascending);

ranktext=text(.21,.05,'Only show top              rows','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','normal');

rankedit=uicontrol('position',[origin(1)+140 origin(2)+21 50 20],...
    'style','edit','string',num2str(udata.view.rowlimit),'callback',@rowlimit_callback,...
    'backgroundcolor',[1 1 1],'tag','rankedit');


%% group items and finish up
handles.tabobjects=[datasettext datasetmenu,...
     analysistext projecttext analysismenu dataseriestext coltext rowtext ranktext sorttext sortmenu ...
     unselectedcheck selectedcheck hiddencheck currentcheck xaxismenu rankedit ascendcheck descendcheck];

handles.activetab='view';

guidata(src,handles);


function datasetmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=get(src,'value');
udata.view.dataset=d;
udata.view.x=1:numel(handles.allnames{d});
udata.view.s=0;
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);

function analysismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);

function xaxismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.x=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);

function rowcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if strcmp(get(src,'tag'),'unselectedcheck')
    udata.view.showrows(1)=get(src,'value');    
elseif strcmp(get(src,'tag'),'selectedcheck')
    udata.view.showrows(2)=get(src,'value'); 
elseif strcmp(get(src,'tag'),'hiddencheck')
    udata.view.showrows(3)=get(src,'value'); 
elseif strcmp(get(src,'tag'),'currentcheck')
    udata.view.showrows(4)=get(src,'value');     
end

set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);


function sortmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.s=get(src,'value')-1;
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);

function sortcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
if strcmp(get(src,'tag'),'ascendcheck')
    udata.view.sortascending=true;
else
    udata.view.sortascending=false;
end
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);


function rowlimit_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.rowlimit=str2double(get(src,'string'));
set(handles.fakeax(handles.activeindex),'userdata',udata)
raveupdatetableallpoints(src,ev);
