function [model,rhmodel]=ravecreatemaintablemodel(src,ev,d,a)
%makes a RaveTableModel of dataset d for the main data table, using
%analysis a.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.


handles=guidata(src);

if isfield(handles,'indata');
    handles=handles.indata;
end

%if d>0;
    data=handles.alldata{d};
    varnames=handles.allnames{d};
    rownums=1:size(data,1);
    datatypes=handles.datatype{d};
    isfunction=handles.isfunction{d};
    isrowshaded=~handles.isvisible{d}{a};
    rowcolors=handles.rowcolors{d}{a};
    isvalid=handles.isvalid{d};
    
%convert to cells:
data=num2cell(data);

colArray=javaArray('java.lang.String',numel(handles.allnames{d}));
for c=1:size(data,2)
    colArray(c)=java.lang.String(varnames{c});
end

isFunction=javaArray('java.lang.Boolean',numel(colArray));
for c=1:numel(colArray)
    isFunction(c)=java.lang.Boolean(isfunction(c));
end


if size(data,1)>0
    
    datatypes=handles.datatype{d};
    
    dblArray=javaArray('java.lang.Object',size(data,1),size(data,2));
    
    for c=1:size(data,2)
        switch datatypes{c}
            case 'string'
                for r=1:size(data,1);
                    if isnan(data{r,c})
                        dblArray(r,c)=java.lang.String('invalid entry');
                    else
                        dblArray(r,c)=java.lang.String(handles.discretevalues{d}{a}{c}{data{r,c}});
                    end
                end
            case {'integer','logical'}
                for r=1:size(data,1);
                    dblArray(r,c)=java.lang.Integer(data{r,c});
                end
            otherwise
                for r=1:size(data,1);
                    %dblArray(r,c)=java.lang.String(num2str(data{r,c}));
                    dblArray(r,c)=java.lang.Double(data{r,c});
                end
        end
    end
    
    
    isRowShaded=javaArray('java.lang.Boolean',size(data,1));
    jRowColors=javaArray('java.lang.Integer',size(data,1));
    isDataValid=javaArray('java.lang.Boolean',size(data));
    rowNumbers=javaArray('java.lang.Integer',size(data,1),1);
    
    
    for r=1:size(data,1)     
        isRowShaded(r)=java.lang.Boolean(isrowshaded(r));
        jRowColors(r)=java.lang.Integer(rowcolors(r)-1);
        rowNumbers(r,1)=java.lang.Integer(r);
           for c=1:size(data,2)
               isDataValid(r,c)=java.lang.Boolean(isvalid(r,c));
           end
        
    end
    
else
    dblArray=[];
end

if isjava(dblArray)
    dmodel=javax.swing.table.DefaultTableModel(dblArray,varnames);
    model1=RaveTableSorter(dmodel);
    model1.setIsFunction(isFunction);
    model1.setIsRowShaded(isRowShaded);
    model1.setRowColors(jRowColors);
    model1.setIsDataValid(isDataValid);
    rhmodel=javax.swing.table.DefaultTableModel(rowNumbers,{' '});
    
else %data set is empty
    dmodel=javax.swing.table.DefaultTableModel(varnames,0); %make an empty table
    model1=RaveTableSorter(dmodel);
    rhmodel=[];
end

index=handles.prefs.rowcolors;
allcolors=get(handles.fig,'colormap');
deezcolors=allcolors(index,:);

for c=1:10
    R=deezcolors(c,1);
    G=deezcolors(c,2);
    B=deezcolors(c,3);
    model1.setRowColorSpec(c-1, java.awt.Color(R,G,B));
end

model=handle(model1,'callbackproperties');
set(model,'TableChangedCallback',{@ravemaintablechangedcallback,handles.fig});

%model=RaveTableModel(dblArray,varnames);
%model.setIsFunction(isFunction);


%% fix column visibility

try %gives error when creating model for a new data set.
    shows=handles.maintable.x{handles.maintable.d};
catch
    return
end

cols=handles.maintable.jtable.getColumnModel().getColumns;
colcount=handles.maintable.jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    if ismember(modelIdx+1,shows) %show this column
        thisCol.setMinWidth(15);
        thisCol.setPreferredWidth(75);
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);
    end
end
