function ravedisplaymaintable(src,ev)
%shows the current tablemodel in the main table.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
d=handles.maintable.d;
a=handles.maintable.a;

if d==0;return;end;

%if ismember('model',handles.tablemodelupdatespending{d})
    %make new model:
    [handles.maintable.models{d} handles.maintable.rhmodels{d}]=ravecreatemaintablemodel(handles.fig,[],d,a);
    handles.handles.tablemodelupdatespending{d}=cell(0);
    guidata(handles.fig,handles);
%end



jtable=handles.maintable.jtable;
%tablemodel=jtable.getModel; %mtable uses defaulttablemodel

sorter=handles.maintable.models{d};
jtable.setModel(handles.maintable.models{d});
sorter.setTableHeader(jtable.getTableHeader());

%set column widths:
cols=jtable.getColumnModel().getColumns;
colcount=jtable.getColumnModel().getColumnCount;
shows=handles.maintable.x{handles.maintable.d};

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    
    if ismember(modelIdx+1,shows) %show this column
       thisCol.setMinWidth(15);
       thisCol.setPreferredWidth(handles.maintable.colwidths{handles.maintable.d}(c+1));        
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);        
    end    
end



%set callback for table:
jtSrc = handle(jtable,'callbackproperties');
set(jtSrc,'MouseReleasedCallback',{@raveTableMouseReleaseCallback,handles.fig})

%set callback for column headers:
jhead=jtable.getTableHeader();
jhead.setReorderingAllowed(1); %allow column reordering %NOTE: this might break some things in RaveTableSorter that assume no reordering.
chSrc = handle(jhead,'callbackproperties');
set(chSrc,'MouseReleasedCallback',{@raveTableHeaderCallback,handles.fig})
set(chSrc,'MouseDraggedCallback',{@raveTableHeaderDraggedCallback,handles.fig})


%set up row headers:
rowHeaderViewport=handles.maintable.jscroll.getComponent(3);
rowHeader=rowHeaderViewport.getComponent(0);
rowHeader.setModel(handles.maintable.rhmodels{d});

%     %reposition viewports:
%     jscroll=handles.maintable.jscroll;
%     v1=jscroll.getComponent(4);
%     

nrows=sorter.getRowCount;
rheight=rowHeader.getRowHeight;
ndigits=ceil(log10(nrows));

rowHeader.setPreferredSize(java.awt.Dimension(10*ndigits+6,nrows*rheight))
rowHeader.setSize(java.awt.Dimension(10*ndigits+6,nrows*rheight))

rowHeaderViewport.setPreferredSize(java.awt.Dimension(10*ndigits+6,rowHeaderViewport.getHeight));

handles.maintable.jscroll.doLayout;

   
%renderer:
rrrr=RaveDefaultRowHeaderRenderer();

tm=rowHeader.getColumnModel();
tc=tm.getColumn(0);

if handles.maintable.colorrows && ~handles.useoldtables
    %rowHeader.setDefaultRenderer(class,rrrr);
    tc.setCellRenderer(rrrr);
else
    tc.setCellRenderer(handles.maintable.defaultRowHeaderRenderer);
end

%callbacks:
rhSrc = handle(rowHeader,'callbackproperties');
%set(rhSrc,'MouseReleasedCallback',{@raveRowHeaderCallback,handles.fig})
set(rhSrc,'MousePressedCallback',{@raveRowHeaderCallback,handles.fig})



% %if using old tables, manually set number of rows in rowheader:
% if handles.useoldtables
%    
%     mmm=rowHeader.getModel;
%     mmm.updateRowCount;
%     
%     %reposition viewports:
%     jscroll=handles.maintable.jscroll;
%     v1=jscroll.getComponent(4);
%     v1.setPreferredSize(java.awt.Dimension(10*ndigits+6,v1.getHeight));
%     jscroll.doLayout;
% end

%set cell renderers:
rdouble=RaveDefaultTableCellRenderer();


obj=java.lang.Double(1);
class=obj.getClass();
jtable.setDefaultRenderer(class,rdouble);

rstring=RaveStringTableCellRenderer();

obj=java.lang.String;
class=obj.getClass();
jtable.setDefaultRenderer(class,rstring);

obj=java.lang.Object;
class=obj.getClass();
jtable.setDefaultRenderer(class,rdouble);

rinteger=RaveIntegerTableCellRenderer();

obj=java.lang.Integer(1);
class=obj.getClass();
jtable.setDefaultRenderer(class,rinteger);

%select rows that are selected in this analysis:
selectedRows=find(handles.isselected{d}{a}); 

if ~isempty(selectedRows)
    selectedRows=selectedRows-1;
    
    for g=1:numel(selectedRows);
        selectedRows(g)=handles.maintable.models{d}.viewIndex(selectedRows(g));
    end
    
    
    ncols=handles.maintable.models{d}.getColumnCount();
    
    jtable.setColumnSelectionInterval(0,ncols-1);
    jtable.setRowSelectionInterval(selectedRows(1),selectedRows(1));
    for i=2:numel(selectedRows)
        jtable.addRowSelectionInterval(selectedRows(i),selectedRows(i));
    end
end

%make sure renderers match current state:
rowHeader.repaint;
jtable.repaint;



