function viewtabmaintable(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for main table
ravechangetab(src,[]);

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
d=handles.maintable.d;
a=handles.maintable.a;


%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.76,'Columns','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.24,'Editing','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
     'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
variablestext=text(.25-.10317,.73,'Select Visible Columns:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');


%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',a);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',d);

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+200 180 250],'style','listbox','max',100,...
    'string',handles.allnames{d},'callback',@xaxismenu_callback,'backgroundcolor',[1 1 1],'value',handles.maintable.x{d});

resetbutton=uicontrol('position',[origin(1)+55 origin(2)+168 180 24],'style','pushbutton',...
    'string','Reset Column Order','callback','','background',handles.colors.buttonbg,'tag','resetbutton','enable','off');

undobutton=uicontrol('position',[origin(1)+55 origin(2)+110 85 24],'style','pushbutton',...
    'string','Undo','callback',@ravemaintableundo,'background',handles.colors.buttonbg,'tag','undobutton');
redobutton=uicontrol('position',[origin(1)+150 origin(2)+110 85 24],'style','pushbutton',...
    'string','Redo','callback',@ravemaintableredo,'background',handles.colors.buttonbg,'tag','redobutton');

validatebutton=uicontrol('position',[origin(1)+55 origin(2)+80 180 24],'style','pushbutton',...
    'string','Evaluate Functions','callback',@validatebutton_callback,'background',handles.colors.buttonbg,'tag','undobutton');


if isempty(handles.maintable.undos{1})
    set(undobutton,'enable','off')
end

if isempty(handles.maintable.redos{1})
    set(redobutton,'enable','off')
end

validatecheck=uicontrol('position',[origin(1)+55 origin(2)+55 200 22],'style','checkbox',...
    'string','Evaluate Functions Immediately','callback','','background',handles.colors.tabbg,'tag','','value',1,'enable','off',...
    'callback',@validatecheck_callback);
editcheck=uicontrol('position',[origin(1)+55 origin(2)+30 150 22],'style','checkbox',...
    'string','Make Table Editable','callback','','background',handles.colors.tabbg,'tag','','value',1,'enable','off');
selectcheck=uicontrol('position',[origin(1)+55 origin(2)+5 150 22],'style','checkbox',...
    'string','Clicking Selects Data','callback','','background',handles.colors.tabbg,'tag','','value',1,'enable','off');


%% buttons


%% group items and finish up
handles.tabobjects=[datasettext optionstext datasetmenu variablestext,...
    analysistext projecttext analysismenu dataseriestext xaxismenu,...
    undobutton redobutton editcheck selectcheck resetbutton validatecheck validatebutton];

handles.activetab='view';
guidata(src,handles);



%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

d=get(src,'value');



%save current column widths so they can be restored later:
cols=handles.maintable.jtable.getColumnModel().getColumns;
colcount=handles.maintable.jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    
    thiswidth=thisCol.getWidth;
    if thiswidth>0
        handles.maintable.colwidths{handles.maintable.d}(modelIdx+1)=thisCol.getWidth;
    end
end

handles.maintable.d=d;
guidata(src,handles);


ravedisplaymaintable(handles.fig,[]);

viewtab(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

a=get(src,'value');
handles.maintable.a=a;
handles.maintableupdatespending=union(handles.maintableupdatespending,'model');
guidata(src,handles);

ravedisplaymaintable(handles.fig,[]);


function xaxismenu_callback(src,ev)
handles=guidata(src);

handles.maintable.x{handles.maintable.d}=get(src,'value');

shows=get(src,'value');

cols=handles.maintable.jtable.getColumnModel().getColumns;
colcount=handles.maintable.jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    
    thiswidth=thisCol.getWidth;
    if thiswidth>0
        handles.maintable.colwidths{handles.maintable.d}(modelIdx+1)=thisCol.getWidth;
    end
    
    if ismember(modelIdx+1,shows) %show this column
       thisCol.setMinWidth(15);
       thisCol.setPreferredWidth(handles.maintable.colwidths{handles.maintable.d}(c+1));        
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);        
    end    
end

guidata(handles.fig,handles);



function validatecheck_callback(src,ev)

function validatebutton_callback(src,ev)
