function ravensgaii(src,ev,settings) %#ok<INUSD>
%2-D nsga pareto front finding. Inputs=nvars from data set, eqaution names from handles, target
%values for outputs from text boxes (tabobjects).
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%settings is a structure with the following fields:
% .animation= logical, export animation or not
% .gif = logical, export as gif vs. avi
% .fps = double, frames per second. or empty for real time.
% .loop = logical, loop gif vs. run once.
% .pcross =decimal, probability of crossover
% .pmutation =decimal, probability of mutation.
% .nbits = int, number of bits
% .method='nsgaii';
% .pop= population size
% .gens = number of generations
% .plotprogress = logical, animate progress on selected graph or not
% .plotvars = list of varaibles whose functions need to be evalualted in order to draw this graph
% .targets = best value for each objective
% .d = dataset to use+
% .a = analysis to use
% .x = list of variables to use as objectives
% .activeconstraints = index of constraints to use
% .startbutton = handle of startbutton
% .stopbuttonpos = location in pixels to put pause button


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


settings.progresswindow=0;
genssincebetter=0;
minever=1;


if settings.progresswindow
   fff=figure;
   aaa=subplot(1,2,1);
   ylim([0 settings.pop+1])
   bbb=subplot(1,2,2);
   set(bbb,'yscale','log')
   
    
    
end



pcross=settings.pcross;
pmutation=settings.pmutation;
nbits=settings.nbits; %number of bits to discretize each input var

grabbyviz=get(handles.grabbers{handles.activeindex}(1),'visible');
bordercolor=get(handles.allblockers(handles.activeindex),'edgecolor');
%get frame bounds:
if settings.animation
    %open movie file:"
    set(handles.allblockers(handles.activeindex),'edgecolor',[1 1 1]);
    
    set(handles.grabbers{handles.activeindex},'visible','off');
    axisbounds=get(handles.allblockers(handles.activeindex),'pixelbounds'); %origin=top left
    figbounds=get(handles.fig,'position');
    framebounds=[axisbounds(1)+3 figbounds(4)-axisbounds(4)+3 axisbounds(3)-axisbounds(1)-4 axisbounds(4)-axisbounds(2)-4];
    if ~strcmp(handles.prefs.animation.format,'gif')
        [FileName,PathName,FilterIndex] = uiputfile('*.avi'); %#ok<NASGU>
        
        if handles.prefs.animation.fps
            mov = avifile([PathName FileName],'compression','none','quality',100,'fps',handles.prefs.animation.fps);
        else
            mov = avifile([PathName FileName],'compression','none','quality',100,'fps',11);
        end
    else
        [FileName,PathName,FilterIndex] = uiputfile('*.gif'); %#ok<NASGU>
        warning('off','MATLAB:writegif:loopcountInAppendMode');
    end
end

%tic %used in calculating fps for animations.
%tic has been moved below...


set(settings.startbutton,'enable','off') %turn off start button


%make stop button:

%origin=get(handles.tabaxes,'position'); %#ok<NASGU>
if ~isempty(settings.stopbuttonpos)
    t=0;
    stopbutton=uicontrol('position',settings.stopbuttonpos,'parent',handles.fig,...
        'style','pushbutton','string','PAUSE','backgroundcolor',handles.colors.buttonbg,...
        'callback',{@stopnsgaii,t},'enable','on','tag','stopbutton','foregroundcolor',[.8 0 0],'fontsize',12,'fontweight','bold');
    
    handles.tabobjects=union(handles.tabobjects,stopbutton);
    guidata(src,handles);
else
    stopbutton=[];
end


drawnow;

paretotarget=settings.targets;%[str2double(get(targ1edit,'string')) str2double(get(targ2edit,'string'))];
paretotarget=-paretotarget;

%constraintlist=findobj(handles.tabobjects,'tag','constraintlist');
handles.temp.activeconstraints=settings.activeconstraints; %get(constraintlist,'value')-1;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.

d=settings.d;
a=settings.a;
x=settings.x;

M=numel(settings.x); %number of objectives
N=floor(settings.pop);
T=floor(settings.gens);
if T<1;T=1;end

%get variables for these functions:

%indexx=handles.funvars{d}{x(1)};

%for k=2:M
%    indexx=union(indexx,handles.funvars{d}{x(k)});
%end

%allnames=handles.allnames{d};
%varnames=allnames(indexx);

%nvars=numel(indexx);%(number of input vars) not the same as size(d,2)

indexx=settings.funvars;
nvars=numel(indexx);


ranges=[handles.minlimit{d}{a}; handles.maxlimit{d}{a}];
rangestep=(ranges(2,:)-ranges(1,:))/(2^nbits-1);

%need to replace columns for discrete variables in varTable with only discrete values:
%but first need to decide how to determine which variables are discrete.
%let this slide for now.

%generate initial population:

domakepopulation=true;
if ~isempty(settings.popinitial)
    if ischar(settings.popinitial)
        dapath=which(settings.popinitial);
        if ~isempty(dapath)
            data=importdata(dapath);
            if isfield(data,'data');
                domakepopulation=false;
                Preal=data.data;
                settings.pop=size(Preal,1);
                N=settings.pop;
                %convert to binary:
                if nbits<=16.1
                    P=uint16(zeros(N,nvars*nbits)); %P is binary representation of population
                elseif nbits<=32.1
                    P=uint32(zeros(N,nvars*nbits));
                else
                    P=uint64(zeros(N,nvars*nbits));
                end
                
                %normalize 0 to 1:
                Preal=bsxfun(@minus,Preal,ranges(1,:));
                Preal=bsxfun(@rdivide,Preal,(ranges(2,:)-ranges(1,:)));
                Preal(isnan(Preal))=0;
                Preal(isinf(Preal))=0;
                Preal=round(Preal*(2^nbits-1));
                
                for g=1:numel(indexx)
                    p=Preal(:,indexx(g));
                    P(:,nbits*(g-1)+1:nbits*g)=rem(floor(p*pow2(1-nbits:0)),2);
                end
                
                
                %check, convert back to decimal:
                P3d=reshape(P,[size(P,1),nbits,nvars]);
                Prowindex=ones(N,size(ranges,2));
                
                colnums=(size(P3d,2):-1:1)-1;
                
                if nbits<=16.1
                    colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                elseif nbits<=32.1
                    colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                else
                    colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                end
                
                prodz=P3d.*colz;
                
                expoz=2.^prodz.*P3d;
                
                sumz=sum(expoz,2);
                
                %convert back to double here:
                Prow2D=permute(sumz,[1 3 2]);
                
                Prowindex(:,indexx)=Prow2D;
                
                Preal=bsxfun(@times,Prowindex,rangestep);
                Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population
                
                ddd=data.data;
                
                
            end
        end
    end
end

startgen=0;
starttime=0;




if domakepopulation
    thistag=get(src,'tag');
    if strcmp(thistag,'startbutton')
        
        
        if ~handles.prefs.gpgpu
            if nbits<=16.1
                P=uint16(round(rand(N,nvars*nbits))); %P is binary representation of population
            elseif nbits<=32.1
                P=uint32(round(rand(N,nvars*nbits)));
            else
                P=uint64(round(rand(N,nvars*nbits)));
            end
            
        else %use jacket:
            
            if nbits<=16.1
                P=gsingle(round(rand(N,nvars*nbits))); %P is binary representation of population
            elseif nbits<=32.1
                P=gsingle(round(rand(N,nvars*nbits)));
            else
                P=gsingle(round(rand(N,nvars*nbits)));
                % set(handles.infobar,'Error: Cannot use >32 bits with GPGPU');
            end
            
        end
        
        startgen=0;
        starttime=0;
        
        
        
    elseif strcmp(thistag,'continuebutton')
        P=handles.temp.finalbin;
        
        if nbits<=16.1
            P=uint16(P); %P is binary representation of population
        elseif nbits<=32.1
            P=uint32(P);
        else
            P=uint64(P);
        end
        
        startgen=handles.temp.lastgen;
        starttime=handles.temp.lasttime;
        delete(handles.temp.lasthandles(ishandle(handles.temp.lasthandles)));
        delete(src);
        handles.tabobjects=setdiff(handles.tabobjects,src);
        N=size(P,1);
    end
end

guidata(handles.fig,handles);

%% evaluate this population:
[objective cviolation]=evaluate_population(P,M,nbits,nvars,paretotarget,ranges,rangestep,indexx,handles,udata,settings);

%% calculate sorting/crowding of initial population:
[F,rank]=fast_non_dominated_sort(objective,cviolation);

%% calculate crowding distance: (for all frontiers)
distance=cell(0);
crowding=zeros(N,1);
for i=1:numel(F)
    I=objective(F{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
    distance{i}=crowding_distance_assignment(I);
    crowding(F{i})=distance{i};
end

Plast=P;
P=make_new_population(P,crowding,rank,pcross,pmutation,cviolation,handles);

tic %used to calculate ETA and total time.
set(handles.infobar,'string','\fontname{monospace} 0 \fontname{comic sans ms} Generations Completed.    Calculating ETA...')
elapsed=0; %initialize timer


%% DO GENERATIONS:
for t=1:T
    %t;
    handles.temp.lastgen=t;
    guidata(handles.fig,handles);
    
    %% combine new/old population:
    R=[P;Plast]; %combine new/old popualtions
    
    %evaluate population at R:
    [objective cviolation]=evaluate_population(R,M,nbits,nvars,paretotarget,ranges,rangestep,indexx,handles,udata,settings);
    %% calculate sorting/crowding of initial population:
    [F,rank]=fast_non_dominated_sort(objective,cviolation);
    %trim down to N members using rank/crowding sort:
    
    %% calculate crowding distance: (for all frontiers)
    distance=cell(0);
    crowding=zeros(N,1);
    for i=1:numel(F)
        if handles.prefs.gpgpu
            gthese=gsingle(F{i});
        else
            gthese=F{i};
        end
        I=objective(gthese,:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
        distance{i}=crowding_distance_assignment(I);
        crowding(F{i})=distance{i};
    end
    
    survivors=[];
    i=1;
    while numel(survivors)<N && i<=N
        if numel(survivors)+numel(F{i})<=N
            survivors=union(survivors,F{i}); %add all members of F to survivors
        else %add only enough members to make numel(survivors)==N
            shortage=N-numel(survivors);
            
            %add this many elements to survivors:
            cdist=crowding(F{i});
            index=F{i}; %index of cdist points into R
            data=[cdist index'];
            data=sortrows(data);
            %take the last (shortage) rows of data (these are least crowded)
            survivors=union(survivors,data(end-shortage+1:end,2));
            
        end
        i=i+1;
    end
    
    %pull out these rows of R:
    if handles.prefs.gpgpu
        gsurvivors=gsingle(survivors);
    else
        gsurvivors=survivors;
    end
    
    Plast=R(gsurvivors,:);
    cviolation=cviolation(survivors,:); 
    rank=rank(survivors);
    crowding=crowding(survivors);
    
    P=make_new_population(Plast,crowding,rank,pcross,pmutation,cviolation,handles);
    
    %bookkeeping stuff:
    lasttime=elapsed;
    elapsed=toc;
    
    timethisgen=elapsed-lasttime;
    timetogo=(timethisgen*(T-t))/(24*3600); %in fractional days.
    timestr=datestr(timetogo,13); %13 is format for hhmmss.
    
    if max(rank)==1 && mean(crowding(~isinf(crowding)))>minever
        
        %minever=mean(crowding(~isinf(crowding)));
        genssincebetter=genssincebetter+1;
    else
        genssincebetter=0;
        
    end
    
    if minever>mean(crowding(~isinf(crowding)));
        minever=mean(crowding(~isinf(crowding)));
    end
    

    if mod(t+startgen,5)<1
        set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t+startgen),'\fontname{',handles.prefs.fonts.tab,'} Generations Completed.    ETA: ' timestr '   Gens since improvment: '  num2str(genssincebetter)])
        drawnow update
    end
    
    
    
    if settings.progresswindow
        rankcounts=histc(rank,[0.5 1.5 2.5 3.5 4.5 5.5 inf]);
        
            colurz=[1 0 0;1 .5 0; 1 1 0; 0 1 0; 0 1 1 ; 0 0 1];
            
            if t==1
                for rc=1:6
                    LL(rc)=line(t,rankcounts(rc),'color',colurz(rc,:),'parent',aaa);
                    cline=line(t,mean(crowding(~isinf(crowding))),'color',[0 0 1],'parent',bbb);
                    drawnow;
                end
            else
                xcdata=get(cline,'xdata');
                ycdata=get(cline,'ydata');
                xcdata(end+1)=t;
                ycdata(end+1)=mean(crowding(~isinf(crowding)));

                set(cline,'xdata',xcdata,'ydata',ycdata);
                for rc=1:6
                    xdata=get(LL(rc),'xdata');
                    ydata=get(LL(rc),'ydata');
                    xdata(end+1)=t;
                    ydata(end+1)=rankcounts(rc);
                    set(LL(rc),'xdata',xdata,'ydata',ydata);
                    drawnow;
                end
                
                if t>20000
                    set(bbb,'xlim',[t-20000 t]);
                end
                
                
            end
    end
    
    %premature stopping:
    if ~isempty(settings.stopbuttonpos) && ~ishandle(stopbutton)
        break
    end
    

    if ~settings.plotprogress
        if rem(t,5)==0
            drawnow %pause button requires a drawnow to register, so check every 5 gens if pause has been pushed.
        end
        continue
    end
    
    %convert final population back to real numbers:
    %convert gray coding to decimal:
    P3d=reshape(Plast,[size(P,1),nbits,nvars]);
    
    Prowindex=ones(N,size(ranges,2));
    
    colnums=(size(P3d,2):-1:1)-1;
    
    if nbits<=16.1
        colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    elseif nbits<=32.1
        colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    else
        colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    end
    
    prodz=P3d.*colz;
    expoz=2.^prodz.*P3d;
    sumz=sum(expoz,2);
    
    %convert back to double here:
    Prow2D=permute(sumz,[1 3 2]);
    
    Prowindex(:,indexx)=Prow2D;
    
    Preal=bsxfun(@times,Prowindex,rangestep);
    Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population
    
    %fix discrete values:
    types=handles.datatype{settings.d}(settings.funvars);
    for u=1:numel(types)
        switch types{u}
            case {'discrete','string'}
                val=Preal(:,settings.funvars(u));
                allowed=repmat(handles.discretevalues{settings.d}{settings.a}{settings.funvars(u)}(:)',size(Preal,1),1);
                absdiff=abs(bsxfun(@minus,allowed,val));
                index=bsxfun(@eq,absdiff,min(absdiff,[],2));
                allowed=allowed';
                Pnew=allowed(index');
                Preal(:,settings.funvars(u))=Pnew;
            case {'integer','logical'}
                Preal(settings.funvars(u))=round(Preal(settings.funvars(u)));
        end
    end
    
    %draw progress:
    zerodata.settings=settings;
    zerodata.Preal=Preal;
    pfun=str2func(settings.plotfunction);
    pfun(handles.fig,[],settings,Preal,cviolation,rank);
    
    if settings.animation && strcmp(handles.prefs.animation.format,'gif')
        Framez(t) = getscreen(handles.fig,framebounds);
    elseif settings.animation
        F = getscreen(handles.fig,framebounds);
        mov = addframe(mov,F);
    end
    
end


%% finish up:


%remove dominated points from final population:
if handles.prefs.gpgpu
    survivors=gsingle(survivors);
end
objective=objective(survivors,:);
[F,rank]=fast_non_dominated_sort(objective,cviolation);

%convert final population back to real numbers:
P3d=reshape(Plast,[size(P,1),nbits,nvars]);

Prowindex=ones(N,size(ranges,2));

colnums=(size(P3d,2):-1:1)-1;

if nbits<=16.1
    colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
elseif nbits<=32.1
    colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
else
    colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
end

prodz=P3d.*colz;
expoz=2.^prodz.*P3d;
sumz=sum(expoz,2);

%convert back to double here:
Prow2D=permute(sumz,[1 3 2]);

Prowindex(:,indexx)=Prow2D;

Preal=bsxfun(@times,Prowindex,rangestep);
Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population

%fix discrete values:
types=handles.datatype{settings.d}(settings.funvars);
for u=1:numel(types)
    switch types{u}
        case {'discrete','string'}
            val=Preal(:,settings.funvars(u));
            allowed=repmat(handles.discretevalues{settings.d}{settings.a}{settings.funvars(u)}(:)',size(Preal,1),1);
            absdiff=abs(bsxfun(@minus,allowed,val));
            index=bsxfun(@eq,absdiff,min(absdiff,[],2));
            allowed=allowed';
            Pnew=allowed(index');
            Preal(:,settings.funvars(u))=Pnew;
        case {'integer','logical'}
            Preal(settings.funvars(u))=round(Preal(settings.funvars(u)));
    end
end



%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});
objective=raveevaluateregression(handles.fig,settings.d,respvars,Preal);
set(settings.startbutton,'enable','on','string','Restart')


if ishandle(stopbutton)
    delete(stopbutton);
end
handles.tabobjects=setdiff(handles.tabobjects,stopbutton);

%make continue button:
settings.gens=T-t;
if settings.gens==0
    settings.gens=T;
end

if ~isempty(settings.stopbuttonpos)
    continuebutton=uicontrol('position',settings.stopbuttonpos,'parent',handles.fig,...
        'style','pushbutton','string','Continue','backgroundcolor',handles.colors.buttonbg,...
        'callback',{@ravensgaii,settings},'enable','on','tag','continuebutton','foregroundcolor',[0 0.8 0],'fontsize',12,'fontweight','bold');
    handles.tabobjects=union(handles.tabobjects,continuebutton);
end

guidata(handles.fig,handles);

elapsed=toc/(24*3600);
timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.
set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t+startgen),'\fontname{',handles.prefs.fonts.tab,'} Generations Completed.    Elapsed Time: ' timestr])

%remove pareto tag so these objects dont get deleted on future runs:
paretos=findobj(handles.activegraph,'tag','pareto');
handles.temp.lasthandles=paretos;
set(paretos,'tag','paretofront');

if settings.animation && strcmp(handles.prefs.animation.format,'gif') && ~all(FileName==0)
    if handles.prefs.animation.loop
        loopcount=inf;
    else
        loopcount=1;
    end
    if handles.prefs.animation.fps
        delaytime=1/handles.prefs.animation.fps;
    else
        delaytime=toc/t;
    end
    movie2gif(Framez,[PathName FileName],'LoopCount',loopcount,'DelayTime',delaytime);
elseif settings.animation && ~all(FileName==0)
    mov = close(mov); %#ok<NASGU>
end

warning('on','MATLAB:writegif:loopcountInAppendMode');

set(handles.allblockers(handles.activeindex),'edgecolor',bordercolor);

% if isjava(handles.trayIcon)
%     trayIconClasses = handles.trayIcon.getClass.getClasses;
%     MessageTypes = trayIconClasses(1).getEnumConstants;
%     handles.trayIcon.displayMessage('RAVE:','Your optimizer is finished.',MessageTypes(3));
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P=double(P);
Preal=double(Preal);

handles.temp.lasttime=elapsed;
handles.temp.lastgen=t+startgen;

Preal(:,respvars)=objective;
[sortedpop,index]=sortrows(Preal,x);
handles.temp.finalbin=P(index,:);
handles.temp.finalpop=sortedpop;
handles.temp.finalresp=objective(index,:);

set(handles.grabbers{handles.activeindex},'visible',grabbyviz);

guidata(handles.fig,handles);
