function raveanimatemultiobj(src,ev,settings,pop,cviolation,rank) %#ok<INUSL>
%a generic multiobjective optimizer plotter, used for exmaple by nsgaii
%
%settings is structure of optimizer settings
%pop is population to display (only input values are needed.)
%cviolation is vector of constraint violations.
%mov is handle to movie, or empty.
%rank is pareto front rank (for multiobjective methods, empty for single
%obj.)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
plotvars=settings.plotvars;

N=size(pop,1); %population size

objective=raveevaluateregression(src,settings.d,plotvars,pop);
switch settings.method
    case {'nsgaii','nsgaiiwlocalmc'} %MULTIOBJECTIVE METHODS, USE RANK
        colorz=hsv(8);

        if max(rank)>8
            colorz=[colorz;repmat(colorz(end,:),(max(rank)-8),1)]; 
        end

        delete(findobj('tag','optplots'))

        switch settings.plottype
            case 'single'

                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives. 
                for q=max(rank):-1:2
                    plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    L=line(plotdata(:,1),plotdata(:,2),'marker','+','linestyle','none','parent',handles.activegraph,...
                        'tag','optplots','linewidth',1,'color',colorz(q,:),'markersize',2,'buttondownfcn',@lineobject_callback);
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                
                if numel(settings.x)==2
                    linestyle='-';
                else
                    linestyle='none';
                end
                
                plotdata=sortrows(objective(rank==1 & cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),'marker','+','linestyle',linestyle,'parent',handles.activegraph,...
                    'tag','optplots','linewidth',1,'color',colorz(1,:),'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                

                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),'marker','+','linestyle','none','parent',handles.activegraph,...
                    'tag','optplots','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback

            case '3D'

                for q=max(rank):-1:1
                    %plotdata=objective(rank==q & cviolation==0,:);
                    
                    if numel(settings.x)==2 %draw line instead of points;
                      linestyle={'linestyle','-','marker','none'};
                       plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    else
                        linestyle={'linestyle','none','marker','+'};
                        plotdata=objective(rank==q & cviolation==0,:);
                    end
                   
                    
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'parent',handles.activegraph,...
                        'tag','optplots','linewidth',1,'color',colorz(q,:),'markersize',2,'buttondownfcn',@lineobject_callback,linestyle{:});
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback

                end

                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','+','linestyle','none','parent',handles.activegraph,...
                    'tag','optplots','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback

        end
         
    otherwise %SINGLE OBJECTIVE METHODS, DON'T USE RANK.

end

%fix axes limits:
% set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
%     'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))

drawnow
