function [functionname,nvars,mins,maxs]=ravemakeminimizefunction(src,ev,settings)
%makes a temporary function that can be used in a MATLAB optimizer.
%distributes inputs and makes all functions "min the best". 
%outputs: functionname= name of the function created.
%         nvars = number of inputs variables to resulting function
%         mins = vector of min bounds for each input
%         maxs = vector of max bounds for each input
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

d=settings.d;
a=settings.a;
x=settings.x;

%get function names:
funnames=handles.funnames{d}(x);

%get variables for these functions:
indexx=handles.funvars{d}{x(1)};

for k=2:numel(x)
    indexx=union(indexx,handles.funvars{d}{x(k)});
end

%varnames=union(handles.funvars{d}{x},handles.funvars{d}{y});
% allnames=handles.allnames{d};
% 
% %remove spaces from these to match varnames format (ravejmp2m removes
% %spaces in variable names)
% for j=1:length(allnames)
%     allnames{j}=allnames{j}(allnames{j}~=char(32));
% end
% 
% %reorder varnames into order it occurs in table:
% indexx=zeros(1,numel(varnames));
% for q=1:numel(varnames)
%     indexx(q)=find(strcmp(allnames,varnames{q}));
% end

%get col numbers of input variables that will be changed (sorted)
indexx=sort(indexx);
nvars=numel(indexx);%(number of input vars) not the same as size(d,2)
mins=handles.minlimit{d}{a}(indexx);
maxs=handles.maxlimit{d}{a}(indexx);

%make fitness functions:
PathName=handles.prefs.defaultdir;
NameRandomizer=num2str(round(rand*10000)); %append a random number to avoid 2 funs having same name. (in case two raves are running at once)
FileName=['RAVEtempobjective',NameRandomizer,'.m'];
fid=fopen(fullfile(PathName,FileName),'w+'); %note, this line gives an error on new matlab installs, maybe because old defaultdir isnt on new path.

functionname=FileName(1:end-2);



%print a help file for this function:
fprintf(fid,'function response=%s(inputs)\n',functionname); %error here on new install, see fid=fopen... line above. 
fprintf(fid,'%%This is a temporary objective function for using an optimizer in RAVE.\n');
fprintf(fid,'%%This file was auto-generated by running an optimizer in RAVE.\n');
fprintf(fid,'%%This function is not meant to be called from the command line.\n');
fprintf(fid,'%%This function should be deleted when the optimizer finishes.\n');
fprintf(fid,'%%If no optimizers are currently running, you can safely delete this file.\n');
fprintf(fid,'%%This function was created with RAVEMAKEMINIMIZEFUNCTION.m\n');
fprintf(fid,'%%Copyright 2010, Matt Daskilewicz\n');
fprintf(fid,'\n\n');


%Print the function:


%distribute inputs to correct columns:

% namestr=repmat('%s;',1,numel(funnames));

fprintf(fid,'data=zeros(size(inputs,1),%s);\n',num2str(max(indexx)));
fprintf(fid,'data(:,%s)=inputs;\n',mat2str(indexx));
fprintf(fid,'num=%s;\n',mat2str(x));
% fprintf(fid,'name={''%s'';',funnames{1});
% for g=2:numel(funnames)
%    fprintf(fid,'''%s'';',funnames{g});
% end
% fprintf(fid,'};\n');

fprintf(fid,'settings=get(0,''userdata'');\n');
fprintf(fid,'src=settings.src;\n');
fprintf(fid,'d=%s;\n',num2str(d));
fprintf(fid,'narg=%s;\n',mat2str(handles.nargout{d}(x)));
fprintf(fid,'prefs=%s;\n',mat2str(handles.preferences{d}));
fprintf(fid,'targets=%s;\n',mat2str(handles.targetvalues{d}));
fprintf(fid,'response=raveevaluateregression(src,d,num,data);\n');


%account for different targets:

for g=1:numel(settings.targets)
   if settings.targets(g)==-inf
       %do nothing
   elseif settings.targets(g)==inf
       %multiply by -1
       fprintf(fid,'response(:,%1.0f)=-1*response(:,%1.0f);\n',g,g);
   else
       %nominal objective, minimize discrepency from target
       fprintf(fid,'response(:,%1.0f)=abs(response(:,%1.0f)-%s);\n',g,g,num2str(settings.targets(g)));
   end
end

fclose(fid);
rehash




