function ravegads(src,ev,settings)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

grabbyviz=get(handles.grabbers{handles.activeindex}(1),'visible');
bordercolor=get(handles.allblockers(handles.activeindex),'edgecolor');
%get frame bounds:
if settings.animation
    %open movie file:"
    set(handles.allblockers(handles.activeindex),'edgecolor',[1 1 1]);
    
    set(handles.grabbers{handles.activeindex},'visible','off');
    axisbounds=get(handles.allblockers(handles.activeindex),'pixelbounds'); %origin=top left
    figbounds=get(handles.fig,'position');
    handles.temp.framebounds=[axisbounds(1)+3 figbounds(4)-axisbounds(4)+3 axisbounds(3)-axisbounds(1)-4 axisbounds(4)-axisbounds(2)-4];
    if ~strcmp(handles.prefs.animation.format,'gif')
        [FileName,PathName,FilterIndex] = uiputfile('*.avi'); %#ok<NASGU>
        handles.temp.savepath=fullfile(PathName,FileName);
        if handles.prefs.animation.fps
            handles.temp.mov = avifile([PathName FileName],'compression','none','quality',100,'fps',handles.prefs.animation.fps);
        else
            handles.temp.mov = avifile([PathName FileName],'compression','none','quality',100,'fps',11);
        end
    else
        [FileName,PathName,FilterIndex] = uiputfile('*.gif'); %#ok<NASGU>
        handles.temp.savepath=fullfile(PathName,FileName);
        
        handles.temp.Framez=struct('cdata',{},'colormap',{});
        warning('off','MATLAB:writegif:loopcountInAppendMode');
    end
end

set(settings.startbutton,'enable','on') %turn off start button


%make stop button:
%origin=get(handles.tabaxes,'position'); %#ok<NASGU>
if ~isempty(settings.stopbuttonpos)
    handles.temp.stopbutton=uicontrol('position',settings.stopbuttonpos,...
        'style','togglebutton','string','PAUSE','backgroundcolor',handles.colors.buttonbg,...
        'callback','','enable','on','tag','stopbutton','foregroundcolor',[.8 0 0],'fontsize',12,'fontweight','bold');
    
    handles.tabobjects=union(handles.tabobjects,handles.temp.stopbutton);
    guidata(src,handles);
else
    stopbutton=[];
end

drawnow;

d=settings.d;
a=settings.a;
x=settings.x;


%generate random initial population:
thistag=get(src,'tag');
if strcmp(thistag,'startbutton')
    
elseif strcmp(thistag,'continuebutton')
    
end


guidata(handles.fig,handles);


%% make objective/constraint functions:
[functionname,nvars,mins,maxs]=ravemakeminimizefunction(src,ev,settings);
FITNESSFCN=str2func(functionname);

%% make options structure
switch settings.method
    case 'ga'
        options=gaoptimset(@ga);
        options.PopulationSize=settings.pop;
        options.CrossoverFraction=settings.pcross;
        options.EliteCount=settings.elitecount;
        options.MigrationDirection=settings.migrationdirection;
        options.MigrationInterval=settings.migrationinterval;
        options.MigrationFraction=settings.migrationfraction;
        options.Generations=settings.gens;
        options.TimeLimit=settings.timelimit;
        options.StallGenLimit=settings.stallgenlimit;
        options.TolFun=settings.tolfun;
        options.TolCon=settings.tolcon;
        options.SelectionFunction=settings.selectionfun;
        % options.CrossoverFcn=settings.crossoverfun;
        options.MutationFcn=@mutationadaptfeasible;
        % options.DistanceMeasureFcn=settings.distancefun;
    case 'gamultiobj'
        options=gaoptimset(@gamultiobj);
        options.PopulationSize=settings.pop;
        options.CrossoverFraction=settings.pcross;
        options.ParetoFraction=settings.pfrac;
        options.MigrationDirection=settings.migrationdirection;
        options.MigrationInterval=settings.migrationinterval;
        options.MigrationFraction=settings.migrationfraction;
        options.Generations=settings.gens;
        options.TimeLimit=settings.timelimit;
        options.StallGenLimit=settings.stallgenlimit;
        options.TolFun=settings.tolfun;
        options.TolCon=settings.tolcon;
        options.SelectionFunction=settings.selectionfun;
        % options.CrossoverFcn=settings.crossoverfun;
        % options.MutationFcn=settings.mutationfun;
        % options.DistanceMeasureFcn=settings.distancefun;  
    case 'simulatedannealing'
        options=saoptimset('simulannealbnd'); 
        options.AnnealingFcn=settings.annealfun;
        options.TemperatureFcn=settings.tempfun;
        options.AcceptanceFcn=settings.acceptfun;
        options.TolFun=settings.tolfun;
        options.StallIterLimit=settings.stalliter;
        options.MaxFunEvals=settings.maxfun;
        options.TimeLimit=settings.timelimit;
        options.MaxIter=settings.maxiter;
        options.ObjectiveLimit=settings.objlimit;
        options.InitialTemperature=settings.tinit;
        options.ReannealInterval=settings.reannealinterval;
    case 'patternsearch'
        options=psoptimset;
        options.Cache=settings.cache;
        options.CacheSize=settings.cachesize;
        options.CacheTol=settings.cachetol;
        options.CompletePoll=settings.completepoll;
        options.CompleteSearch=settings.completesearch;
        options.Display='final';
        options.InitialMeshSize=settings.initmeshsize;
        options.InitialPenalty=10;
        options.MaxFunEvals=settings.maxfun;
        options.MaxIter=settings.maxiter;
        options.MaxMeshSize=settings.maxmeshsize;
        options.MeshAccelerator=settings.meshaccel;
        options.MeshContraction=settings.meshcontract;
        options.MeshExpansion=settings.meshexpand;
        options.MeshRotate=settings.meshrotate;
        options.OutputFcn=[];
        options.PenaltyFactor=settings.penalty;
        options.PollingOrder=settings.pollorder;
        options.PollMethod=settings.pollmeth;
        options.ScaleMesh=settings.scalemesh;
        options.SearchMethod=settings.searchmeth;
        options.TimeLimit=settings.timelimit;
        options.TolBind=settings.bindtol;
        options.TolCon=settings.consttol;
        options.TolFun=settings.objtol;
        options.TolMesh=settings.meshtol;
        options.TolX=settings.inputvartol;
end

if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';



%% do optimizer:
tic
switch settings.method
    case 'ga'
        [X,FVAL,EXITFLAG,OUTPUT,population] = ga(FITNESSFCN,nvars,[],[],[],[],mins,maxs,[],options); %#ok<ASGLU>
    case 'gamultiobj'
        [X,FVAL,EXITFLAG,OUTPUT,population] = gamultiobj(FITNESSFCN,nvars,[],[],[],[],mins,maxs,options); %#ok<ASGLU>
    case 'simulatedannealing'
        [X,FVAL,EXITFLAG,OUTPUT] = simulannealbnd(FITNESSFCN,handles.xdials{d}{a}(settings.funvars),mins,maxs,options); %#ok<NASGU>
    case 'patternsearch'
        [X,FVAL,EXITFLAG,OUTPUT] = patternsearch(FITNESSFCN,handles.xdials{d}{a}(settings.funvars),[],[],[],[],mins,maxs,options); %#ok<NASGU>    
end

handles=guidata(src);

if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
indexx=settings.funvars;

if strcmp(settings.method,'simulatedannealing') || strcmp(settings.method,'patternsearch')
    population=X;
end


finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
finalpop(:,indexx)=population;

%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});

objective=raveevaluateregression(src,settings.d,respvars,finalpop);


%if ishandle(hObject) %turn the find frontier button back on
set(settings.startbutton,'enable','on','string','Restart')
%end

finalpop(:,respvars)=objective;

if ishandle(handles.temp.stopbutton)
    delete(handles.temp.stopbutton);
end
handles.tabobjects=setdiff(handles.tabobjects,handles.temp.stopbutton);
%
% %make continue button:
% %origin=get(handles.tabaxes,'position');
% settings.gens=T-t;
% if settings.gens==0
%     settings.gens=T;
% end
%
% if ~isempty(settings.stopbuttonpos)
% continuebutton=uicontrol('position',settings.stopbuttonpos,...
%     'style','pushbutton','string','Continue','backgroundcolor',handles.colors.buttonbg,...
%     'callback',{@ravensgaii,settings},'enable','on','tag','continuebutton','foregroundcolor',[0 0.8 0],'fontsize',12,'fontweight','bold');
% handles.tabobjects=union(handles.tabobjects,continuebutton);
% end

guidata(handles.fig,handles);

elapsed=toc/(24*3600);
timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.


%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

if settings.animation && strcmp(handles.prefs.animation.format,'gif') && ~all(FileName==0)
    if handles.prefs.animation.loop
        loopcount=inf;
    else
        loopcount=1;
    end
    if handles.prefs.animation.fps
        delaytime=1/handles.prefs.animation.fps;
    else
        delaytime=toc/settings.gens;
    end
    movie2gif(handles.temp.Framez,[PathName FileName],'LoopCount',loopcount,'DelayTime',delaytime);
elseif settings.animation && ~all(FileName==0)
    mov = close(handles.temp.mov); %#ok<NASGU>
end

warning('on','MATLAB:writegif:loopcountInAppendMode');

set(handles.allblockers(handles.activeindex),'edgecolor',bordercolor);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% handles.temp.lasttime=elapsed;
% handles.temp.lastgen=settings.gens;

[sortedpop,index]=sortrows(finalpop,x);
handles.temp.finalpop=sortedpop;
handles.temp.finalresp=objective(index,:);

set(handles.grabbers{handles.activeindex},'visible',grabbyviz);

guidata(handles.fig,handles);
