function ravearrangeobjects(src,ev)
% Arranges all axes, text, controls, etc related to a particular graph to make them
% fit within the bounds defined by the handles.allgraphs axes.
% Used by raveselectgraph (to move without resizing) and by ravemovegraph (to resize).
%should be called after calling raveresizeaxes, which makes handles.allgraphs axes
%fit in the boudns defined by handles.allblockers.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


persistent cbar

handles=guidata(src);
getudata;

%do type-specific stuff:
switch handles.graphinfo.(handles.activetype).axesformat
    case 'matrix'
        numy=size(handles.subax{handles.activeindex},1);
        numx=size(handles.subax{handles.activeindex},2);
        pos = floor(get(handles.activegraph,'position'));

        vspace=handles.prefs.matrixspacing;
        hspace=handles.prefs.matrixspacing;


        if strcmp(handles.activetype,'derivativeprofiler')
            %pos=pos;
            vspace=handles.prefs.profilervspacing;
            hspace=handles.prefs.profilerhspacing;
        end

        titel=findobj(handles.activegraph,'tag','title');
        titlestring=get(titel,'string');

        if isempty(titlestring)
            titoffset=0;
        else
            titoffset=10;
            if strcmp(handles.activetype,'derivativeprofiler')
                titoffset=25;
            end
        end

        width = floor((pos(3)-numx*hspace)/numx);
        height = floor((pos(4)-titoffset-numy*vspace)/numy);
        % pos(1:2) = pos(1:2) + [hspace*width vspace*height];

        ax=handles.subax{handles.activeindex};
        fakeax=handles.subfakeax{handles.activeindex};
        fakerax=handles.subfakerax{handles.activeindex};

        xlabelz=findall(ax,'tag','xticklabel'); %use findall since handle visibility is off.
        if pos(3)/numx<250 && strcmp(handles.activetype,'derivativeprofiler')
            set(xlabelz,'rotation',90,'horizontalalignment','right','verticalalignment','default')
        elseif strcmp(handles.activetype,'derivativeprofiler')
            set(xlabelz,'rotation',0,'horizontalalignment','center','verticalalignment','top')
        end

        %set up grid of axes.
        for r=numy:-1:1,
            for c=numx:-1:1,
                axPos = [pos(1)+(c-1)*(width+hspace) pos(2)+(numy-r)*(height+vspace) ...
                    width height];
                set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',uint64(axPos));

            end
        end

        %for scattermatrix, resize axis labels to fit on diagonals:
        labelz=findobj(handles.subax{handles.activeindex},'tag','axeslabel');
        maxchars=round(width/8);%approx 8 pixel per character
        udata=get(handles.fakeax(handles.activeindex),'userdata');
        for g=1:numel(labelz)
            newstring=linewrap(handles.allnames{udata.view.dataset}{udata.view.x(g)},maxchars);
            set(labelz(g),'string',newstring);
        end

    case 'row' %need to move subax and statsax (if stats are shown.)

        udata=get(handles.fakeax(handles.activeindex),'userdata');


        numvars=numel(handles.subax{handles.activeindex});

        ax=handles.subax{handles.activeindex};
        fakeax=handles.subfakeax{handles.activeindex};
        fakerax=handles.subfakerax{handles.activeindex};
        statsax=handles.statsax{handles.activeindex};

        if udata.view.showstats
            margin=150;
        else
            margin=20;
        end

        pos=get(handles.activegraph,'position');

        xlabelspace=20;
        ylabelspace=32;

        if  pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
            width = pos(3)/numvars-ylabelspace+min(5,ylabelspace/numvars);
            height = pos(4)-xlabelspace-margin;
            space = .03; % 3 percent space between subaxes
            pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];

        else %vertical stacking
            width=pos(3)-ylabelspace-margin;
            height=pos(4)/numvars-xlabelspace;
            space = .03; % 3 percent space between subaxes
            pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
        end

        if width<3;width=3;end
        if height<3;height=3;end

        for c=1:numvars

            if pos(3) > pos(4) %HORIZONTAL STACKING
                axPos = [pos(1)+(c-1)*(width+ylabelspace)+ylabelspace pos(2)+margin ...
                    width-ylabelspace height*(1-space)]; %subtract margin to make space for statistics
                statsPos=[axPos(1) axPos(2)-150 axPos(3) 100];
            else
                axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
                    width-ylabelspace height*(1-space)-20]; %subtract margin to make space for statistics
                statsPos=[axPos(1)+width-ylabelspace+10 axPos(2)-15 150 100];
            end

            set([ax(c) fakerax(c) fakeax(c)],'position',axPos)
            if udata.view.showstats
                set(statsax(c),'position',statsPos);

                if height<100 || width<150
                    statsize=8;
                    set(statsax(c),'pos',statsPos.*[1 1 1 .8]);
                    statz=findobj(statsax(c),'tag','histogramstat');
                    set(statz,'fontsize',statsize);
                else
                    statsize=10;
                    statz=findobj(statsax(c),'tag','histogramstat');
                    set(statz,'fontsize',statsize);
                end


            end

            set([fakerax(c) fakeax(c)],'xlim',get(ax(c),'xlim'),'ylim',get(ax(c),'ylim'),'zlim',get(ax(c),'zlim'));
            set([fakerax(c) fakeax(c)],'xtick',get(ax(c),'xtick'),'ytick',get(ax(c),'ytick'),'ztick',get(ax(c),'ztick'))
        end
    case '3D'
        ravesynchaxes;
    case 'custom'

        if ~isempty(which(['ravearrangeobjects',handles.activetype]))
            eval(['ravearrangeobjects',handles.activetype,'(handles.fig,[])']);
        end

end

%resynch axes limits (they can get screwed up if axes is too small)
%THIS IS NOW HANDLED BY LISTENERS SET UP IN RAVENEWGRAPH
%set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),...
%    'ylim',get(handles.activegraph,'ylim'),'xtick',get(handles.activegraph,'xtick'),'ytick',get(handles.activegraph,'ytick'));

%move the colorbar too:
if handles.graphinfo.(handles.activetype).hascolorbar
    newpos=get(handles.activegraph,'position');
    if udata.format.colorbar
        if isempty(cbar)
            legdata=get(handles.fakerax(handles.activeindex),'userdata');
            cbar=findobj(legdata,'tag','Colorbar');
        end
        set(cbar,'pos',[newpos(1)+newpos(3)+30 newpos(2) 27 newpos(4)]);
    end
end


