function f=ravechangerangesgui(d,a,oldhandles)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


%make figure:
f=figure('units','pixels','position',[0 0 651 700],'name',...
    'Define Roles and Data Types','numbertitle','off','menubar','none','resize','off');
handles.gui=f;
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

handles.d=d;
handles.a=a;
handles.oldhandles=oldhandles;

%get theme
load raveprefs
handles.prefs=raveprefs;
set(f,'color',raveprefs.colors.frame)


%make initial uicontrols:
handles.background=axes('units','pixels','position',[88 55 506 570],'visible','off','xlimmode','manual','ylimmode','manual');

handles.tabaxes=axes('units','pixels','position',[58 75 30 550],'parent',handles.gui,'visible','off','tag','tabaxes');
handles.titleaxes=axes('units','pixels','position',[11 660 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
handles.tabbg=patch(x,y,raveprefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],raveprefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',[.5 .5 .5]);

handles.constant=text(.2,v(1),' Constant','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.text,'vert','cap','buttondownfcn',@constant_tab);
handles.continuous=text(.2,v(2),' Continuous','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@continuous_tab);
handles.integer=text(.2,v(3),' Integer','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@integer_tab);
handles.discrete=text(.2,v(4),' Discrete','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@discrete_tab);
handles.string=text(.2,v(5),' String','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@string_tab);
handles.logical=text(.2,v(6),' Logical','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@logical_tab);

handles.tabs=[handles.constant handles.continuous handles.integer handles.discrete handles.string handles.logical];

line([0 0 1 1 0],[.355 0 0 1 1],'color',raveprefs.colors.tabfg,'parent',handles.background);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Ok','callback',@donebutton_Callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Cancel','callback',@cancelbutton_Callback);


%make titles
set(handles.titleaxes,'color','none','xcolor',raveprefs.colors.frame,...
    'ycolor',raveprefs.colors.frame,'zcolor',raveprefs.colors.frame);


text(0,.2,'Change feasible values of input variables','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');


%get data from managedatagui
handles.indata=guidata(gcbo); %gcbo = "define roles" button in managedatagui

handles.savebutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[320 65 120 26],...
    'string','Save Settings','callback',@savesettings_callback,'enable','off');
handles.loadbutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[460 65 120 26],...
    'string','Load Settings','callback',@loadsettings_callback,'enable','off');

% Choose default command line output
handles.output = f;
handles.tabobjects=[];

% Update handles structure
guidata(f, handles);

%make table:
dblArray=java.lang.Object; %the table data
dmodel=javax.swing.table.DefaultTableModel(dblArray,{' '});
jtable=javax.swing.JTable(dmodel);
jtable.setAutoResizeMode(jtable.AUTO_RESIZE_OFF);
jscroll=javax.swing.JScrollPane(jtable);
[jhandle mhandle]=javacomponent(jscroll,[120 100 440 460],handles.gui);

handles.blankmodel=dmodel;
handles.jtable=jtable;

guidata(f, handles);

%show first tab:
constant_tab(handles.constant,[]);


function constant_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};
thistype=strcmp('constant',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
data=handles.oldhandles.xdials{d}{a}(thistype);

if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',numel(varnamez),2);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Double(data(r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name','     Value      '});
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@constantchanged_callback,handles.gui})
end

guidata(src,handles);

function continuous_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);

varnamez=handles.oldhandles.allnames{d};
thistype=strcmp('continuous',handles.oldhandles.datatype{d});
varnamez=varnamez(thistype);
data=[handles.oldhandles.minlimit{d}{a}(thistype);handles.oldhandles.maxlimit{d}{a}(thistype)];

if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Double(data(1,r));
        tableData(r,3)=java.lang.Double(data(2,r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name            ','   Min Value    ','    Max Value    '});
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@continuouschanged_callback,handles.gui})
end

guidata(src,handles);


function integer_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};
thistype=strcmp('integer',handles.oldhandles.datatype{d});
varnamez=varnamez(thistype);
data=[handles.oldhandles.minlimit{d}{a}(thistype);handles.oldhandles.maxlimit{d}{a}(thistype)];

if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Integer(data(1,r));
        tableData(r,3)=java.lang.Integer(data(2,r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name            ','   Min Value    ','    Max Value    '});
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@integerchanged_callback,handles.gui})
end

guidata(src,handles);


function discrete_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('discrete',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.oldhandles.discretevalues{d}{a}(thistype);

datasizes=cellfun(@size,tempdata','UniformOutput',0);

datasizes=cell2mat(datasizes);
if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=num2cell(tempdata{v});
    end
else
    data=cell(0);
end


if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',size(data,1)+10,3);
    for c=1:numel(varnamez)
        for r=1:size(data,1);
            if ~isempty(data{r,c})
                tableData(r,c)=java.lang.Double(data{r,c});
            end
        end
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@discretechanged_callback,handles.gui})
end

guidata(src,handles);


function string_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('string',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.oldhandles.discretevalues{d}{a}(thistype);
datasizes=cellfun(@size,tempdata','UniformOutput',0);
datasizes=cell2mat(datasizes);

if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=tempdata{v};
    end
else
    data=cell(0);
end

if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',size(data,1)+10,3);
    for c=1:numel(varnamez)
        for r=1:size(data,1);
            if ~isempty(data{r,c})
                tableData(r,c)=java.lang.String(data{r,c});
            end
        end
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@stringchanged_callback,handles.gui})
end
guidata(src,handles);

function logical_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected


[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};
thistype=strcmp('logical',handles.oldhandles.datatype{d});
varnamez=varnamez(thistype);
falsedata=handles.oldhandles.minlimit{d}{a}(thistype);
falsedata=~falsedata;
truedata=logical(handles.oldhandles.maxlimit{d}{a}(thistype));

data=[falsedata;truedata];

if isempty(varnamez)
    handles.jtable.setModel(handles.blankmodel);
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Boolean(data(1,r));
        tableData(r,3)=java.lang.Boolean(data(2,r));
    end
    
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name            ','   False    ','    True    '});
    handles.jtable.setModel(dmodel);
    set(dmodel,'TableChangedCallback',{@logicalchanged_callback,handles.gui})
end

guidata(src,handles);



function donebutton_Callback(src, eventdata)
handles=guidata(src);

set(0,'userdata',handles.oldhandles);
close(handles.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);



function constantchanged_callback(jsrc,jev,src)
handles=guidata(src);

%determine which cell is selected:
eventDetails = jev;

c=eventDetails.getColumn;
r=eventDetails.getFirstRow;

newvalue = handles.jtable.getValueAt(r,c);

if ischar(newvalue);newvalue=str2double(newvalue);end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('constant',datatypes));

index=thistype(r+1);

[handles.oldhandles.xdials{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.minlimit{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(newvalue);

guidata(src,handles);

function continuouschanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev;

r=eventDetails.getFirstRow;

newmin = handles.jtable.getValueAt(r,1);
newmax = handles.jtable.getValueAt(r,2);

if ischar(newmin);newmin=str2double(newmin);end
if ischar(newmax);newmax=str2double(newmax);end

if newmin>newmax
    errordlg('Minimum value must be less than or equal to maximum value','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('continuous',datatypes));

index=thistype(r+1);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(newmin);
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(newmax);

guidata(src,handles);


function integerchanged_callback(jsrc,jev,src)

handles=guidata(src);

eventDetails = jev;
r=eventDetails.getFirstRow;

newmin = handles.jtable.getValueAt(r,1);
newmax = handles.jtable.getValueAt(r,2);

if ischar(newmin);newmin=str2double(newmin);end
if ischar(newmax);newmax=str2double(newmax);end

if newmin>newmax
    errordlg('Minimum value must be less than or equal to maximum value','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('continuous',datatypes));

index=thistype(r+1);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(round(newmin));
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(round(newmax));

guidata(src,handles);


function discretechanged_callback(jsrc,jev,src)
handles=guidata(src);

%determine which cell is selected:
eventDetails = jev;
c=eventDetails.getColumn;
R=handles.jtable.getRowCount;

newdata=cell(R,1);
for r=1:numel(newdata)
    data=handles.jtable.getValueAt(r-1,c);
    
    if ischar(data)
        try
            newdata{r}=str2double(data);
        catch %#ok<CTCH>
            newdata{r}=[]; %if they didnt enter a number
        end
    else
        newdata{r}=data;
    end
end

newdata=cell2mat(newdata);
newdata=unique(newdata); %sort and remove repeats

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('discrete',datatypes));

index=thistype(c+1);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(min(newdata));
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(max(newdata));
[handles.oldhandles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function stringchanged_callback(jsrc,jev,src)
handles=guidata(src);

eventDetails = jev;

c=eventDetails.getColumn;
R=handles.jtable.getRowCount;

newdata=cell(R,1);
for r=1:numel(newdata)
    newdata{r}=handles.jtable.getValueAt(r-1,c);
end

charz=~cellfun(@isempty,newdata);

%make charz true for all values that appear in alldata, even if their
%string is now empty:

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('string',datatypes));
index=thistype(c+1);

truevalues=unique(handles.oldhandles.alldata{d}(:,index));
charz(truevalues)=true;

newdata=newdata(charz);

%updata alltext to reflect new names:
handles.oldhandles.alltext{d}(:,index)=newdata(handles.oldhandles.alldata{d}(:,index));
%[handles.oldhandles.minlimit{d}{a}(index)]=deal(min(newdata)); %minlimit is always 1 for strings
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(numel(newdata));
[handles.oldhandles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function logicalchanged_callback(jsrc,jev,src)

handles=guidata(src);

eventDetails = jev;
r=eventDetails.getFirstRow;

newmin = handles.jtable.getValueAt(r,1);
newmax = handles.jtable.getValueAt(r,2);

if ischar(newmin);newmin=logical(eval(newmin));end
if ischar(newmax);newmin=logical(eval(newmax));end

if ~newmin && ~newmax
    errordlg('You must enable at least one option for each variable','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('logical',datatypes));

index=thistype(r+1);

if newmin
    [handles.oldhandles.minlimit{d}{a}(index)]=deal(0);
else
    [handles.oldhandles.minlimit{d}{a}(index)]=deal(1);
end

if ~newmax
    [handles.oldhandles.maxlimit{d}{a}(index)]=deal(0);
else
    [handles.oldhandles.maxlimit{d}{a}(index)]=deal(1);
end

guidata(src,handles);




