function ravedragview(src,ev)
%drag around workspace.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



%NOTE: this function depends on using varargin instead of specifying an exact number of inputs,
%so don't change that in the future.

%ravemoveview(handles.fig,[],clicked);
%clicked is optional

handles=guidata(src);

%get existing position and click location
handles.temp.firstclick=get(handles.mainaxes,'currentpoint');
handles.temp.oldviewpos=get(handles.currentview,'position');
% wsextent=get(handles.wspanel,'position');

%allow dragging

guidata(src,handles);

set(handles.fig,'WindowButtonMotionFcn',@ravedragviewmove);
set(handles.fig,'WindowButtonUpFcn',@movedone);

handdata=[nan nan 1 1 nan 1 1 nan 1 1 nan nan nan;
    nan 1 2 2 1 2 2 1 2 2 1 1 nan;
    nan 1 2 2 2 2 2 2 2 2 1 2 1;
    nan nan 1 2 2 2 2 2 2 2 2 2 1;
    nan 1 1 2 2 2 2 2 2 2 2 2 1;
    1 2 2 2 2 2 2 2 2 2 2 2 1;
    1 2 2 2 2 2 2 2 2 2 2 2 1;
    1 2 2 2 2 2 2 2 2 2 2 1 nan;
    nan 1 2 2 2 2 2 2 2 2 2 1 nan;
    nan nan 1 2 2 2 2 2 2 2 1 nan nan;
    nan nan nan 1 2 2 2 2 2 2 1 nan nan;
    nan nan nan 1 2 2 2 2 2 2 1 nan nan];

handdata=[handdata NaN(12,3)];
handdata=[handdata;NaN(4,16)];

set(handles.fig,'pointer','custom','pointershapecdata',handdata,'pointershapehotspot',[6,6]);




function ravedragviewmove(src,ev)
handles=guidata(src);

newclick=get(handles.mainaxes,'currentpoint');

try
shiftx=handles.temp.firstclick(1,1)-newclick(1,1);
shifty=handles.temp.firstclick(1,2)-newclick(1,2);
catch
    return
end


handles.temp.firstclick=newclick;


%newpos=[shiftx+handles.temp.oldviewpos(3)/2 shifty+handles.temp.oldviewpos(4)/2 0 0]+handles.temp.oldviewpos;

minh=get(handles.hscroll,'min');
maxh=get(handles.hscroll,'max');

oldhvalue=get(handles.hscroll,'value');
newhvalue=oldhvalue+shiftx;

if newhvalue>minh && newhvalue<maxh
set(handles.hscroll,'value',oldhvalue+shiftx)
elseif newhvalue<=minh
    set(handles.hscroll,'value',minh)
else
    set(handles.hscroll,'value',maxh)
end

minv=get(handles.vscroll,'min');
maxv=get(handles.vscroll,'max');

oldvvalue=get(handles.vscroll,'value');
newvvalue=oldvvalue+shifty;

if newvvalue>minv && newvvalue<maxv
    set(handles.vscroll,'value',oldvvalue+shifty)
elseif newvvalue<=minv
    set(handles.vscroll,'value',minv)
else
    set(handles.vscroll,'value',maxv)
end

clicked=[get(handles.hscroll,'value') get(handles.vscroll,'value')];

%center box on click location
clicked(1,1,1)=clicked(1,1,1)-handles.temp.oldviewpos(3)/2;
clicked(1,2,1)=clicked(1,2,1)-handles.temp.oldviewpos(4)/2;

%round to nearest integers:
clicked=floor(clicked);

cvpos=[clicked(1,1,1) clicked(1,2,1) handles.temp.oldviewpos(3) handles.temp.oldviewpos(4)];


%move to new location
set(handles.currentview,'position',[clicked(1,1,1) clicked(1,2,1) handles.temp.oldviewpos(3) handles.temp.oldviewpos(4)]);


%update workspace to show new view:
%calculate where to put it.
mainaxespos=get(handles.wspanel,'position');
%cvpos=get(handles.currentview,'position');
wspos=handles.workspacepos;
wsextent=get(handles.mainaxes,'position');

mainaxespos(1)=wspos(1)-cvpos(1);
mainaxespos(2)=wspos(2)-cvpos(2);

if cvpos(1)==0
    mainaxespos(1)=mainaxespos(1)+0;
end

if cvpos(1)+cvpos(3)>=wsextent(3)
    mainaxespos(1)=mainaxespos(1)+2;
end

if cvpos(2)==0
    mainaxespos(2)=mainaxespos(2)-2;
end

if cvpos(2)+cvpos(4)>=wsextent(4)
    mainaxespos(2)=mainaxespos(2)+1;
end

set(handles.wspanel,'position',mainaxespos);


%need to update this AFTER moving the main axes:
newclick=get(handles.mainaxes,'currentpoint');
handles.temp.firstclick=newclick;

guidata(src,handles);


%fix table uipanel bug:
for ai=1:numel(handles.allgraphs)
   if ishandle(handles.fakerax(ai)) && strcmp(get(handles.fakerax(ai),'type'),'uipanel');
       p=handles.fakerax(ai);
       ppos=get(p,'position');
       set(p,'position',ppos); %makes table redraw in correct position
   end
end








function movedone(src,eventdata)
%load handles
handles=guidata(src);

%stop dragging
set(handles.fig,'WindowButtonMotionFcn',[]);
set(handles.fig,'windowbuttonupfcn',[]);
set(handles.fig,'pointer','arrow');


