function h=ravedrawconstraint2D(targetaxes,xcol,ycol,d,a,thisconstraint,ai)
%draws a 2-D contour-style constraint in axes targetaxes.
%xvar is the index of targetaxes's x-axis variable.
%yvar is the index of targetaxes's y-axis variable.
%d is the index of the dataset.
%a is the index of the analysis.
%thisconstraint is hte index of the constraint.
%ai is the index of the graphic that contains axes in targetaxes (either as the main axes or as a subax)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

h=[];

handles=guidata(targetaxes);

%prepare data for gridding:
xcol=[xcol ycol]; %#ok<AGROW>

datatypes=handles.datatype{d}(xcol);
totalrows=zeros(size(datatypes));
numpts=handles.prefs.constraintgridsize;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end

%precompute data so contour levels can be shared among all plots:
allthedata=cell(1,3);

thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,xcol(2))=ycalldata;

%evaluate the functions:
ignore=false(1,size(constants,2));
ignore([xcol ycol])=true;
Z=raveevaluateconstraint(targetaxes,thisconstraint,constants,ignore);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(Z,numel(thisydata),numel(thisxdata));

% zdataz=allthedata(3);
% zdataz=zdataz(:);
% zdataz=cell2mat(zdataz);
% zdataz=zdataz(:);
% 
% lowestvalue=min(zdataz);
% highestvalue=max(zdataz);

zplotdata=allthedata{3};
levelz=[0 0];
% if ~udata.format.reversecolors
%     zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
%     levelz=(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
% else
%     zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
%     levelz=128-(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
% end

try
    C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,[levelz levelz]);
catch %#ok<CTCH>
    C=[];
end


if isempty(C);return;end


h=ravedrawconstraintcore(handles.activegraph,ai,C,thisconstraint,xcol,ycol,2);
% 
% set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),'ylim',...
%     get(handles.activegraph,'ylim'));
