function yout=raveevaluateregression(src,d,outnum,data)
%evaluates the function for the columns in "outnum" in dataset "d" using the input data stored in the matrix
%"data".
%if num is a vector, evaluates all named functions and
%returns all responses as columns of a matrix in the order the appear in num, attempting to avoid calling the same function
%multiple times. All responses must share the same input data (data)
%
%NOTE: this function des not enforce discrete variable values. That must be handled by whatever
%calls this.
%
%NOTE: if any of the inputs are strings, data will be a cell array instead
%of a double array.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
prefs=handles.preferences{d};
targets=handles.targetvalues{d};

%get unique rows:

[udata,junk,rowidx]=unique(data,'rows');


%deterine dependency:
num=sort(outnum,'descend');
f=numel(num);
while f<=numel(num)
    thesefunvars=handles.funvars{d}{num(f)};
    num=union(num,thesefunvars(handles.isfunction{d}(thesefunvars)));
    f=f+1;
end

name=handles.funnames{d}(num);
narg=handles.nargout{d}(num);

if ~iscell(name) %one response only
    if handles.isfunction{d}(num)
        xfun=str2func(name);
        alloutputs=cell(1,nargout(xfun));
        if nargin(xfun)==3
            [alloutputs{:}]=xfun(udata,prefs,targets);
        else
            [alloutputs{:}]=xfun(udata);
        end
        
        
        y=alloutputs{narg};
    elseif num<=size(udata,2)
        y=udata(:,num);
    end


else %multiple responses:
    
    %enlarge (if necessary) data to have as many columns as largest index:
    if size(udata,2)<max(num)
        udata=[udata zeros(size(udata,1),max(num)-size(udata,2))];        
    end

    isfuns=find(handles.isfunction{d}(num));
    if ~isempty(isfuns)

        [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
        funname=name(isfuns);
        uniquefuns = funname(sort(index)); %extract unique funs without changing their evaluation order.
        
        for g=1:numel(uniquefuns);
            thisfun=uniquefuns{g};
            isthisfun=find(strcmp(name,thisfun)); %logical, responses matching current function
            xfun=str2func(thisfun);
            alloutputs=cell(1,nargout(xfun));
            if nargin(xfun)==3
                [alloutputs{:}]=xfun(udata,prefs,targets);
            else
                [alloutputs{:}]=xfun(udata);
            end
            alloutputs=cell2mat(alloutputs);
            udata(:,num(isthisfun))=alloutputs(:,narg(isthisfun));
            
            %y(:,isthisfun)=alloutputs(:,narg(isthisfun));
        end

    end
    y=udata(:,outnum);
end

yout=y(rowidx,:);













