function [cmap cmapnames]=ravemakecolormap(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src); %custom colormaps are stored in handles;

%make the global colormap to apply to rave figure. includes builtin matlab colormaps,
%colorblind-safe colormaps, and 3 user defined colormaps. 

%http://geography.uoregon.edu/datagraphics/color_scales.htm
%Green to Magenta, 16 steps 
baseMap=[0.000   0.316   0.000;
   0.000   0.526   0.000;
   0.000   0.737   0.000;
   0.000   0.947   0.000;
   0.316   1.000   0.316;
   0.526   1.000   0.526;
   0.737   1.000   0.737;
   1.000   1.000   1.000;
   1.000   0.947   1.000;
   1.000   0.737   1.000;
   1.000   0.526   1.000;
   1.000   0.316   1.000;
   0.947   0.000   0.947;
   0.737   0.000   0.737;
   0.526   0.000   0.526;
   0.316   0.000   0.316];
idx1 = linspace(0,1,size(baseMap,1));
idx2 = linspace(0,1,128);
greenmagenta = interp1(idx1,baseMap,idx2);

%http://geography.uoregon.edu/datagraphics/color_scales.htm
%Dark Red to Blue, 18 steps, based on ColorBrewer RdYlBu_11              
baseMap=[0.142   0.000   0.850; 
   0.097   0.112   0.970; 
   0.160   0.342   1.000; 
   0.240   0.531   1.000; 
   0.340   0.692   1.000; 
   0.460   0.829   1.000; 
   0.600   0.920   1.000;
   0.740   0.978   1.000; 
   0.920   1.000   1.000; 
   1.000   1.000   0.920;  
   1.000   0.948   0.740;  
   1.000   0.840   0.600; 
   1.000   0.676   0.460; 
   1.000   0.472   0.340; 
   1.000   0.240   0.240; 
   0.970   0.155   0.210; 
   0.850   0.085   0.187; 
   0.650   0.000   0.130];
idx1 = linspace(0,1,size(baseMap,1));
idx2 = linspace(0,1,128);
bluered = interp1(idx1,baseMap,idx2);


%Blue to Dark Orange, 18 steps       
baseMap=[0.000   0.400   0.400
   0.000   0.600   0.600
   0.000   0.800   0.800
   0.000   1.000   1.000
   0.200   1.000   1.000
   0.400   1.000   1.000
   0.600   1.000   1.000
   0.700   1.000   1.000
   0.800   1.000   1.000
   0.900   1.000   1.000
   1.000   0.900   0.800
   1.000   0.793   0.600
   1.000   0.680   0.400
   1.000   0.560   0.200
   1.000   0.433   0.000
   0.800   0.333   0.000
   0.600   0.240   0.000
   0.400   0.153   0.000];
idx1 = linspace(0,1,size(baseMap,1));
idx2 = linspace(0,1,128);
blueorange = interp1(idx1,baseMap,idx2);

%customs:
idx2 = linspace(0,1,128);
idx1 = linspace(0,1,size(handles.prefs.colormap1,1));
custom1=interp1(idx1,handles.prefs.colormap1,idx2);
idx1 = linspace(0,1,size(handles.prefs.colormap2,1));
custom2=interp1(idx1,handles.prefs.colormap2,idx2);
idx1 = linspace(0,1,size(handles.prefs.colormap3,1));
custom3=interp1(idx1,handles.prefs.colormap3,idx2);

hsvz=hsv(144);



cmap=[hsvz(1:128,:);jet(128);hot(128);cool(128);spring(128);summer(128);autumn(128);winter(128);gray(128);bluered;blueorange;greenmagenta;custom1;custom2;custom3];


cmapnames={'HSV';'Jet';'Hot';'Cool';'Spring';'Summer';'Autumn';'Winter';'Grayscale';'Blue-Red';'Blue-Orange';'Green-Magenta';'Custom 1';'Custom 2';'Custom 3'};



cmap(cmap<0)=0;
cmap(cmap>1)=1;




%for reference, the default raveprefs custom colormaps are:
%{

raveprefs.colormap1
=[246 239 247
    189 201 225
    103 169 207
    28 144 153
    1 108 89]./255;

raveprefs.colormap2
=[215 25 28
    253 174 97
    255 255 191
    166 217 106 
    26 150 65]./255;

%new colormap3 (for row colors)
[55 126 184;
    255 127 0;
    44 160 44;
    228 26 28;
    152 78 163;
    174 199 232;
    255 187 120;
    152 223 138;
    255 152 150;
    197 176 213]./255;

old colormap 3
=[241 238 246
    215 181 216
    223 101 176
    221 28 119
    152 0 67]./255;

%}