function [linecolors,markercolors]=ravemakelinecolors(src,ev,numlines,nummarkers) %#ok<INUSL>
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


numlines=max(numlines,nummarkers);

linecolormap=handles.cmapnames{udata.format.lcmap};
lindex=find(strcmp(handles.cmapnames,linecolormap));
markercolormap=handles.cmapnames{udata.format.mcmap};
mindex=find(strcmp(handles.cmapnames,markercolormap));

wholemap=get(handles.fig,'colormap');

if numlines>6      
        cmap=wholemap(128*(lindex-1)+1:128*lindex,:) ;
        idx1 = linspace(0,1,size(cmap,1));
        idx2 = linspace(0,1,numlines);
        linecolors = interp1(idx1,cmap,idx2);
else
    goodcolors=getgoodcolors(linecolormap,lindex,wholemap,numlines);
    linecolors=goodcolors(1:numlines,:);
end

if nummarkers>6      
        cmap=wholemap(128*(mindex-1)+1:128*mindex,:) ;
        idx1 = linspace(0,1,size(cmap,1));
        idx2 = linspace(0,1,nummarkers);
        markercolors = interp1(idx1,cmap,idx2);
else
    goodcolors=getgoodcolors(markercolormap,mindex,wholemap,nummarkers);
    markercolors=goodcolors(1:nummarkers,:);
end





        
function goodcolors=getgoodcolors(cmapname,index,wholemap,numlines)

switch lower(cmapname)
    case 'hsv'
        goodcolors=[0 .5 1;
         1 0 .5;
         0 1 .5;
         .5 0 1;
         1 .5 0;
         .5 1 0];



    case 'jet'
        
        goodcolors=[0 64 255
            128 0 0
            0 0 159
            255 48 0
            0 255 255
            255 255 0]./255;
    case 'gray'
        
        goodcolors=[0 0 0
            121 121 121
            198 198 198
            255 255 255 
            64 64 64
            162 162 162]./255;
        
    case 'hot'
        goodcolors=[255 0 0
            255 255 0
            128 0 0
             255 128 0
             42 0 0
             255 255 175]./255;
         
    case 'spring'
        goodcolors=[255 0 255
            255 255 0
            255 77 178
            255 158 97
            255 206 48
            255 36 219]./255;       
    case 'summer'
        goodcolors=[0 128 102
            255 255 102
            101 178 102
            190 223 102
            113 211 58
            25 174 46]./255;       
    case 'winter'
        goodcolors=[0 0 225;
            0 225 128
            0 97 206
            0 178 106
            0 225 207
            0 127 255]./255;        
    case 'autumn'
        goodcolors=[255 0 0
            255 255 0
            255 77 0
            255 190 0
            255 133 0
            204 102 0]./255;
    otherwise       
        cmap=wholemap(128*(index-1)+1:128*index,:) ;
        idx1 = linspace(0,1,size(cmap,1));
        idx2 = linspace(0,1,numlines);
        goodcolors = interp1(idx1,cmap,idx2);

end

