function ravemakenewfitcolumn(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

indata=guidata(src);


%make figure:
f=figure('units','pixels','position',[0 0 651 734],'name',...
    'Make Fit-Data Column','numbertitle','off','menubar','none','resize','off');
handles.gui=f;
origin=get(indata.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
load raveprefs
handles.prefs=raveprefs;
set(f,'color',raveprefs.colors.frame)

%make initial uicontrols:
handles.background=axes('units','pixels','position',[88 55 506 597],'visible','off','xlimmode','manual','ylimmode','manual');
   
handles.tabaxes=axes('units','pixels','position',[58 207 30 445],'parent',handles.gui,'visible','off','tag','tabaxes');  
handles.titleaxes=axes('units','pixels','position',[11 699 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

disclaimer=uicontrol('style','text','parent',f,'position',[26 689-36 560 46],'hor','left',...
    'string','Each tab below offers a different method to select a subset of rows in the data set. A new column will be added to the data set with the value "0" for fit rows, "1" for validation rows, and "2" for test rows. These values can then be used to indicate which rows to use as a basis for fitting a model.',...
    'backgroundcolor',get(f,'color'));



[x,y,v]=calculatetabprofile([1 1 1.2 .8],.1,2,1); 
handles.tabbg=patch(x,y,raveprefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],raveprefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',[.5 .5 .5]);

handles.random=text(.2,v(1),' Random','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.text,'vert','cap','buttondownfcn',@random_callback);
handles.logic=text(.2,v(2),' Logical','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@logic_callback);
handles.selection=text(.2,v(3),' Selection','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@selection_callback);
handles.range=text(.2,v(4),' Range','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@range_callback);


line([0 0 1 1 0],[.253 0 0 1 1],'color',raveprefs.colors.tabfg,'parent',handles.background);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Ok','callback',@donebutton_Callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
        'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Cancel','callback',@cancelbutton_Callback);
    
%make titles
text(0,.2,'Make new fit-data column','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
   'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

%get data from managedatagui
handles.indata=indata; %gcbo = "define roles" button in managedatagui


handles.output = f;
handles.tabobjects=[];
% Update handles structure
guidata(f, handles);
%show data types tab:

random_callback(handles.random,[]);





function random_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1 1.2 .8],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.logic handles.selection handles.range],'color',handles.prefs.colors.tabfg);

%make uicontrols:
handles.tabobjects=[];
guidata(src,handles);



function logic_callback(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1 1.2 .8],.1,2,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.random handles.selection handles.range],'color',handles.prefs.colors.tabfg);


guidata(src,handles);

function selection_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1 1.2 .8],.1,3,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.random handles.logic handles.range],'color',handles.prefs.colors.tabfg);


%make uicontrols:
handles.tabobjects=[];
guidata(src,handles);

function range_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1 1.2 .8],.1,4,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.logic handles.selection handles.random],'color',handles.prefs.colors.tabfg);

%make uicontrols:
handles.tabobjects=[];
guidata(src,handles);


% --- Executes on button press in donebutton.
function donebutton_Callback(src, eventdata)
handles=guidata(src);

%isresponse=ismember(handles.names,get(handles.ybox,'string'));

%set(0,'userdata',handles.indata.indata);
close(handles.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);





