function ravemaketextticks(src,ev,xvar,yvar,zvar,cvar) %#ok<INUSL>
%src must be handle of axes for which to apply text labels.
%only handles one variable for each of the 4 inputs... if any of the 4 inputs is a vector, error will occur.
%for matrix graphs must call this for each of the subgraphs.
%cvar only affects colorbar. zvar only affects 3-d graphs.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;
a=udata.view.analysis;

fakeax=handles.fakeax(handles.activeindex);
fakerax=handles.fakerax(handles.activeindex);

if ~isempty(xvar) %&& handles.istext{udata.view.dataset}(xvar);
    switch handles.datatype{d}{xvar}
        case {'constant','continuous','integer'}
            set(src,'xtickmode','auto','xticklabelmode','auto');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xtick',get(src,'xtick'),...
                'xticklabel','');
            
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{xvar};
            if numel(discretez)<=12;
                %ticknumbers=unique(handles.alldata{d}(:,xvar));
                set(src,'xtick',discretez,'xticklabel',discretez,'xminortick','off');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',get(src,'ytick'),...
                    'yticklabel','');
            else %too many, just treat as continuous
                set(src,'ytickmode','auto','yticklabelmode','auto');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',get(src,'ytick'),...
                    'yticklabel','');
            end
            
        case 'string'
            thesetexts=handles.discretevalues{d}{a}{xvar};
            ticknumbers=unique(handles.alldata{d}(:,xvar));
            
            set(src,'xtick',ticknumbers,'xminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xtick',ticknumbers,'xticklabel',thesetexts,'xminortick','off');
            
            set(src,'XTickLabel',[]);
            set(fakeax,'XTickLabel',[]);
            set(fakerax,'XTickLabel',[]);
            
            usethesevalues=handles.discretevalues{d}{a}{xvar};
            putthemhere=unique(handles.alldata{d}(:,xvar));
            
            ylimits=get(src,'Ylim');
            ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
            
            if ~isfield(udata.format,'orient') || strcmp(udata.format.orient,'vertical')
                rot=90;                
            else
                rot=0;
            end
                
            
            
            th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
                'HorizontalAlignment','right','rotation',rot,'parent',src,'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag','xticklabel',...
                'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
                'color',get(src,'xcolor')); 
            
            %determine how for down to move xlabel so it's out of the way:
            xlabel=get(src,'xlabel');
            
            pts=[6 8 10 12 14 18 2 436 48 72];
            pxs=[10 14 16 18 24 29 37 58 74 106];
            
            drawnow;
            pixes=get(th,'pixelbounds');
            if iscell(pixes)
                pixes=cell2mat(pixes);
            end
            
            if rot==90
                longestlength=max(pixes(:,4))-pixes(1,2);
            else
                longestlength=max(pixes(:,3))-pixes(1,1);
            end
            
            thispts=get(xlabel,'fontsize');
            
            pptperrow=ceil(interp1(pts,pxs,thispts));
            nrows=ceil(longestlength/pptperrow);

            xlabelstr=ravemakelabelstrings(src);
            newxstr=ravemakelabelcells(xlabelstr,nrows);
            
            if rot==0
                newxstr=flipud(newxstr);
            end

            set(xlabel,'string',newxstr);
            
            udata.format.nxlabelrows=nrows;
            set(handles.fakeax(handles.activeindex),'userdata',udata);
                        

        case 'logical'
            thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            ticknumbers=[0 1];
            
            set(src,'xtick',ticknumbers,'xticklabel',thesetexts,'xminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xtick',ticknumbers,'xticklabel',thesetexts,'xminortick','off');
            
    end
end



if ~isempty(yvar) % && handles.istext{udata.view.dataset}(yvar);
    switch handles.datatype{d}{yvar}
        case {'constant','continuous','integer'}
            set(src,'ytickmode','auto','yticklabelmode','auto');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',get(src,'ytick'),...
                'yticklabel','');
            
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{yvar};
            if numel(discretez)<=12;
                %ticknumbers=unique(handles.alldata{d}(:,yvar));
                set(src,'ytick',discretez,'yticklabel',discretez,'yminortick','off');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',get(src,'ytick'),...
                    'yticklabel','');
            else %too many, just treat as continuous
                set(src,'ytickmode','auto','yticklabelmode','auto');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',get(src,'ytick'),...
                    'yticklabel','');
            end
            
        case 'string'
            thesetexts=handles.discretevalues{d}{a}{yvar};
            ticknumbers=unique(handles.alldata{d}(:,yvar));
            
            set(src,'ytick',ticknumbers,'yticklabel',thesetexts,'yminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',ticknumbers,'yticklabel',thesetexts,'yminortick','off');
        case 'logical'
            thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            ticknumbers=[0 1];
            
            set(src,'ytick',ticknumbers,'yticklabel',thesetexts,'yminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',ticknumbers,'yticklabel',thesetexts,'yminortick','off');
            
    end
end

if ~isempty(zvar) % && handles.istext{udata.view.dataset}(zvar);
    switch handles.datatype{d}{zvar}
        case {'constant','continuous','integer'}
            set(src,'ztickmode','auto','zticklabelmode','auto');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',get(src,'ztick'),...
                'zticklabel','');
            
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{zvar};
            if numel(discretez)<=12;
                %ticknumbers=discretez;
                set(src,'ztick',discretez,'zticklabel',discretez,'zminortick','off');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',get(src,'ztick'),...
                    'zticklabel','');
            else %too many, just treat as continuous
                set(src,'ztickmode','auto','zticklabelmode','auto');
                set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',get(src,'ztick'),...
                    'zticklabel','');
            end
            
        case 'string'
            thesetexts=handles.discretevalues{d}{a}{zvar};
            ticknumbers=unique(handles.alldata{d}(:,zvar));
            
            set(src,'ztick',ticknumbers,'zticklabel',thesetexts,'zminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',ticknumbers,'zticklabel',thesetexts,'zminortick','off');
        case 'logical'
            thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            ticknumbers=[0 1];
            
            set(src,'ztick',ticknumbers,'zticklabel',thesetexts,'zminortick','off');
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',ticknumbers,'zticklabel',thesetexts,'zminortick','off');
            
    end
end











