function ravemovegraph(src,ev) %#ok<INUSD>
%RAVE: move the selected graph to a new location or resize graph by
%clicking movegrabbers.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

%get starting point:
clicked=get(handles.mainaxes,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

set(handles.infobar,'string','To move without resizing, ctrl+click graph and drag to desired location.');
%NOTE: MOVING WITHOUT RESIZING IS DONE BY RAVESELECTGRAPH function.

set(handles.fig,'WindowButtonMotionFcn',@ravemovegrabber);
set(handles.fig,'WindowButtonUpFcn',@ravemovegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);


if  handles.graphinfo.(handles.activetype).hidewhenmoving
    %turn off data series visibility:
    
    if ismember(handles.graphinfo.(handles.activetype).axesformat,{'matrix','row'}) && ~isempty(handles.subax{handles.activeindex})
        allkidz=get(handles.subax{handles.activeindex},'children');
        if iscell(allkidz) %wont be a cell if only one subax.
            allkidz=cell2mat(allkidz);
        end
    else
        allkidz=get(handles.activegraph,'children');
    end
    
    handles.temp.visibiliteez=get(allkidz,'visible');
    %set(get(handles.activegraph,'children'),'visible','off');
    set(allkidz,'visible','off');  %HAS SOME BUGS WHEN MOVING NON-CURRENT GRAPH
    %set(cell2mat(get(handles.statsax{handles.activeindex},'children')),'visible','off');
else
    handles.temp.visibiliteez=get(get(handles.activegraph,'children'),'visible');
end

guidata(src,handles);

%get current position of this grabber:
%grabpos=[get(src,'xdata') get(src,'ydata')];


function ravemovegrabber(src,ev) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

finish=get(handles.mainaxes,'currentpoint');

endx=finish(1,1,1);
endy=finish(1,2,1);

if handles.prefs.snaptogrid
    endx=handles.snapsize*round(endx/handles.snapsize);
    endy=handles.snapsize*round(endy/handles.snapsize);
end


minheight=handles.graphinfo.(handles.activetype).minheight;
minwidth=handles.graphinfo.(handles.activetype).minwidth;

switch get(gco,'tag') %grabber order: [SW NW NE SE];
    case 'NE'
        %enforce minimum graph size:
        if endx<get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth
            endx=get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth;
        end
        
        if endy<get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1
            endy=get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1;
        end
        
        set(gco,'xdata',endx-3,'ydata',endy-2)
        set(handles.grabbers{handles.activeindex}(2),'ydata',endy-2)
        set(handles.grabbers{handles.activeindex}(4),'xdata',endx-3)
        
    case 'SE'
        %enforce minimum graph size:
        if endx<get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth
            endx=get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth;
        end
        
        if endy>get(handles.grabbers{handles.activeindex}(2),'ydata')-minheight
            endy=get(handles.grabbers{handles.activeindex}(2),'ydata')-minheight;
        end
        
        set(gco,'xdata',endx-3,'ydata',endy+3)
        set(handles.grabbers{handles.activeindex}(1),'ydata',endy+3)
        set(handles.grabbers{handles.activeindex}(3),'xdata',endx-3)
        
    case 'NW'
        %enforce minimum graph size:
        if endx>get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1
            endx=get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1;
        end
        
        if endy<get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1
            endy=get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1;
        end
        
        set(gco,'xdata',endx+2,'ydata',endy-2)
        set(handles.grabbers{handles.activeindex}(3),'ydata',endy-2)
        set(handles.grabbers{handles.activeindex}(1),'xdata',endx+2)
    case 'SW'
        %enforce minimum graph size:
        if endx>get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1
            endx=get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1;
        end
        
        if endy>get(handles.grabbers{handles.activeindex}(3),'ydata')-minheight
            endy=get(handles.grabbers{handles.activeindex}(3),'ydata')-minheight;
        end
        set(gco,'xdata',endx+2,'ydata',endy+3)
        set(handles.grabbers{handles.activeindex}(4),'ydata',endy+3)
        set(handles.grabbers{handles.activeindex}(2),'xdata',endx+2)
end


pos=[get(handles.grabbers{handles.activeindex}(1),'xdata')-2 get(handles.grabbers{handles.activeindex}(1),'ydata')-3 ...
    get(handles.grabbers{handles.activeindex}(3),'xdata')-get(handles.grabbers{handles.activeindex}(1),'xdata')+5 ...
    get(handles.grabbers{handles.activeindex}(3),'ydata')-get(handles.grabbers{handles.activeindex}(1),'ydata')+5];

set([handles.allblockers(handles.activeindex) handles.expblockers(handles.activeindex)],'position',pos)



widthinches=sprintf('%5.2f',round(pos(3)/96*4)/4);
heightinches=sprintf('%5.2f',round(pos(4)/96*4)/4);
set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'" at 96dpi resolution'])

if strcmp(handles.graphinfo.(handles.activetype).axesformat,'table')
    %Get code from ravemovetable
else
    
    
    
    
    try %#ok<TRYNC> %THIS GIVES ERROR IF YOU TRY TO MAKE GRAPH TOO SMALL.
        raveresizeaxes(handles.fig,[],handles.activeindex);
        ravearrangeobjects(handles.fig,[]);
    end
end

function ravemovegraphdone(src,ev)

clear ravearrangeobjects %has persistent variables

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','');
set(handles.fig,'windowbuttonupfcn','');

if  handles.graphinfo.(handles.activetype).hidewhenmoving
    %turn off data series visibility:
    
    if ismember(handles.graphinfo.(handles.activetype).axesformat,{'matrix','row'}) && ~isempty(handles.subax{handles.activeindex})
        allkidz=get(handles.subax{handles.activeindex},'children');
        if iscell(allkidz) %wont be a cell if only one subax.
            allkidz=cell2mat(allkidz);
        end
        set(allkidz,{'visible'},handles.temp.visibiliteez);
    else
        allkidz=get(handles.activegraph,'children');
        set(allkidz,{'visible'},handles.temp.visibiliteez);
    end
end

set(handles.infobar,'string','');

