function graphname=ravenamegraph(src,ev)
%generate a name for the current graph.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

allnames=handles.allnames{udata.view.dataset}; 

%make size modifier:
pos=get(handles.activegraph,'position');
graphsize=pos(3)*pos(4);

if graphsize<50000
    size=' really small';
elseif graphsize<200000
    size=' small';
elseif graphsize<500000;
    size=' average-sized';
elseif graphsize<1000000;
    size=' large';
elseif graphsize<1700000;
    size=' very large';
else
    size=' gigantic';
end

%make aspect ratio modifier:

ar=pos(3)/pos(4);
if ar<.3334
    artext=', short';
elseif ar>=3
    artext=', tall';
else
    artext='';
end

%make graph type text:

switch handles.activetype;
    case 'line'
        type=' line graph';
    case 'scatter'
        type=' scatter plot';
    case 'scatter3D'
        type=' 3-D scatter plot';
    case 'scattermatrix'
        type=' scatter plot matrix';
    case 'histogram'
        type=' histogram';
    case 'histogram3D'
        type=' 3-D histogram';
    case 'parallelcoords'
        type=' parallel coordinates graph';
    case 'image'
        type=' picture';
    case 'wate'
        type=' engine cross section';
    case 'surfaceprofiler'
        type=' surface profiler';
    case 'derivativeprofiler'
        type=' derivative profiler';
    case 'cdf'
        type=' CDF';
    case 'tablesinglepoint'
        type=' single-point table';
    case 'tableallpoints'
        type=' data table';
    case 'tablestatistics'
        type=' statistics table';
    case 'tablerankedlist'
        type=' ranked list';
    case 'greatness'
        type =' End of Greatness';
    otherwise
        type= ' unknown graph';
end


%make variable names text:

switch handles.activetype
    case 'line'
        if numel(udata.view.y)==1
             vars=[' of ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}];
        else
            vars=[' of ',num2str(numel(udata.view.y)),' variables vs. ',allnames{udata.view.x}];
                end
        
    case 'scatter'
        vars=[' of ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}];
        
        
        
    case {'scatter3D','histogram3D'}
        vars=[' of ',allnames{udata.view.z},' vs. ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}];
    otherwise
        vars='';

end

graphname=['The',size,artext,type,vars];








