function raveopenstate(varargin)
%
%This function is used to open saved rave projects when they are
%doubleclicked in windows explorer.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



if nargin==1
    %only one input, was double clicked in windows.
    fullname=varargin{1}; 
else
    %two inputs
    src=varargin{1};
    eventdata=varargin{2};
    [name,path,index]=uigetfile({'*.rve','RAVE Project (*.rve)';'*.*','All Files (*.*)'},'Choose state to load:');
    if name==0;return;end
    fullname=fullfile(path,name);
end


[pathstr, name, ext, versn] = fileparts(fullname); 

copyfile(fullname,fullfile(pathstr,['RAVETEMP' name '.fig']),'f')

handy=openfig(fullfile(pathstr,['RAVETEMP' name '.fig']));
delete(fullfile(pathstr,['RAVETEMP' name '.fig']))

handles=guidata(handy);


%% size to fit new monitor:

%resize to fill screen


load raveprefs

set(handles.fig,'units','pixels')
screensize=get(0,'screensize');

outerwidth=raveprefs.preferredposition(3); %new
outerheight=raveprefs.preferredposition(4); %new

%handles.prefs.ravemonitor=raveprefs.ravemonitor;
handles.prefs.fonts=raveprefs.fonts;
handles.colors=raveprefs.colors;

% if raveprefs.ravemonitor==1 || size(get(0,'monitorpositions'),1)==1
%     screensize=get(0,'screensize');
% else %handles.prefs.ravemonitor=index of monitor to use.
%     screensize=get(0,'monitorpositions');
%     offset=max(screensize(:,4));
%     screensize=screensize(handles.prefs.ravemonitor,:);
%     screensize(2)=offset-screensize(4);
%     screensize(3)=screensize(3)-screensize(1);
% end
% 
% outerwidth=screensize(3);
% outerheight=screensize(4);


% switch raveprefs.os
%     case 'vista'
%         startmenuheight=30;
%         handles.colors.buttonbg = [0.941176 0.941176 0.941176]; %USING THIS COLOR SETS BUTTONS TO NEW VISTA STYLE.
%     case 'classic'
%         startmenuheight=28;
%     case 'xp'
%         startmenuheight=31;
%     case 'windows7'
%         startmenuheight=50;
% end
% 
% %dont allow for taskbar on 2nd monitor:
% if raveprefs.ravemonitor~=1 && size(get(0,'monitorpositions'),1)>1
%     startmenuheight=0;
% end


%size for current theme:
%set(handles.fig,'outerposition',[screensize(1)+1 screensize(2)+startmenuheight outerwidth outerheight-startmenuheight-1])
%

set(handles.fig,'position',raveprefs.preferredposition)


msgstate=warning('query','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');
warning('off','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');
innerposition=get(handles.fig,'position');

% switch raveprefs.os
%     case 'vista'
%         innerposition=innerposition+[-5 -5 5 10];
%     case 'classic'
%         innerposition=innerposition+[-2 -2 9 4];
%     case 'windows7'
%         innerposition=innerposition+[-5 -5 5 0];
%     case 'xp'
%         innerposition=innerposition+[-5 -5 5 10]; %not really sure what these values should be...don't use xp.
% end

handles.width=innerposition(3);
handles.height=innerposition(4);


set(handles.leftmargin,'position',[0 -5 30+315*raveprefs.tabscale handles.height+10])
set(handles.rightmargin,'position',[handles.width-13 -5 40 handles.height+20])
set(handles.topmargin,'position',[-5 handles.height-35 handles.width+5 60])
set(handles.bottommargin,'position',[-5 1 handles.width+10 17])

leftpos=get(handles.leftmargin,'position');
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
handles.workspacepos=[leftpos(1)+leftpos(3) botpos(2)+botpos(4) handles.workspacedims(1) handles.workspacedims(2)];


%make infobar:
infopos=get(handles.topmargin,'position');
infoaxes=get(handles.infobar,'parent');
set(infoaxes,'position',infopos+[360 0 -360 0]);

%% set up workspace explorer axes
if handles.height>=900
    pushdown=30; %move down a little so its not so cramped
else
    pushdown=0;
end

if handles.wscols>=240/150*handles.wsrows %width constrained 
    set(handles.explorer,'position',[100 handles.height-155-pushdown 240 handles.wsrows/handles.wscols*240]);
else
    set(handles.explorer,'position',[340-handles.wscols/handles.wsrows*150 handles.height-155-pushdown handles.wscols/handles.wsrows*150 150])
end


%% draw current-view frame
mainaxespos=get(handles.wspanel,'position');
makethistall=handles.workspacepos(4);
makethiswide=handles.workspacepos(3);
putthisfarup=handles.workspacepos(2)-mainaxespos(2);
putthisfarleft=handles.workspacepos(1)-mainaxespos(1);

set(handles.currentview,'position',[putthisfarleft putthisfarup makethiswide makethistall]);
  
cvpos=get(handles.currentview,'position');

%% make explorer title if on large screen
delete(handles.explorertitle);
if handles.height>900

    handles.explorertitle=axes('units','pixels','position',[340-handles.wscols/handles.wsrows*150 handles.height-1-pushdown 200 50],'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,...
        'buttondownfcn',@ravemoveview,'hittest','on','color','none');
    text(0,.2,'Navigator','fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,'color',handles.colors.title,...
        'parent',handles.explorertitle,'verticalalignment','baseline')
else
    handles.explorertitle=[];

end


%% make scrollbars

set(handles.vscroll,'position',[handles.width-18 18 16 handles.height-54],'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,...
    'sliderstep',[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))],'value',cvpos(4)/2)
set(handles.hscroll,'position',[357 1 handles.width-373 16],'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2,...
    'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],'value',cvpos(3)/2)

%% set up title axes if on a large monitor
delete(handles.titleaxes);
if handles.height>=900
    handles.titleaxes=axes('units','pixels','position',[20 20 320 40],'parent',handles.fig,...
       'visible','off','xlim',[0 1],'ylim',[0 1]);
    text(.5,0,'rave','fontname','magneto','parent',handles.titleaxes,'fontsize',80,'color',handles.colors.title,...
        'verticalalignment','baseline','backgroundcolor','none','hor','center','hittest','on','buttondownfcn',@ravefixtabs)
    axesoffset=85;
else
    handles.titleaxes=[];
    axesoffset=30;
end

set(handles.bgtabaxes,'position',[20+30*(raveprefs.tabscale) 15+axesoffset-20-650*(raveprefs.tabscale-1) 261*raveprefs.tabscale 631*raveprefs.tabscale]);
set(handles.tabaxes,'position',[20+60*(raveprefs.tabscale) 20+15-20+axesoffset-650*(raveprefs.tabscale-1) 261*raveprefs.tabscale 631*raveprefs.tabscale]);
set(handles.fronttabaxes,'position',[20+30*(raveprefs.tabscale) 35-20+axesoffset-650*(raveprefs.tabscale-1) 30*raveprefs.tabscale 631*raveprefs.tabscale]);
set(handles.backtabaxes,'position',[20 15-20+axesoffset-650*(raveprefs.tabscale-1) 30*raveprefs.tabscale 630*raveprefs.tabscale]);
 
delete(handles.controltitle);
if handles.height>=900
    %show title on controlpanel:
    handles.controltitle=text(.15,1,'Control Panel','backgroundcolor','none','color',handles.colors.title,'fontsize',20*handles.prefs.fonts.titlescale,...
        'horizontalalignment','left','verticalalignment','baseline','parent',handles.tabaxes,...
        'fontname',handles.prefs.fonts.title,'margin',1,'handlevisibility','off');
else
    handles.controltitle=[];
end
% 
% set(handles.stopbutton,'position',[handles.width-105 handles.height-32 100 32]);
% set(handles.showgridbutton,'position',[18 handles.height-155-pushdown 32 32])
% set(handles.highlightbutton,'position',[59 handles.height-155-pushdown 32 32])
% set(handles.unhidebutton,'position',[18 handles.height-115-pushdown 32 32])
% set(handles.openglbutton,'position',[59 handles.height-115-pushdown 32 32])
% set(handles.zoombutton,'position',[18 handles.height-75-pushdown 32 32])
% set(handles.hideshowbutton,'position',[59 handles.height-75-pushdown 32 32])
% set(handles.newbutton,'position',[18 handles.height-35-pushdown 32 32])
% set(handles.putawaybutton,'position',[59 handles.height-35-pushdown 32 32])
%   

set(handles.tablemodebutton,'position',[18 handles.height-35-pushdown 73 32]);
set(handles.newbutton,'position',[18 handles.height-75-pushdown 32 32]);
set(handles.zoombutton,'position',[59 handles.height-75-pushdown 32 32]);
set(handles.showgridbutton,'position',[18 handles.height-155-pushdown 32 32]);
set(handles.openglbutton,'position',[59 handles.height-115-pushdown 32 32]);
set(handles.hideshowbutton,'position',[18 handles.height-115-pushdown 32 32]);


%delete old table and make a new one:
delete(handles.maintable.handle);

%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

%depending on which table method i'm currently using, invisible is either "off" or "0"
set(handles.maintable.handle,'visible','off');

%create table models for data sets:
a=handles.maintable.a;
for d=1:numel(handles.alldata);
   [handles.maintable.models{d} handles.maintable.rhmodels{d}]=ravecreatemaintablemodel(handles.fig,[],d,a);    
end

guidata(handles.fig,handles);

%fix table visiblity:
showhideworkspace(handles.fig,[]);

%finish up:
ravemoveview(handles.fig,[],[0 handles.prefs.wsheight-1+35]);

managetab(handles.fig,[]);

% pos=get(handles.fig,'position');

% 
% 
% switch raveprefs.os
%     case 'classic'
%         set(handles.fig,'visible','on','windowscrollwheelfcn',@wheelscroll,'position',pos+[-4 -3 8 8]+raveprefs.nudge)
%         % set(splashax,'position',splashpos+[0 -2 0 0])
%     case {'vista','windows7'}
%         set(handles.fig,'windowscrollwheelfcn',@wheelscroll,'position',pos+[0 0 0 0]+raveprefs.nudge)
%     otherwise
%         set(handles.fig,'visible','on','windowscrollwheelfcn',@wheelscroll,'position',pos+[-4 -3 8 8]+raveprefs.nudge)
% end

%fix point sizes on scatter plots:

for a=1:numel(handles.allgraphs)
    if ~ishandle(handles.allgraphs(a));continue;end
    thistype=get(handles.allgraphs(a),'userdata');
    if strcmp(thistype,'scatter') ||strcmp(thistype,'scatter3D')
        udata=get(handles.fakeax(a),'userdata');
        main=findobj(handles.allgraphs(a),'tag','main');
        if strcmp(get(main,'type'),'hggroup');
            hhh=get(main,'children');
            set(hhh,'markersize',sqrt(udata.format.minmarkersize))            
        end
        
        selected=findobj(handles.allgraphs(a),'tag','selected');
        if strcmp(get(selected,'type'),'hggroup');
            hhh=get(selected,'children');
            set(hhh,'markersize',sqrt(udata.format.minmarkersize))            
        end

    end   
    
end

warning(msgstate.state,'MATLAB:hg:uicontrol:ParameterValuesMustBeValid');

drawnow;
set(handles.fig,'DoubleBuffer','on'); %fix CoDE "out of windows resources" error.




