function success=raveprint_pdf(name, fig)
%Copyright (c) 2009, Oliver Woodford
% 
% All rights reserved.
% 
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
%
%
%PRINT_PDF  Prints figures to pdfs better than MATLAB does
%
% Examples:
%   print_pdf filename
%   print_pdf(filename, fig_handle)
%
% This function saves a figure as a pdf nicely, without the need to specify
% multiple options. It improves on MATLAB's print command (using default
% options) in several ways:
%   - The figure borders are cropped
%   - Fonts are embedded (as subsets)
%   - Lossless compression is used on vector graphics
%   - High quality jpeg compression is used on bitmaps
%
% This function requires that you have ghostscript installed on your system
% and that the executable binary is on your system's path.
%
%IN:
%   filename - string containing the name (not path) of the file the figure
%              is to be saved as (including the .pdf file extension). The
%              figure is saved in the current directory.
%   fig_handle - The handle of the figure to be saved. Default: current
%                figure.
%
% Copyright (C) Oliver Woodford 2008

% This function is inspired by Peder Axensten's SAVEFIG (fex id: 10889)
% which is itself inspired by EPS2PDF (fex id: 5782)

% $Id: print_pdf.m,v 1.5 2008/11/06 22:53:31 ojw Exp $


if nargin < 2
    fig = gcf;
end
% Get current image dimensions
set(fig, 'Units', 'Inches');
sz = get(fig, 'Position');
sz(1:2) = 0;
% Set paper units
set(fig, 'PaperUnits', 'Inches', 'PaperSize', sz(3:4), 'PaperPosition', sz);
% Print to eps file
tmp_nam = [tempname '.eps'];
%name = [cd '\' name];
print('-depsc2', '-noui', '-painters', ['-f' num2str(fig)], '-r600', tmp_nam);
% Convert to pdf
% This assumes that the ghostscript binary is on your path - you can also
% give the complete path
if ispc
    cmd = 'gswin32c.exe';
else
    cmd = 'gs';
end
cmd = [cmd ' -dPDFSETTINGS=/prepress -q -dNOPAUSE -dBATCH -dEPSCrop -dSubsetFonts=true -dEmbedAllFonts=true -dAutoRotatePages=/None -dUseFlateCompression=true -dLZWEncodePages=true -dAntiAliasColorImages=false -dAutoFilterColorImages=false -dColorImageFilter=/DCTEncode -dCompatibilityLevel=1.5 -sDEVICE=pdfwrite -sOutputFile="' name '" -c ".setpdfwrite << /ColorImageDict << /QFactor 0.25 /Blend 1 /HSample [2 1 1 2] /VSample [2 1 1 2] >> >> setdistillerparams" -f "' tmp_nam '"'];
success=system(cmd);
% Delete the temporary file
delete(tmp_nam);
