function raverestoregraph(src,ev)
%restore a graph that has been put away.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

set(0,'userdata',[]);

screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-250 screensize(2)+screensize(4)/2-75 500 150],...
    'numbertitle','off','menubar','none','name','Put Away Selected Graph','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 80 225 40],...
    'string','Choose the graph you would like to restore: ','horizontalalignment','left')
uicontrol('style','popupmenu','string',handles.graphnames(handles.putaway),'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 70 440 22],'tag','namebox','horizontalalignment','left')
uicontrol('style','pushbutton','string','Restore','parent',askbox,'units','pixels',...
    'position',[270 10 100 30],'callback',@restorepopup_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[380 10 100 30],'callback',@restorepopup_callback,'fontsize',10);


uiwait(askbox)

index=get(0,'userdata');


if isempty(index)
    return %user hit cancel
end

ai=handles.putaway(index);
handles.putaway=setdiff(handles.putaway,ai);

handles.temp.ai=ai;

%else prompt for placment of this graph:
set(handles.infobar,'string','Click and drag to place this graph.')
set(handles.mainaxes,'buttondownfcn',{@raveplacegraph,handles})
set(handles.explorer,'buttondownfcn',{@raveplacegraph,handles})


guidata(src,handles)



function raveplacegraph(src,eventdata,handles) %#ok<INUSL>
set(handles.fig,'windowbuttondownfcn',[]);
buttons=findobj(handles.tabobjects,'type','uicontrol');
set(buttons,'enable','on');

%check that they clicked on exploreraxes or mainaxes:
if ~any([gco==handles.explorer,gco==handles.mainaxes])
    % set(handles.mainaxes,'buttondownfcn',[])
    % set(handles.explorer,'buttondownfcn',@ravemoveview)

    return
end

clicked=get(handles.mainaxes,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

xclick=handles.snapsize*floor(xclick/handles.snapsize);
yclick=handles.snapsize*floor(yclick/handles.snapsize);

handles.temp.putgraphhere(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[1 .5 0],...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor',handles.colors.wsbg);
handles.temp.putgraphhere(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[1 .5 0],...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizegraph);
set(handles.fig,'WindowButtonUpFcn',@ravesizegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);

function ravesizegraph(src,eventdata) %#ok<INUSD>

handles=guidata(src);

finish=get(handles.mainaxes,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

pos1=handles.snapsize*floor(min(startx,endx)/handles.snapsize);
pos3=handles.snapsize*ceil(max(startx,endx)/handles.snapsize)-pos1;
pos2=handles.snapsize*floor(min(starty,endy)/handles.snapsize);
pos4=handles.snapsize*ceil(max(starty,endy)/handles.snapsize)-pos2;

widthinches=sprintf('%6.2f',round(pos3/96*4)/4);
heightinches=sprintf('%6.2f',round(pos4/96*4)/4);

set(handles.temp.putgraphhere,'position',[pos1 pos2 pos3 pos4]);

set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'"  at 96 dpi'])

function ravesizegraphdone(src,eventdata) %#ok<INUSD>
handles=guidata(src);
%stop dragging

set(handles.fig,'WindowButtonMotionFcn',[]);
set(handles.fig,'WindowButtonUpFcn',[]);
set(handles.fig,'WindowButtondownFcn',[]);

%make graph in this position


%...

newpos=get(handles.temp.putgraphhere(1),'position');
delete(handles.temp.putgraphhere);



set(handles.allblockers(handles.temp.ai),'position',newpos,'visible','on')
set(handles.expblockers(handles.temp.ai),'position',newpos,'visible','on')
set(handles.rasters{handles.temp.ai},'position',newpos+[20 20 -40 -40],'visible','on')


%grabber order: [SW NW NE SE]
set(handles.grabbers{handles.temp.ai},'visible','on')
set(handles.grabbers{handles.temp.ai}(1),'xdata',newpos(1)+2,'ydata',newpos(2)+3)
set(handles.grabbers{handles.temp.ai}(2),'xdata',newpos(1)+2,'ydata',newpos(2)+newpos(4)-2)
set(handles.grabbers{handles.temp.ai}(3),'xdata',newpos(1)+newpos(3)-3,'ydata',newpos(2)+newpos(4)-2)
set(handles.grabbers{handles.temp.ai}(4),'xdata',newpos(1)+newpos(3)-3,'ydata',newpos(2)+3)


%handles.activeindex=handles.temp.ai;
%handles.activegraph=handles.allgraphs(handles.temp.ai);


%bring back the graph:

%raveresizeaxes(handles.fig,[],handles.temp.ai);
%ravevectorize(handles.fig,[],handles.temp.ai);

%select this graph:
raveresizeaxes(handles.fig,[],handles.temp.ai);


raveselectgraph(handles.allgraphs(handles.temp.ai),[]);


raveresizeaxes(handles.fig,[],handles.activeindex);


handles=guidata(handles.fig);





%turn tick labels back on:
switch handles.activetype
    case 'scattermatrix'
        set(get(handles.allgraphs(handles.activeindex),'children'),'visible','on');
end


%%% THIS BIG CHUNK IS COPIED DIRECTLY FROM RAVEMOVEGRAPH.

    %do type-specific stuff:
    switch handles.activetype
        case {'scattermatrix','derivativeprofiler'}


            
            numy=size(handles.subax{handles.activeindex},1);
            numx=size(handles.subax{handles.activeindex},2);
            pos = get(handles.activegraph,'position');

            vspace=0.03;
            hspace=0.03;

            if strcmp(handles.activetype,'derivativeprofiler')
                %pos=pos;
                vspace=0.05;
                hspace=0.00;
            end

            width = pos(3)/numx;
            height = pos(4)/numy;
            pos(1:2) = pos(1:2) + [hspace*width vspace*height];

            ax=handles.subax{handles.activeindex};
            fakeax=handles.subfakeax{handles.activeindex};
            fakerax=handles.subfakerax{handles.activeindex};

            xlabelz=findall(ax,'tag','xticklabel'); %use findall since handle visibility is off.
            if pos(3)/numx<250
                set(xlabelz,'rotation',90,'horizontalalignment','right','verticalalignment','default')
            else
                set(xlabelz,'rotation',0,'horizontalalignment','center','verticalalignment','top')
            end

            %set up grid of axes.
            for r=numy:-1:1,
                for c=numx:-1:1,
                    axPos = [pos(1)+(c-1)*width pos(2)+(numy-r)*height ...
                        width*(1-hspace) height*(1-vspace)];
                    set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',axPos);

                end
            end
            
            %for scattermatrix, resize axis labels to fit on diagonals:
            labelz=findobj(handles.subax{handles.activeindex},'tag','axeslabel');
            maxchars=round(width*(1-hspace)/8);%approx 8 pixel per character
            udata=get(handles.fakeax(handles.activeindex),'userdata');
            for g=1:numel(labelz)
               newstring=linewrap(handles.allnames{udata.view.dataset}{udata.view.x(g)},maxchars);
               set(labelz(g),'string',newstring);
            end
            
            
        case 'parallelcoords'
            udata=get(handles.fakeax(handles.activeindex),'userdata');
            axpos=get(handles.activegraph,'position');
            nvars=numel(udata.view.x);
            width=axpos(3)/(nvars-1);
            maxchars=round(width/9); %approx 8 pixel per character

            
            %for resize axis labels to fit on diagonals:
            labelz=findobj(handles.activegraph,'tag','axeslabel');
            
            for g=numel(labelz):-1:1
               
                oldstring=unlinewrap(get(labelz(g),'string'));
                newstring=linewrap(oldstring,maxchars);
                
                %newstring=linewrap(handles.allnames{udata.view.dataset}{udata.view.x(g)},maxchars);
               
               
               set(labelz(g),'string',newstring);
            end
            
            
            
            
            

        case 'greatness'

            numy=size(handles.subax{handles.activeindex},1);
            numx=size(handles.subax{handles.activeindex},2);
            pos = get(handles.activegraph,'position');

            vspace=0.03;
            hspace=0.03;

            %             if strcmp(handles.activetype,'derivativeprofiler')
            %                 %pos=pos;
            %                 vspace=0.05;
            %                 hspace=0.00;
            %             end

            width = pos(3)/numx;
            height = pos(4)/numy;
            pos(1:2) = pos(1:2) + [hspace*width vspace*height];

            ax=handles.subax{handles.activeindex};
            fakeax=handles.subfakeax{handles.activeindex};
            fakerax=handles.subfakerax{handles.activeindex};

            xlabelz=findall(ax,'tag','xticklabel'); %use findall since handle visibility is off.
            if pos(3)/numx<250
                set(xlabelz,'rotation',90,'horizontalalignment','right','verticalalignment','default')
            else
                set(xlabelz,'rotation',0,'horizontalalignment','center','verticalalignment','top')
            end

            %set up grid of axes.
            for r=numy:-1:1,
                for c=numx:-1:1,
                    axPos = [pos(1)+(c-1)*width pos(2)+(numy-r)*height ...
                        width*(1-hspace) height*(1-vspace)];
                    set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',axPos);

                end
            end



        case {'histogram','cdf'} %need to move subax and statsax (if stats are shown.)

            udata=get(handles.fakeax(handles.activeindex),'userdata');


            numvars=numel(handles.subax{handles.activeindex});

            ax=handles.subax{handles.activeindex};
            fakeax=handles.subfakeax{handles.activeindex};
            fakerax=handles.subfakerax{handles.activeindex};
            statsax=handles.statsax{handles.activeindex};

            if udata.view.showstats
                margin=150;
            else
                margin=20;
            end

            pos=get(handles.activegraph,'position');

            xlabelspace=20;
            ylabelspace=32;

            if  pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
                width = pos(3)/numvars-ylabelspace+min(5,ylabelspace/numvars);
                height = pos(4)-xlabelspace-margin;
                space = .03; % 3 percent space between subaxes
                pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];

            else %vertical stacking
                width=pos(3)-ylabelspace-margin;
                height=pos(4)/numvars-xlabelspace;
                space = .03; % 3 percent space between subaxes
                pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
            end

            if width<3;width=3;end
            if height<3;height=3;end

            for c=1:numvars

                if pos(3) > pos(4) %HORIZONTAL STACKING
                    axPos = [pos(1)+(c-1)*(width+ylabelspace)+ylabelspace pos(2)+margin ...
                        width-ylabelspace height*(1-space)]; %subtract margin to make space for statistics
                    statsPos=[axPos(1) axPos(2)-150 axPos(3) 100];
                else
                    axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
                        width-ylabelspace height*(1-space)-20]; %subtract margin to make space for statistics
                    statsPos=[axPos(1)+width-ylabelspace+10 axPos(2)-15 150 100];
                end

                set([ax(c) fakerax(c) fakeax(c)],'position',axPos)
                if udata.view.showstats
                    set(statsax(c),'position',statsPos);
                    
                    if height<100 || width<150
                        statsize=8;
                        set(statsax(c),'pos',statsPos.*[1 1 1 .8]);
                        statz=findobj(statsax(c),'tag','histogramstat');
                        set(statz,'fontsize',statsize);
                    else
                        statsize=10;
                         statz=findobj(statsax(c),'tag','histogramstat');
                        set(statz,'fontsize',statsize);
                    end

                    
                end
                
                set([fakerax(c) fakeax(c)],'xlim',get(ax(c),'xlim'),'ylim',get(ax(c),'ylim'),'zlim',get(ax(c),'zlim'));
                set([fakerax(c) fakeax(c)],'xtick',get(ax(c),'xtick'),'ytick',get(ax(c),'ytick'),'ztick',get(ax(c),'ztick'))
            end

            
            
        case 'scatter3D'
            ravesynchaxes;

    end


%resynch axes limits (they can get screwed up if axes is too small)
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),...
    'ylim',get(handles.activegraph,'ylim'));



%%% END OF STUFF COPIED FROM RAVEMOVEGRAPH



%ditch the temp crap
handles.temp=[];

%clear the infobar
set(handles.infobar,'string','');

%undo creation function
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);

%update guidata
%guidata(handles.fig,handles);






function restorepopup_callback(src,ev)

popup=get(src,'parent');
if strcmp(get(src,'string'),'Cancel');
    delete(popup);
    return
end




namebox=findobj(popup,'tag','namebox');
index=get(namebox,'value');

set(0,'userdata',index);

delete(popup);






