function raveselectgraph(src,eventdata)
%
% this function activates when the user clicks a graph in rave to make it
% the current graph. this function highlights the graph, sets it as the
% current plotting target, and updates the tabs to display the correct
% options for that type of graph. Also needs to unselect the graph that was
% selected last.
%
%eventdata = 1 if this function is being called because user clicked
%highlight button. Otherwise eventdata is empty.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



%load handles
try
    handles=guidata(src);
catch %#ok<CTCH>
    return %avoids error when creating invalid derivativeprofiler.
end

%unhighlight last annotation
set(cell2mat(handles.Agrabbers),'visible','off')
lastindex=handles.activeindex;

%this function can take several src inputs, so check for all of them:
newindex=find(handles.allblockers==src | handles.expblockers==src | handles.allgraphs==src);
if isempty(newindex)
    newindex=get(src,'userdata'); %for raster images
end

%this graph is already selected, do nothing.
if ~isempty(lastindex) && lastindex==newindex && isempty(eventdata)
    switch get(handles.fig,'selectiontype');
        case 'alt'  %MOVE GRAPH WITHOUT SELECTING (ctrl click)
            set(0,'userdata',get(handles.fig,'currentpoint'));
            if ~strcmp(handles.graphinfo.(handles.activetype).class,'table');
                set(handles.fig,'WindowButtonMotionFcn',@raverelocategraph);
                set(handles.fig,'WindowButtonUpFcn',@raverelocategraphdone);
            else
                set(handles.fig,'WindowButtonMotionFcn',@raverelocatetable);
                set(handles.fig,'WindowButtonUpFcn',@raverelocatetabledone);
            end 
    end
    return %and stop
end

if ~ishandle(src) %if all graphs have been deleted...
    return
end

%unhighlight last active graph
if ~isempty(lastindex) && lastindex~=newindex
    %if get(handles.showgridbutton,'value')
       set(handles.expblockers(lastindex),'facecolor',[.7 .7 .7])
    %else  
       % set(handles.expblockers(lastindex),'linewidth',1)%,'edgecolor',[.5 .5 .5])
    %end
    
    if ~strcmp(handles.graphinfo.(handles.activetype).class,'annotation')
        if get(handles.showgridbutton,'value')
            set(handles.allblockers(lastindex),'linewidth',1,'edgecolor',handles.colors.wsgrid,'buttondownfcn',@raveselectgraph)
            % set(handles.expblockers(lastindex),'linewidth',1,'edgecolor',[.5 .5 .5])
        else
            set(handles.allblockers(lastindex),'linewidth',1,'edgecolor','none','buttondownfcn',@raveselectgraph)
            % set(handles.expblockers(lastindex),'linewidth',1,'edgecolor',[.5 .5 .5])
        end
    end
    
    %turn off grabbers:
    set(handles.grabbers{lastindex},'visible','off');
       
    %rasterize last graph if applicable
    if handles.prefs.alwaysrasterize
        
        %if any(strcmp(handles.activetype,{'scatter';'scattermatrix';'line';'scatter3D';'derivativeprofiler';'wate';'histogram';'parallelcoords';'surfaceprofiler'}))
        if handles.graphinfo.(handles.activetype).israsterizable
            guidata(src,handles);
            %   raverasterize(src,[]);
            raverasterfarian(src,[]);
            handles=guidata(src);
        end
    end
    
end

%unhighlight last active subgraph if it exists...
if ishandle(handles.activesubgraph)
    set(handles.activesubgraph,'xcolor',[0 0 0],'ycolor',[0 0 0]);
    
end

%highlight new graph
if ~isempty(newindex)
    
    %make this the active graph
    handles.activegraph=handles.allgraphs(newindex);
    %set(handles.activegraph,'hittest','on');
    handles.activeindex=newindex;
    handles.activetype=get(handles.activegraph,'userdata'); %userdata contains the graph type string.
    
    if 1 %get(handles.highlightbutton,'value')
        %if strcmp(handles.activetype,'sliders') || strcmp(handles.activetype,'preferencesliders')
        if strcmp(handles.graphinfo.(handles.activetype).class,'control');
            set(handles.allblockers(newindex),'linewidth',2,'edgecolor',[0 .5 1])    
        elseif strcmp(handles.graphinfo.(handles.activetype).class,'annotation');
            %do nothing, edge color controlled by format tab.
        elseif strcmp(handles.graphinfo.(handles.activetype).class,'table');
            set(handles.allblockers(newindex),'linewidth',2,'edgecolor',[0.7 0.9 0.1])    
        else
            set(handles.allblockers(newindex),'linewidth',2,'edgecolor',[1 0.5 0])
        end
    end
    
    if strcmp(handles.graphinfo.(handles.activetype).class,'control');
        set(handles.expblockers(newindex),'linewidth',1,'facecolor',[0.2 0.7 1],'buttondownfcn',@raveslidegraph)
    elseif strcmp(handles.graphinfo.(handles.activetype).class,'annotation');
        set(handles.expblockers(newindex),'linewidth',1,'facecolor',[0.8 0.4 1],'buttondownfcn',@raveslidegraph)
    elseif strcmp(handles.graphinfo.(handles.activetype).class,'table');
        set(handles.expblockers(newindex),'linewidth',1,'facecolor',[0.8 1 0.2],'buttondownfcn',@raveslidegraph)     
    else
        set(handles.expblockers(newindex),'linewidth',1,'facecolor',[1 .7 0],'buttondownfcn',@raveslidegraph)
    end
    
    %select new subax:
    if ismember(handles.graphinfo.(handles.activetype).axesformat,{'matrix','row'}) && ~isempty(handles.subax{newindex})
        handles.activesubgraph=handles.subax{newindex}(1);
    end
    
    %allow dragging to relocate graph:
    % set(handles.fig,'windowbuttonmotionfcn',@raveslidegraph,'windowbuttonupfcn',@raveslidecancel)
    %    if handles.prefs.automovemode || get(handles.highlightbutton,'value') %if "always show grabbers" is on or if orange highlight border is on:
    set(handles.grabbers{handles.activeindex},'visible','on')
    %    end
end

%savehandles
guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%note, this line must be last or else reload handles after this line.

%vectorize new graph if applicable:
ravevectorize(src,[])
%refresh the active tab


udata=get(handles.fakeax(handles.activeindex),'userdata');
if isstruct(udata)%avoids error when creating a new graph not from the manage tab.
    eval([handles.activetab,'tab(handles.fig,[])'])
end



switch get(handles.fig,'selectiontype');
    case 'alt'  %MOVE GRAPH WITHOUT SELECTING (ctrl click)
        set(get(handles.activegraph,'children'),'visible','off');
        set(0,'userdata',get(handles.fig,'currentpoint'));
        
        if ~strcmp(handles.graphinfo.(handles.activetype).class,'table');
            set(handles.fig,'WindowButtonMotionFcn',@raverelocategraph);
            set(handles.fig,'WindowButtonUpFcn',@raverelocategraphdone);
        else
            set(handles.fig,'WindowButtonMotionFcn',@raverelocatetable);
            set(handles.fig,'WindowButtonUpFcn',@raverelocatetabledone);
            
        end
end


function raverelocategraph(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

startpoint=get(0,'userdata');
newpoint=get(handles.fig,'currentpoint');

xoffset=newpoint(1)-startpoint(1);
yoffset=newpoint(2)-startpoint(2);

oldpos=get(handles.activegraph,'position');

blockpos=get(handles.allblockers(handles.activeindex),'position');
grabbies=handles.grabbers{handles.activeindex};
xgrabpos=get(grabbies,'xdata');
ygrabpos=get(grabbies,'ydata');

for j=1:4
    set(grabbies(j),'xdata',xgrabpos{j}+xoffset,'ydata',ygrabpos{j}+yoffset);
end

newpos=[oldpos(1)+xoffset oldpos(2)+yoffset oldpos(3:4)];
newblockpos=[blockpos(1)+xoffset blockpos(2)+yoffset blockpos(3:4)];

set([handles.allblockers(handles.activeindex),handles.expblockers(handles.activeindex)],'position',newblockpos)
set(handles.activegraph,'position',newpos);
set(handles.fakeax(handles.activeindex),'position',newpos);
set(handles.fakerax(handles.activeindex),'position',newpos);

set(0,'userdata',newpoint);

raveresizeaxes(handles.fig,[],handles.activeindex);
ravearrangeobjects(handles.fig,[]);

function raverelocatetable(src,ev)
handles=guidata(src);
set(handles.infobar,'string','You can''t yet move tables without resizing. Sorry about that.')



function raverelocategraphdone(src,ev)

%clear persistent vars
clear raveselectgraph
clear ravearrangeobjects

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','');
set(handles.fig,'windowbuttonupfcn','');

raveresizeaxes(src,[],handles.activeindex)

set(handles.infobar,'string','');

function raverelocatetabledone(src,ev)

%clear persistent vars
clear raveselectgraph
clear ravearrangeobjects

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','');
set(handles.fig,'windowbuttonupfcn','');

raveresizetable(src,[],handles.activeindex)

set(handles.infobar,'string','');




