function raveupdateanalysis(varargin) %#ok<INUSL>
%raveupdateanalysis(src,ev,updatetype,thisgraphtoo,d,a,thisxdial)
%applies the update in updatetype to all graphs that share the same
%analysis as the active graph. (or updates the analysis "a" if 6 inputs are
%used)
%
%options for updatetype:
%'xdials','visiblerows','constraints','isselected,'rowcolors',
%'variablecolors'
%
%if thisgraphtoo=1, will update ALL graphs including the selected one. otherwise only updates
%unselected graphs.
%
%the fifth input, d, is only used if you also supply a sixth input, a.
%
%the seventh input, thisxdial is not currently used. Idea is to only update
%things that depend on this variable to improve efficiency.
%
%note, might need another function to use when switching the analysis of a
%graph, or maybe just redo createfunction like other viewtab menus do.

%NEED to make this not rasterize/vectorize when updating the currently selected graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


src=varargin{1};
ev=varargin{2};
updatetype=varargin{3};

handles=guidata(src);


if nargin<6
    udata=get(handles.fakeax(handles.activeindex),'userdata');
    thisanalysis=udata.view.analysis;
    thisdataset=udata.view.dataset;
else
    thisdataset=varargin{5};
    thisanalysis=varargin{6};
end


%update maintable:
if thisdataset==handles.maintable.d && thisanalysis==handles.maintable.a;
    switch updatetype
        case 'xdials'
            
        case 'visiblerows'
           % if handles.maintable.shadehidden;
                isrowshaded=~handles.isvisible{thisdataset}{thisanalysis};
                isRowShaded=javaArray('java.lang.Boolean',numel(isrowshaded));
                for r=1:numel(isrowshaded)
                    isRowShaded(r)=java.lang.Boolean(isrowshaded(r));
                end
                handles.maintable.models{thisdataset}.setIsRowShaded(isRowShaded);
                
           % end
        case 'constraints'
            
        case 'isselected'
            
        case 'rowcolors'    
            rowcolors=handles.rowcolors{thisdataset}{thisanalysis};
            jRowColors=javaArray('java.lang.Integer',numel(rowcolors));
            for r=1:numel(rowcolors)
                jRowColors(r)=java.lang.Integer(rowcolors(r)-1);
            end
            handles.maintable.models{thisdataset}.setRowColors(jRowColors);
        case 'variablecolors'     
    end
end


%if any variables are decided, update data set:
if thisanalysis==1 %only applies to analysis 1.
    switch updatetype
        case 'xdials'
            if any(handles.isdecided{thisdataset})
                
                decidedvars=find(handles.isdecided{thisdataset});
                
                handles.alldata{thisdataset}(:,decidedvars)=repmat(handles.xdials{thisdataset}{1}(decidedvars),size(handles.alldata{thisdataset},1),1);
                
                
                
                %determine which functions need to update:
                updatefuns=[];
                for c=find(handles.isfunction{thisdataset})
                    if any(ismember(decidedvars,handles.funvars{thisdataset}{c}))
                        updatefuns(end+1)=c; %#ok<AGROW>
                    end
                end
                
                if ~isempty(updatefuns)
                    y=raveevaluateregression(handles.fig,thisdataset,updatefuns,handles.alldata{thisdataset});
                    handles.alldata{thisdataset}(:,updatefuns)=y;
                end
                
                guidata(handles.fig,handles);
                raveupdatedataset(handles.fig,[],'datachanged',thisdataset)
                handles=guidata(handles.fig);
            end
    end
end


%find indices of other graphs with this dataset and analysis:
othergraphs=setdiff(find(handles.graphanalysis==thisanalysis),handles.activeindex);
if nargin>=4
    thisgraphtoo=varargin{4};
    if thisgraphtoo
        othergraphs=find(handles.graphanalysis==thisanalysis);
    end
end

if isempty(othergraphs);return;end

for ai=othergraphs
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    
    thistype=get(othergraph,'userdata'); %type of other graph
    udata=get(handles.fakeax(ai),'userdata'); %user data of othergraph
    if udata.view.dataset~=thisdataset;continue;end %skip if this graph is from a different data set.

    %check if thistype of graph gets updated:
    if ismember(updatetype,lower(handles.graphinfo.(thistype).updates));
        %check for a function to perform the update:
        ravevectorize(handles.fig,[],ai);
        
        oldindex=handles.activeindex;
        oldtype=handles.activetype;
        handles.activeindex=ai;
        handles.activegraph=handles.allgraphs(ai);
        handles.activetype=thistype;
        guidata(handles.fig,handles);
        
        if ~isempty(which(['raveupdate',thistype,updatetype]))
            eval(['raveupdate',thistype,updatetype,'(handles.fig,[]);'])
        else
            %brute force method:
            eval(['ravecreate',thistype,'(handles.fig,[]);'])
        end
        handles=guidata(handles.fig);
        handles.activeindex=oldindex;
        handles.activetype=oldtype;
        handles.activegraph=handles.allgraphs(oldindex);
        guidata(handles.fig,handles);
        
        if ai~=handles.activeindex;raverasterfarian(handles.fig,[],ai);end;
        %drawnow;
        handles=guidata(handles.fig);
        
    end
    
end
