function raveupdatedataset(varargin)
%raveupdatedataset(src,ev,updatetype,dataset)
%applies the update in updatetype to all graphs that share the same
%dataset as the active graph or the dataset arguement.
%options for updatetype:
%'preferences','targets','replaceall','addrows','addcolumns','removerows',
%'removecolumns','datachanged'
%
%Note: addrows does not reset selection/coloration etc. Replaceall resets those if number of rows changes. (really should prompt and ask what to do). 
% columns just updates the maintable RaveTableModel
%
%Use "replaceall" to also replace the backupdata set. Use "datachanged" to
%keep the same backupdata set.
%
%NEED to make this not rasterize/vectorize when updating the currently selected graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

src=varargin{1};
ev=varargin{2};
updatetype=varargin{3};

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if numel(varargin)==4
    d=varargin{4};
else
    d=udata.view.dataset;
end

%update backupdata (if data set grew)

switch updatetype
    case 'replaceall'
        handles.backupdata{d}=handles.alldata{d};
    case {'addrows','addcolumns'}
        newbackupdata=handles.alldata{d};
        olddims=size(handles.backupdata{d});
        newbackupdata(1:olddims(1),1:olddims(2))=handles.backupdata{d};
        handles.backupdata{d}=newbackupdata;
    case 'removerows'
        handles.backupdata{d}=handles.backupdata{d}(size(handles.alldata));    
end


%reset minlimit/maxlimit if replacing data set:

switch updatetype
    case {'replaceall','addrows'}
        handles.minlimit{d}{1}=min(handles.alldata{d});
        handles.maxlimit{d}{1}=max(handles.alldata{d});
end



othergraphs=find(handles.graphdataset==d & ishandle(handles.allgraphs));
needsupdating=false(size(othergraphs));
%apply changes to data set:

for o=1:numel(othergraphs);
    if ismember(handles.graphinfo.(get(handles.allgraphs(othergraphs(o)),'userdata')).updates,updatetype)
        needsupdating(o)=true;
    end
end
switch updatetype
    case {'preferences','targets'}
        %re-evaluate objective functions that use these variables:
        
        objectives=find(handles.isobjective{d});
        
        %NOTE: this loop won't work if one objective fcn is a function of another objective fcn.
        %should really check for dependency and evaluate in correct order.
        if ~isempty(objectives)
            
            newvalues=raveevaluateregression(src,d,objectives,handles.alldata{d});
            handles.alldata{d}(:,objectives)=newvalues;
        end
        
        %determine which graphs need updating:
        if ~isempty(objectives)
            for o=1:numel(othergraphs);
                udat=get(handles.fakeax(othergraphs(o)),'userdata');
                allvars=unique([udat.view.x(:);udat.view.y(:);udat.view.z(:);udat.view.c(:);udat.view.s(:)]);
                if any(ismember(allvars,objectives))
                    needsupdating(o)=true;
                end
            end
        end
        
       
        
            
    case {'replaceall','addrows','removerows','datachanged'}
        %handles.maintableupdatespending=union(handles.maintableupdatespending,'model');
        handles.tablemodelupdatespending{d}=union(handles.tablemodelupdatespending{d},'data');
        
        
        needsupdating=false(size(othergraphs));
        for o=1:numel(othergraphs);
            thistype=get(handles.allgraphs(othergraphs(o)),'userdata');
            if ismember(updatetype,handles.graphinfo.(thistype).updates);
                needsupdating(o)=true;
            end
        end 
end

guidata(src,handles);


%update table model:


switch updatetype
    case 'addcolumns'
        a=1;
        [handles.maintable.models{d} junk]=ravecreatemaintablemodel(handles.fig,[],d,a);
        guidata(handles.fig,handles);
end

if handles.tablemode
    ravedisplaymaintable(handles.fig,[]);
end





%find indices of other graphs with this dataset and analysis:
othergraphs=othergraphs(needsupdating);

%


if isempty(othergraphs);return;end

originalindex=handles.activeindex;


for ai=othergraphs
    handles.activeindex=ai;
    
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    thistype=get(othergraph,'userdata'); %type of other graph in this analysis
    
    handles.activetype=thistype;
    handles.activegraph=othergraph;
    guidata(handles.fig,handles);
     
    
    if ~isempty(which(['raveupdate',thistype,updatetype]))
        eval(['raveupdate',thistype,updatetype,'(handles.fig,[]);'])
    else
        %brute force method:
        eval(['ravecreate',thistype,'(handles.fig,[]);'])
    end

    handles=guidata(handles.fig); %refresh handles;
    
end

othergraph=handles.allgraphs(originalindex);
thistype=get(othergraph,'userdata'); %type of other graph in this analysis

handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=othergraph;
guidata(handles.fig,handles);



