function raveupdategraphlist(src,ev)
%updates the graph list of a currently running rave session.
%src MUST be the handle of the rave figure to be updated.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%% load data for all graphtypes:
raveroot=which('rave');
raveroot=raveroot(1:end-6);
handles.graphinfo=struct;
graphclasses={'discrete';'continuous';'special';'table';'control';'annotation'};
for G=1:numel(graphclasses)

    maindir=fullfile(raveroot,'graphics\',[graphclasses{G} '\']);
    graphtypes=dir(maindir);

    for g=3:numel(graphtypes)
       % h=g-2;
        if graphtypes(g).isdir && graphtypes(g).name(1)~='X'
            thisname=graphtypes(g).name;
            %remove leading numbers:
            firstletter=find(isstrprop(thisname,'alpha'),1);
            keyword=thisname(firstletter:end);
            %get graphinfo:
            fid=fopen(fullfile(maindir,thisname,'graphinfo.txt'),'r');
              
            %get properties for this graph type:
            properties=struct;
            try
            eval(['graphproperties',keyword]);
            catch %#ok<CTCH>
               addpath([maindir,thisname],'-begin');
               rehash toolboxcache
               try
                   eval(['graphproperties',keyword]);
               catch %#ok<CTCH>
                   error(['rave:filenotfound','The graphproperties.txt file could not be found for graph type ' ...
                       keyword '. Either create this file or remove the ' keyword ' folder from Rave''s graphics directory']);
               end
                
            end
            
            handles.graphinfo.(keyword)=properties;
            handles.graphinfo.(keyword).longname=fgetl(fid);
            handles.graphinfo.(keyword).class=graphclasses{G};
            
            fclose(fid);
        end
    end
end

guidata(src,handles);


