function exploretab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);
handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
end
handles.fronttabsfront=1;

set(handles.infobar,'string','');

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,4,1);  %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
delete(handles.tabobjects)

%%  get methods:

raveroot=which('rave');
raveroot=raveroot(1:end-6);

filez=dir(fullfile(raveroot,'exploremethods\'));

h=1;
for g=3:numel(filez)

    thisfile=filez(g).name;
    if strcmp(thisfile(end-1:end),'.m')
        fid=fopen(fullfile(raveroot,'exploremethods\',thisfile),'r');
        firstline=fgetl(fid);
        secondline=fgetl(fid);
        fclose(fid);
        if ~isempty(secondline) && strcmp(secondline(1),'%')
            %this is an explore method. Store it.
            methodnames{h}=secondline(2:end);
            methodfunctions{h}=thisfile(1:end-2);
            h=h+1;
        end
    end
end


%check for graphs:

if isempty(handles.activetype)
    handles.tabobjects=[];
    guidata(handles.fig,handles);
    return
end



%compare to allowable methods for selected graph type:
if isfield(handles.graphinfo.(handles.activetype),'exploremethods')
    allowedmethods=handles.graphinfo.(handles.activetype).exploremethods;
    
    [finalmethods,ia,ib]=intersect(lower(allowedmethods),lower(methodfunctions));
    
else
   ib=[];
    
end




%% text headers
methodtext=text(.15-.10317,.95*1.0317-.0317,'Method','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% pulldown menu
methodmenu=uicontrol('position',[origin(1)+40 origin(2)+550 180 30],...
    'style','popupmenu','string',[cellstr('None Selected') methodnames(ib)],...
    'callback',@methodmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','methodmenu','userdata',[cellstr('') methodfunctions(ib)]);

%% finish up

handles.tabobjects=[methodtext methodmenu];
guidata(src,handles);

%% if a method is selected, load that method:
% udata=get(handles.fakeax(handles.activeindex),'userdata');
% %
% % selectedoption=udata.explore.option;
% % % if ~isempty(selectedoption)
% % set(methodmenu,'value',selectedoption)
% methodmenu_callback(methodmenu,[]);


function methodmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

%origin=get(handles.tabaxes,'position');
methods=get(src,'string');
selected=methods{get(src,'value')};

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.explore.option=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%% get input info from graph:
%udata=get(handles.fakeax(handles.activeindex),'userdata');
%view=udata.view; %get view tab structure.

%delete method specific uicontrols:
if length(handles.tabobjects)>2 %only two method-indepenedent uicontrols
    delete(handles.tabobjects(3:end));
    handles.tabobjects=handles.tabobjects(1:2);
    guidata(src,handles);
end

% %delete java slider if present.
% if ishandle(handles.javacomps)
%     delete(handles.javacomps)
%     handles.javacomps=[];
% end

%remove any callbacks associated with figure:
set(handles.activegraph,'hittest','on','buttondownfcn','')

%turn original data visibility on in case it was turned off (some methods
%hide original data):
maindata=findobj(get(handles.allgraphs(handles.activeindex),'children'),'type','hggroup','tag','main');
set(maindata,'visible','on')

%delete any existing legends: (caution: also removes any legends added from view tab)
legend(handles.activegraph,'off');


%do selected method:

methodfunctions=get(src,'userdata');
thismethod=get(src,'value');
thisfunction=methodfunctions{thismethod};

if thismethod>1 %1 = none selected
    if ~isempty(which([thisfunction,handles.activetype]))
        eval([thisfunction,handles.activetype,'(src,[]);']);
    else %run the generic flavor of this function:
        eval([thisfunction,'(src,[]);']);
    end
end

%DO NOT save guidata here. (or if so, reload it first).

