function modeltab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==1 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,0); %#ok<NASGU>
    set(handles.fronttablines,'xdata',x,'ydata',y)
end
handles.fronttabsfront=0;

%delete(handles.tabobjects);
set(handles.infobar,'string','');

%raise this tab to top
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,1); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

udata=get(handles.fakeax(handles.activeindex),'userdata');
if ~isempty(udata)
    d=udata.view.dataset;
    a=udata.view.analysis;
else
    d=1;
    a=1;
end

%% text headers
txttext=text(.15-.10317,.95*1.0317-.0317,'txt','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
mattext=text(.15-.10317,.79*1.0317-.0317,'matlab','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
createtext=text(.15-.10317,.63*1.0317-.0317,'Create','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
flagtext=text(.15-.10317,.37*1.0317-.0317,'Add ''Flag'' Column','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% uicontrols

%buttons 
importbutton=uicontrol('position',[origin(1)+55 origin(2)+550 170 30],'style','pushbutton',...
    'string','Import .txt Function','callback',@importTXT_callback,'backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12);

loadfolderbutton=uicontrol('position',[origin(1)+55 origin(2)+510 170 30],'style','pushbutton',...
    'string','Load Folder of Functions','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'callback',@raveloadfunfolder);

importmbutton=uicontrol('position',[origin(1)+55 origin(2)+450 170 30],'style','pushbutton',...
    'string','Import MATLAB Function','callback',@importMAT_callback,'backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12);

loadmfolderbutton=uicontrol('position',[origin(1)+55 origin(2)+410 170 30],'style','pushbutton',...
    'string','Load Folder of Functions','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'callback',@raveloadmfunfolder,'enable','off','visible','off');

createobjectivebutton=uicontrol('position',[origin(1)+55 origin(2)+345 170 30],'style','pushbutton',...
    'string','Create New Objective Fcn','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'enable','on','callback',@ravecreateobjectivegui);

createmodelbutton=uicontrol('position',[origin(1)+55 origin(2)+305 170 30],'style','pushbutton',...
    'string','Create New Metamodel','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'enable','on','callback',@ravecreatemodelgui,'enable','off');
selectmodelbutton=uicontrol('position',[origin(1)+55 origin(2)+265 170 30],'style','pushbutton',...
    'string','Select a Metamodel','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'enable','off');

analysistext=text(.25-.10317,.340*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.27*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+160 180 30],...
    'style','popupmenu','string','Default','callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',a);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+115 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',d);

makeselectionbutton=uicontrol('position',[origin(1)+30 origin(2)+70 210 30],'style','pushbutton',...
    'string','Make Variable of Row Selection','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'enable','on','callback',@makeselectioncolumn);
makecolorsbutton=uicontrol('position',[origin(1)+30 origin(2)+30 210 30],'style','pushbutton',...
    'string','Make Variable of Row Colors','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial narrow','fontsize',12,'enable','on','callback',@makerowcolorcolumn);

%% 
handles.tabobjects=[createtext importbutton createmodelbutton selectmodelbutton loadfolderbutton txttext mattext importmbutton loadmfolderbutton...
    flagtext analysistext datasettext analysismenu datasetmenu makeselectionbutton makecolorsbutton createobjectivebutton];

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% callbacks

function importTXT_callback(src,eventdata)

handles=guidata(src);

h=ravetxt2mgui(handles.fig,[]);
uiwait(h);


%raveupdatedataset(handles.fig,[],'addcolumns',d);



function importMAT_callback(src,ev) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handles=guidata(src);
set(0,'userdata',handles);

h=ravem2mgui(src,ev);
uiwait(h);

%get data from pop up gui:

indata=get(0,'userdata');

cancelled=indata.cancelled;
if cancelled
    return
end

responsename=indata.responsename';
path=indata.path;
d=indata.dataset;
%analysis=indata.analysis;
varnames=indata.varnames;
%numvars=indata.nargout;


%remove spaces from varnames, cause that's how ravejmp2m works
for j=1:length(varnames)
    varnames{j}=varnames{j}(varnames{j}~=char(32));
end

%find which column of data set holds each input variable:
existingnames=strrep(handles.allnames{d},char(32),'');

nvars=numel(varnames);

varorder=zeros(1,nvars);
for i=1:nvars
    if ~isempty(find(strcmp(existingnames,varnames(i)),1))
        varorder(i)=find(strcmp(existingnames,varnames(i)),1);    
    else
        varorder(i)=0; %a non-dataset variable
    end
end


%get file name from path:
indexx=find(path=='\',1,'last');
filename=path(indexx+1:end);

functionname=filename(1:end-2);

if isempty(which(functionname))
   warndlg('The function you selected is not on your MATLAB search path. You must move it there before you can use it in RAVE.','You can''t do that');
    return
end


funhandle=str2func(functionname);
allvalues=cell(1,nargout(funhandle));
prefs=handles.preferences{d};
targets=handles.targetvalues{d};
if nargin(funhandle)==1
    [allvalues{:}]=funhandle(handles.alldata{d});
else %should have 3 inputs
    [allvalues{:}]=funhandle(handles.alldata{d},prefs,targets);   
end


%update fields:
for jjj=1:numel(indata.nargout) %for each variable being modelled
    newvalues=allvalues{indata.nargout(jjj)}(:,1); %just take first column, this protects against array outputs, which rave cant handle.
    
    handles.alldata{d}(:,end+1)=newvalues;
    handles.isvalid{d}(:,end+1)=true;
    handles.alltext{d}=[handles.alltext{d},cell(length(newvalues),1)];
    handles.allnames{d}{end+1}=responsename{indata.nargout(jjj)};
    handles.variablecolors{d}(end+1,:)=rand(1,3);
    
    handles.isresponse{d}(end+1)=1;
    handles.allunits{d}{end+1}=''; %havent implemented this yet
    handles.isincluded{d}(end+1)=1;
    handles.isdecided{d}(end+1)=0;
    %handles.istext{d}(end+1)=0;
    handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
    handles.isfunction{d}(end+1)=1;
    handles.funclass{d}{end+1}='matlab_generic';
    handles.funnames{d}{end+1}=functionname;
    handles.funvars{d}{end+1}=sort(varorder);
    handles.nargout{d}(end+1)=indata.nargout(jjj);
    handles.datatype{d}{end+1}='continuous';
    handles.preferences{d}(end+1)=1;
    handles.targetvalues{d}(end+1)=-inf;
    handles.isobjective{d}(end+1)=false;
    handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
    for a=1:numel(handles.analysisnames)
        handles.xdials{d}{a}(end+1)=0;
        handles.minlimit{d}{a}(end+1)=min(newvalues);
        handles.maxlimit{d}{a}(end+1)=max(newvalues);
        handles.discretevalues{d}{a}{end+1}=[];
    end
  
end

guidata(src,handles);
set(handles.infobar,'string','Function loaded successfully.');

%update maintable udata to display new columns:
handles.maintable.x{d}=union(handles.maintable.x{d},(size(handles.alldata{d},2)-numel(indata.nargout)):size(handles.alldata{d},2));
raveupdatedataset(handles.fig,[],'addcolumns',d);
ravefinish; %make a done sound.




function raveloadfunfolder(src,eventdata)%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Lets user pick a
%directory and then tries to load all functions in that folder as new
%variables. 
handles=guidata(src);

path=uigetdir(handles.defaultdir,'Choose a directory. All .txt files in this directory will be loaded as new functions.');

if path==0;return;end

files=dir(path);
filenames=struct2cell(files);
if size(filenames,2)==2
    %folder is empty
    set(handles.infobar,'string','The folder you chose is empty.')
    ravefinish
    return
end

%filenames=lower(filenames(1,3:end));
%filenames=filenames(1,3:end);
filenames=filenames(1,:);


istxt=false(1,numel(filenames));
for g=1:numel(filenames)
    if filenames{g}(1)=='.'
        continue
    end
    if strcmpi(filenames{g}(end-2:end),'txt')
        istxt(g)=true;
    end
end
filenames=filenames(istxt);

if isempty(filenames)
    set(handles.infobar,'string','The folder you chose contains no txt files.')
    ravefinish
    return
end

%if more than one data set, ask which one to put this function in:
if numel(handles.alldata)>1
    screensize=get(0,'screensize');

    askbox=figure('units','pixels','position',[screensize(3)/2-150 screensize(4)/2-75 300 150],...
        'numbertitle','off','menubar','none','name','Select a Data Set','color',handles.colors.frame,...
        'resize','off','windowstyle','modal');
    uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
        'position',[30 100 250 40],...
        'string','Select the data set that contains the input variables for these equations.',...
        'horizontalalignment','left')
    uicontrol('style','popupmenu','string',handles.datasetnames,'backgroundcolor',[1 1 1],'units','pixels',...
        'position',[60 70 180 22],'tag','datasetmenu')
    uicontrol('style','pushbutton','string','Ok','parent',askbox,'units','pixels',...
        'position',[100 10 100 30],'callback',@datasetpopup_callback,'fontsize',10);
    uiwait(askbox)

    value=get(0,'userdata');
    if isempty(value)
        return %user hit cancel
    end

    dataset=value;
else
    dataset=1;
end


load raveprefs

%make please wait notice:
pos=get(handles.fig,'position');
[j,h]=javacomponent('javax.swing.JTextPane',[pos(3)/2-150 pos(4)/2-25 300 50],handles.fig);
j.setBackground(java.awt.Color(1,1,.8));
j.setBorder(javax.swing.border.LineBorder.createBlackLineBorder);
j.setFont(java.awt.Font(raveprefs.fonts.tab,1,24*raveprefs.fonts.tabscale)); %1=bold 
j.setText('Loading, Please Wait...');
drawnow;



for f=1:numel(filenames)
    %read this file:
    fid=fopen(fullfile(path,filenames{f}),'r');
    linein='';
    while 1
        tline=fgetl(fid);

        if ~ischar(tline),   break,   end %stop at end of file)
        linein = [linein tline]; %#ok<AGROW>
    end
    fclose(fid);

    set(handles.infobar,'string','Loading functions, please wait...')

    varname=filenames{f}(1:end-4);

    %read in text file
    try
        ravetxt2m(src,[],varname,linein,dataset); 
    catch
       delete(h)
       set(handles.infobar,'string','Error while loading folder.')
       errordlg('Could not load specified folder. If this is a new install, check to see that the ravefiles directory is on your path.','Could not load folder');
       return       
    end
    
    
end
set(handles.infobar,'string','Folder loaded successfully.');


%update maintable udata to display new columns:
handles.maintable.x{dataset}=union(handles.maintable.x{dataset},(size(handles.alldata{dataset},2)-numel(filenames)):size(handles.alldata{dataset},2));
raveupdatedataset(handles.fig,[],'addcolumns',dataset);
ravefinish; %make a done sound.
%NOTE: DO NOT SAVE GUIDATA AT END OF THIS FUNCTION. RAVETXT2M DOES IT FOR
%YOU.

%delete please wait notice.
delete(h);

function raveloadmfunfolder(src,eventdata)

function datasetpopup_callback(src,eventdata)
damenu=findobj(get(src,'parent'),'tag','datasetmenu');

set(0,'userdata',[]);
if strcmp(get(src,'string'),'Ok')
set(0,'userdata',get(damenu,'value'));
end

close(get(gcbo,'parent'))


function makeselectioncolumn(src,ev)
handles=guidata(src);

%make askbox:
screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-150 screensize(2)+screensize(4)/2-75 300 150],...
    'numbertitle','off','menubar','none','name','Name this variable','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[30 100 250 40],...
    'string','Enter a name for this variable. The value of this variable will be 0 or 1 depending on which rows are currently selected.',...
    'horizontalalignment','left')
u=uicontrol('style','edit','string','','backgroundcolor',[1 1 1],'units','pixels',...
    'position',[60 70 180 22],'tag','variablename','hor','left');
uicontrol('style','pushbutton','string','Ok','parent',askbox,'units','pixels',...
    'position',[120 10 100 30],'callback',@makenewcol_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[20 10 80 30],'callback',@makenewcol_callback,'fontsize',10);

uicontrol(u);
uiwait(askbox)

value=get(0,'userdata');
if isempty(value)
    return %user hit cancel
end

newname=value;

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

%newname=inputdlg('Name this variable:','Name it.');
newdata=double(handles.isselected{d}{a});

handles.alldata{d}(:,end+1)=newdata;
handles.isvalid{d}(:,end+1)=true;
handles.alltext{d}=[handles.alltext{d},cell(length(newdata),1)];
handles.allnames{d}{end+1}=newname;
handles.variablecolors{d}(end+1,:)=rand(1,3);
handles.isresponse{d}(end+1)=1;
handles.allunits{d}{end+1}=''; %havent implemented this yet
handles.isincluded{d}(end+1)=1;
handles.isdecided{d}(end+1)=0;
handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
handles.isfunction{d}(end+1)=0;
handles.funclass{d}{end+1}='independent';
handles.funnames{d}{end+1}='';
handles.funvars{d}{end+1}=[];
handles.nargout{d}(end+1)=0;
handles.datatype{d}{end+1}='logical';
handles.preferences{d}(end+1)=1;
handles.targetvalues{d}(end+1)=-inf;
handles.isobjective{d}(end+1)=false;
handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;

for a=1:numel(handles.analysisnames)
    
    handles.xdials{d}{a}(end+1)=0;
    handles.minlimit{d}{a}(end+1)=0;
    handles.maxlimit{d}{a}(end+1)=1;
    handles.discretevalues{d}{a}{end+1}=[0 1];
end

guidata(src,handles);

%update maintable udata to display new columns:
handles.maintable.x{dataset}=union(handles.maintable.x{dataset},(size(handles.alldata{dataset},2)-numel(filenames)):size(handles.alldata{dataset},2));
raveupdatedataset(handles.fig,[],'addcolumns',d);
ravefinish; %make a done sound.



set(handles.infobar,'string','New column successfully added to data set.')

function makerowcolorcolumn(src,ev)
handles=guidata(src);

%make askbox:
screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-150 screensize(2)+screensize(4)/2-165 300 330],...
    'numbertitle','off','menubar','none','name','Name this variable','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[30 400-120 250 40],'string','Enter a name for this variable:','horizontalalignment','left')
u=uicontrol('style','edit','string','','backgroundcolor',[1 1 1],'units','pixels',...
    'position',[60 400-120 180 22],'tag','variablename','hor','left');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[30 350-120 250 40],'string','Choose entry format:','horizontalalignment','left')
uicontrol('style','popupmenu','string',{'Number';'Simple Colors';'Detail Colors'},'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[60 350-120 180 22],'tag','entryformat','hor','left','callback',{@entryformat_callback,handles.fig});
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[30 300-120 250 40],'string','Preview:','horizontalalignment','left')
uicontrol('style','listbox','string',{'1';'2';'3';'4';'5';'6';'7';'8';'9';'10'},'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[60 55 180 144],'tag','previewlist','hor','left');


uicontrol('style','pushbutton','string','Ok','parent',askbox,'units','pixels',...
    'position',[200 10 80 30],'callback',@makenewcolorcol_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[100 10 80 30],'callback',@makenewcolorcol_callback,'fontsize',10);


uicontrol(u);
uiwait(askbox)

userdata=get(0,'userdata');
if isempty(userdata)
    return %user hit cancel
end


newname=userdata.varname;
entryformat=userdata.entryformat;
values=userdata.values;

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

%newname=inputdlg('Name this variable:','Name it.');
if entryformat==1 %numbers
    newdata=double(handles.rowcolors{d}{a});
    handles.alldata{d}(:,end+1)=newdata;
    handles.isvalid{d}(:,end+1)=true;
    handles.alltext{d}=[handles.alltext{d},cell(length(newdata),1)];
    handles.allnames{d}{end+1}=newname;
    handles.variablecolors{d}(end+1,:)=rand(1,3);
    handles.isresponse{d}(end+1)=1;
    handles.allunits{d}{end+1}=''; %havent implemented this yet
    handles.isincluded{d}(end+1)=1;
    handles.isdecided{d}(end+1)=0;
    %handles.istext{d}(end+1)=0;
    handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
    handles.isfunction{d}(end+1)=0;
    handles.funclass{d}{end+1}='independent';
    handles.funnames{d}{end+1}='';
    handles.funvars{d}{end+1}=[];
    handles.nargout{d}(end+1)=0;
    handles.datatype{d}{end+1}='discrete';
    handles.preferences{d}(end+1)=1;
    handles.targetvalues{d}(end+1)=-inf;
    handles.isobjective{d}(end+1)=false;
     handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
    
    for a=1:numel(handles.analysisnames)
        
        handles.xdials{d}{a}(end+1)=min(newdata);
        handles.minlimit{d}{a}(end+1)=min(newdata);
        handles.maxlimit{d}{a}(end+1)=max(newdata);
        handles.discretevalues{d}{a}{end+1}=unique(newdata);
        
    end
else %text
    newdata=double(handles.rowcolors{d}{a});
    handles.alldata{d}(:,end+1)=newdata;   
    handles.isvalid{d}(:,end+1)=true;
    textssss=cell(length(newdata),1);
    [textssss{:}]=values{newdata};
    handles.alltext{d}=[handles.alltext{d},textssss];
    handles.allnames{d}{end+1}=newname;
    handles.variablecolors{d}(end+1,:)=rand(1,3);
    handles.isresponse{d}(end+1)=1;
    handles.allunits{d}{end+1}=''; %havent implemented this yet
    handles.isincluded{d}(end+1)=1;
    handles.isdecided{d}(end+1)=0;
    %handles.istext{d}(end+1)=0;
    handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
    handles.isfunction{d}(end+1)=0;
    handles.funclass{d}{end+1}='independent';
    handles.funnames{d}{end+1}='';
    handles.funvars{d}{end+1}=[];
    handles.nargout{d}(end+1)=0;
    handles.datatype{d}{end+1}='string'; 
    handles.preferences{d}(end+1)=0;
    handles.targetvalues{d}(end+1)=-inf;
    handles.isobjective{d}(end+1)=false;
    handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
    
    for a=1:numel(handles.analysisnames)
        
        handles.xdials{d}{a}(end+1)=min(newdata);
        handles.minlimit{d}{a}(end+1)=min(newdata);
        handles.maxlimit{d}{a}(end+1)=max(newdata);
        handles.discretevalues{d}{a}{end+1}=values(unique(newdata));
        
    end
end


guidata(src,handles);



%update maintable udata to display new columns:
handles.maintable.x{dataset}=union(handles.maintable.x{dataset},(size(handles.alldata{dataset},2)-numel(filenames)):size(handles.alldata{dataset},2));

raveupdatedataset(handles.fig,[],'addcolumns',d);
ravefinish; %make a done sound.






function makenewcol_callback(src,ev)
varedit=findobj(get(src,'parent'),'tag','variablename');
set(0,'userdata',[]);
if strcmp(get(src,'string'),'Ok')
set(0,'userdata',get(varedit,'string'));
end
close(get(gcbo,'parent'))



function makenewcolorcol_callback(src,ev)
varedit=findobj(get(src,'parent'),'tag','variablename');
entryformat=findobj(get(src,'parent'),'tag','entryformat');
previewlist=findobj(get(src,'parent'),'tag','previewlist');


set(0,'userdata',[]);
if strcmp(get(src,'string'),'Ok')
    udata.varname=get(varedit,'string');
    udata.entryformat=get(entryformat,'value');
    udata.values=get(previewlist,'string');
    set(0,'userdata',udata);
end
close(get(gcbo,'parent'))


function entryformat_callback(src,ev,hfig)
handles=guidata(hfig);
thisfig=get(src,'parent');


previewlist=findobj(thisfig,'tag','previewlist');

if get(src,'value')==1 %numbers
    set(previewlist,'string',{'1';'2';'3';'4';'5';'6';'7';'8';'9';'10'});
    
elseif get(src,'value')==2 %simple color names
    themap=get(hfig,'colormap');
    deezcolors=themap(handles.prefs.rowcolors,:);
    set(previewlist,'string',rgb2name(deezcolors,'simple'));
else %long color names
    themap=get(hfig,'colormap');
    deezcolors=themap(handles.prefs.rowcolors,:);
    set(previewlist,'string',rgb2name(deezcolors));
end
